var loading_object;
var error_object;
var timeout;
var scriptsLoaded = [];
var loaded_scripts = [];
var log_events = 0;

$.ajaxSetup({
    beforeSend: function (xhr, settings) {
        if (settings.type !== 'GET' || settings.dataType !== 'script')
            return;
        var re = new RegExp('^' + site_url);
        if (!re.test(settings.url))
            return;
        var url = settings.url.replace(/\?_=[\d]+/g, '');
        if ($.inArray(url, scriptsLoaded) !== -1) {
            xhr.abort();
            return;
        }
        scriptsLoaded.push(url);
    }
});

function showLoginForm(redirectAfterLogin) {
    var user_ajax_login = new loadingContent({
        loadBlockWidth: '520px',
        closeBtnClass: 'load_content_close btn btn-edge',
        closeBtnImage: 'pg pg-close-edge pg-lg w'
    });
    $.ajax({
        url: site_url + 'users/ajax_login_form',
        cache: false,
        method: 'post',
        data: {'redirect_after_login': redirectAfterLogin},
        success: function (data) {
            user_ajax_login.show_load_block(data);
        }
    });
};

function checkAccess(module, action, onAllowed, onNotAllowed, onError) {
    $.ajax({
        url: site_url + 'start/acl_check',
        type: 'post',
        cache: true,
        dataType: 'json',
        data: ({
            module: module,
            action: action
        }),
        success: function (response) {
            if (response.is_allowed) {
                onAllowed();
            } else {
                onNotAllowed();
            }
        },
        error: function (response) {
            console.error(response);
            if ('function' === typeof onError) {
                onError(response);
            }
        }
    });
}

function preCheckAccess(module, action) {
    var url = site_url + module + '/' + action;
    checkAccess(module, action,
        function () {
            document.location.href = url;
        },
        function () {
            showLoginForm(url);
        }
    );
    return false;
}


$(function () {
    loading_object = new Loading;
    $(document).ajaxStart(function () {
        loading_object.setLoading();
    }).ajaxStop(function () {
        loading_object.unsetLoading();
    }).ajaxError(function (event, jqxhr, settings, exception) {
        switch (jqxhr.status) {
            case 200:
                console.error('ajaxStart() error');
                break;
            case 403:
                showLoginForm();
                break;
            case 404:
                if (site_url.indexOf(window.location.hostname) == -1 ||
                        site_url.indexOf('www.' + window.location.hostname) != -1) {
                    window.location.href = site_url;
                } else {
                    window.location.href = site_url + 'start/error';
                }
                break;
            case 500:
                var errorObj = new Errors();
                errorObj.show_error_block(jqxhr.statusText, 'error');
                break;
            default:
                var errorObj = new Errors();
                errorObj.show_error_block(jqxhr.responseText, 'error');
                break;
        }
    });

    error_object = new Errors({position: site_error_position});
    timeout = 0;

    if ($('#error_block').html().trim()) {
        error_object.show_error_block($('#error_block').html(), 'error');
        timeout = 2000;
    }
    if ($('#info_block').html().trim()) {
        if (timeout) {
            setTimeout(function () {
                error_object.show_error_block($('#info_block').html(), 'info');
            }, timeout);
        } else {
            error_object.show_error_block($('#info_block').html(), 'info');
        }
    }
    if ($('#success_block').html().trim()) {
        if (timeout) {
            setTimeout(function () {
                error_object.show_error_block($('#success_block').html(), 'success');
            }, timeout);
        } else {
            error_object.show_error_block($('#success_block').html(), 'success');
        }
    }
});

function loadScripts(url, callback, obj_for_kill, ajaxOptions) {
    obj_for_kill = obj_for_kill || '';
    ajaxOptions = ajaxOptions || {};
    var script_url = '', cb;

    if (typeof url === 'object' && url.length) {
        script_url = url.shift();
        cb = url.length ? function () {
            loadScripts(url, callback, obj_for_kill, ajaxOptions);
        } : callback;
    } else if (typeof url === 'string') {
        script_url = url;
        cb = callback;
    }

    if (typeof obj_for_kill === 'object' && obj_for_kill.length) {
        for (var i in obj_for_kill) {
            tmp_objects[obj_for_kill[i]] = script_url;
        }
    } else if (obj_for_kill) {
        tmp_objects[obj_for_kill] = script_url;
    }

    if (script_url) {
        var scriptname = script_url.match(/[^\/?#]+(?=$|[?#])/)[0];
        if (typeof scriptname !== 'undefined') {
            var ext = scriptname.lastIndexOf('.');
            if (ext) {
                scriptname = scriptname.substr(0, ext);
            }
            if (scriptname.substr(-4) === '.min') {
                scriptname = scriptname.substr(0, scriptname.length - 4);
            }
        } else {
            var scriptname = '';
        }
        var event = scriptname ? 'scriptLoad:' + scriptname : 'scriptLoad';

        for (var i in loaded_scripts) {
            if (script_url === loaded_scripts[i]) {
                cb();
                $(document).trigger(event);
                if (log_events)
                    log(event);
                return;
            }
        }

        var options = {
            url: script_url,
            success: function () {
                loaded_scripts.push(script_url);
                cb();
                $(document).trigger(event);
                if (log_events)
                    log(event);
            },
            error: function (xhr, textStatus, errorThrown) {
                log('error loading script: ' + script_url + '. ' + textStatus + '. ' + errorThrown, 'error');
            },
            dataType: 'script',
            cache: true
        };

        $.extend(true, options, ajaxOptions);
        $.ajax(options);
    } else if (typeof console !== 'undefined') {
        console.warn('Error. Load script: invalid url');
    }
}

function sendAnalytics(event, category, type) {
    if (!event) {
        return;
    }

    if (typeof sendAnalyticsF === 'function') {
        sendAnalyticsF(event, category, type);
    }
}
