<?php

/**
 * Contact module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Conact management
 *
 * @package 	PG_RealEstate
 * @subpackage 	Contact
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists("show_contact_form")) {
    /**
     * Show contact form
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function show_contact_form($params)
    {
        $ci = &get_instance();

        $auth_type = $ci->session->userdata("auth_type");
        if ($auth_type == 'user' && $ci->session->userdata("user_id") > 0) {
            return '';
        }

        $ci->load->model("Contact_model");
        $ci->load->plugin("captcha");
        $ci->config->load("captcha_settings");
        $captcha_settings = $ci->config->item("captcha_settings");
        $captcha = create_captcha($captcha_settings);
        $ci->session->set_userdata("captcha_word", $captcha["word"]);
        $data["captcha_image"] = $captcha["image"];
        $data["captcha_word_length"] = strlen($captcha["word"]);

        $data["user_id"] = $params['user_id'];
        if(isset($params['header'])) {
            $data["header"] = $params['header'];
        }
        $ci->view->assign("data", $data);

        $message = isset($params['message']) ? $params['message'] : '';
        $ci->view->assign("message", $message);

        $ci->load->helper("start");
        $ci->view->assign('phone_format', get_phone_format());

        $ci->view->assign('rand', rand(100000, 999999));

        // show template from contact module default user theme
        $html = $ci->view->fetch("helper_contact_form", "user", "contact");

        return $html;
    }
}

if (!function_exists("admin_home_contact_block")) {
    /**
     * Info of contact module for admin homepage
     *
     * @return string
     */
    function admin_home_contact_block()
    {
        $ci = &get_instance();

        $auth_type = $ci->session->userdata("auth_type");
        if ($auth_type != "admin") {
            return "";
        }

        $user_type = $ci->session->userdata("user_type");

        $show = true;

        $stat_contact = array(
            "index_method" => true,
        );

        if ($user_type == "moderator") {
            $show = false;
            $ci->load->model("Moderators_model");
            $methods_contact = $ci->Moderators_model->get_module_methods("contact");
            if ((is_array($methods_contact) && !in_array("index", $methods_contact))) {
                $show = true;
            } else {
                $permission_data = $ci->session->userdata("permission_data");
                if ((isset($permission_data["contact"]["index"]) && $permission_data["contact"]["index"] == 1)) {
                    $show = true;
                    $stat_contact["index_method"] = (bool) $permission_data["contact"]["index"];
                }
            }
        }

        if (!$show) {
            return "";
        }

        $ci->load->model("Contact_model");

        $stat_contact['data']["contact_make_copies"] = $ci->pg_module->get_module_config("contact", "contact_make_copies");

        if ($stat_contact['data']["contact_make_copies"]) {
            $stat_contact['data']["contact_need_approve"] = $ci->pg_module->get_module_config("contact", "contact_need_approve");

            $stat_contact['data']["all"] = $ci->Contact_model->get_all_contacts_count();
            $stat_contact['data']["unread"] = $ci->Contact_model->get_unread_contacts_count();

            if ($stat_contact['data']["contact_need_approve"]) {
                $stat_contact['data']["need_approve"] = $ci->Contact_model->get_need_approve_contacts_count();
            }
        }
        $ci->view->assign("stat_contact", $stat_contact);

        // show template from contact module default user theme
        $html = $ci->view->fetch("helper_home_contact_block", "admin", "contact");

        return $html;
    }
}
