<?php

namespace Pg\Modules\Countries\Controllers;

/**
 * Countries module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Countries user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Countries
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 * */
class Countries extends \Controller
{

    /**
     * Constructor
     *
     * @return Countries
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Countries_model");
    }

    /**
     * Get locations by ajax
     *
     * @param string $name location filter
     *
     * @return void
     */
    public function ajax_get_locations($name = '')
    {
        $return = array();
        
        if ($name) {
            $locations = $this->Countries_model->get_locations($name, array("priority" => "ASC"), $this->pg_language->current_lang_id, $this->pg_language->languages);
            $return["all"] = count($locations['countries']) + count($locations['regions']) + count($locations['cities']) + count($locations['districts']);
            $return["items"] = $locations ? $locations : array();
        } else {
            $return["all"] = 0;
            $return["items"] = array();
        }
        
        $this->view->assign($return);

        $this->view->render();

        return;
    }

    /**
     * Get countries by ajax
     *
     * @return void
     */
    public function ajax_get_countries()
    {
        $return = array();

        $countries = $this->Countries_model->get_countries(array("priority" => "ASC"), array(), array(), $this->pg_language->current_lang_id);
        $return["all"] = count($countries);
        if ($return["all"]) {
            foreach ($countries as $country) {
                $return["items"][] = array(
                    "id" => $country["id"],
                    "code" => $country["code"],
                    "name" => $country["name"],
                );
            }
        }
        
        $this->view->assign($return);

        $this->view->render();
        
        return;
    }

    /**
     * Get regions by ajax
     *
     * @param string $country_code country code
     *
     * @return void
     */
    public function ajax_get_regions($country_code)
    {
        $return = array();
        $return["country"] = $this->Countries_model->get_country($country_code, $this->pg_language->current_lang_id);
        $regions = $this->Countries_model->get_regions($country_code, array("priority" => "ASC"), array(), array(), $this->pg_language->current_lang_id);
        $return["all"] = count($regions);
        if ($return["all"]) {
            foreach ($regions as $region) {
                $return["items"][] = array(
                    "id" => $region["id"],
                    "code" => $region["code"],
                    "name" => $region["name"],
                );
            }
        }
        
        $this->view->assign($return);
        
        $this->view->render();

        return;
    }

    /**
     * Get cities by ajax
     *
     * @param integer $id_region region identifier
     * @param integer $page      page of results
     *
     * @return void
     */
    public function ajax_get_cities($id_region, $page = 1)
    {
        $search_string = trim(strip_tags($this->input->post('search', true)));
        $return = array();
        $items_on_page = 400;
        $return["region"] = $this->Countries_model->get_region($id_region, $this->pg_language->current_lang_id);
        $return["country"] = $this->Countries_model->get_country($return["region"]["country_code"], $this->pg_language->current_lang_id);

        $params["where"]["id_region"] = $id_region;
        if (!empty($search_string)) {
            $var = "name";
            if ($this->pg_language->current_lang_id) {
                $var = 'lang_' . $this->pg_language->current_lang_id;
            }
            $params["where"][$var . " LIKE"] = "%" . $search_string . "%";
        }
        $return["all"] = $this->Countries_model->get_cities_count($params);
        $return["pages"] = ceil($return["all"] / $items_on_page);
        $return["current_page"] = $page;
        $return["items"] = array_values($this->Countries_model->get_cities($page, $items_on_page, array("priority" => "ASC"), $params, array(), $this->pg_language->current_lang_id));

        $this->view->assign($return);
        
        $this->view->render();

        return;
    }

    /**
     * Get location form by ajax
     *
     * Location type: region, city, district
     *
     * @param string $type location type
     * @param string $var  location identifier/code
     *
     * @return void
     */
    public function ajax_get_form($type, $var = 0)
    {
        $data = array();

        switch ($type) {
            case "region":
                $data["country"] = $this->Countries_model->get_country($var, $this->pg_language->current_lang_id);
                break;
            case "city":
                $data["region"] = $this->Countries_model->get_region($var, $this->pg_language->current_lang_id);
                $data["country"] = $this->Countries_model->get_country($data["region"]["country_code"], $this->pg_language->current_lang_id);
                break;
            case "district":
                $data['city'] = $this->Countries_model->get_city($var, $this->pg_language->current_lang_id);
                $data['region'] = $this->Countries_model->get_region($data['city']['id_region'], $this->pg_language->current_lang_id);
                $data['country'] = $this->Countries_model->get_country($data['region']['country_code'], $this->pg_language->current_lang_id);
                break;
        }

        $this->view->assign("data", $data);
        $this->view->assign("type", $type);
        $this->view->render('ajax_country_form');
    }

    /**
     * Get location data by ajax
     *
     * Location type: region, city, district
     *
     * @param string $type location type
     * @param string $var  location identifier/code
     *
     * @return void
     */
    public function ajax_get_data($type, $var)
    {
        $data = array();

        switch ($type) {
            case "country":
                $data["country"] = $this->Countries_model->get_country($var, $this->pg_language->current_lang_id);
                break;
            case "region":
                $data["region"] = $this->Countries_model->get_region($var, $this->pg_language->current_lang_id);
                $data["country"] = $this->Countries_model->get_country($data["region"]["country_code"], $this->pg_language->current_lang_id);
                break;
            case "city":
                $data["city"] = $this->Countries_model->get_city($var, $this->pg_language->current_lang_id);
                $data["region"] = $this->Countries_model->get_region($data["city"]["id_region"], $this->pg_language->current_lang_id);
                $data["country"] = $this->Countries_model->get_country($data["city"]["country_code"], $this->pg_language->current_lang_id);
                break;
            case "district":
                $data["district"] = $this->Countries_model->get_district($var, $this->pg_language->current_lang_id);
                $data["city"] = $this->Countries_model->get_city($data['district']['id_city'], $this->pg_language->current_lang_id);
                $data["region"] = $this->Countries_model->get_region($data["city"]["id_region"], $this->pg_language->current_lang_id);
                $data["country"] = $this->Countries_model->get_country($data["city"]["country_code"], $this->pg_language->current_lang_id);
                break;
        }

        $this->view->assign($data);
        
        $this->view->render();

        return;
    }

    /**
     * Get districts of city as array
     *
     * @param integer $city_id city identifier
     * @param integer $page    page of results
     *
     * @return void
     */
    public function ajax_get_districts($city_id, $page = 1)
    {
        $search_string = trim(strip_tags($this->input->post('search', true)));
        $return = array();
        $items_on_page = 400;
        $return["city"] = $this->Countries_model->get_city($city_id, $this->pg_language->current_lang_id);
        $return["region"] = $this->Countries_model->get_region($return["city"]["id_region"], $this->pg_language->current_lang_id);
        $return["country"] = $this->Countries_model->get_country($return["city"]["country_code"], $this->pg_language->current_lang_id);

        $params["where"]["id_city"] = $city_id;
        if (!empty($search_string)) {
            $var = "name";
            if ($this->pg_language->current_lang_id) {
                $var = 'lang_' . $this->pg_language->current_lang_id;
            }
            $params["where"][$var . " LIKE"] = "%" . $search_string . "%";
        }
        $return["all"] = $this->Countries_model->get_districts_count($params);
        $return["pages"] = ceil($return["all"] / $items_on_page);
        $return["current_page"] = $page;
        $return["items"] = array_values($this->Countries_model->get_districts($page, $items_on_page, array("priority" => "ASC"), $params, array(), $this->pg_language->current_lang_id));

        $this->view->assign($return);
        
        $this->view->render();

        return;
    }

}
