<?php

namespace Pg\Modules\Currencies\Controllers;

use Pg\Libraries\View;

/**
 * Currencies module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Currencies admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Currencies
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Currencies extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Currencies
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'system-items');
    }

    public function index()
    {
        $this->settings();
    }

    /**
     * Manage curencies settings
     *
     * @return void
     */
    public function settings()
    {
        $this->load->model("Currencies_model");

        $currency = $this->Currencies_model->get_currency_list(null, null, array("gid" => "ASC"));
        $this->view->assign('currency', $currency);

        $updaters = $this->Currencies_model->rates_updaters;
        $this->view->assign("updaters", $updaters);

        $use_rates_update = $this->pg_module->get_module_config("currencies", "use_rates_update");
        $this->view->assign("use_rates_update", $use_rates_update);

        $rates_update_driver = $this->pg_module->get_module_config("currencies", "rates_update_driver");
        $this->view->assign("rates_update_driver", $rates_update_driver);

        $this->Menu_model->set_menu_active_item('admin_payments_menu', 'payments_currencies_menu_item');
        $this->view->setHeader(l('admin_header_currency_list', 'currencies'));
        $this->view->render('list_settings');
    }

    /**
     * Edit currency data
     *
     * @param integer $id currency identifier
     *
     * @return void
     */
    public function settings_edit($id = null)
    {
        $this->load->model("Currencies_model");
        if ($id) {
            $data = $this->Currencies_model->get_currency_by_id($id);
        } else {
            $data = array();
        }

        $base_currency = $this->pg_module->get_module_config("start", "base_currency");
        $this->view->assign("base_currency", $base_currency);

        if ($this->input->post('btn_save')) {
            $template = $this->input->post("template", true);
            $gr_sep = $this->input->post("gr_sep", true);
            $dec_sep = $this->input->post("dec_sep", true);
            $dec_part = $this->input->post("dec_part", true);
            // Add the number format
            $template = str_replace('value',
                                    'value' .
                    '|dec_part:' . $dec_part .
                    '|dec_sep:' . $dec_sep .
                    '|gr_sep:' . $gr_sep, $template);

            $post_data = array(
                "gid" => $this->input->post("gid", true),
                "abbr" => $this->input->post("abbr", true),
                "template" => $template,
                "name" => $this->input->post("name", true),
            );

            if ($id && $data['gid'] != $base_currency) {
                $post_data["per_base"] = $this->input->post("per_base", true);
            }

            $validate_data = $this->Currencies_model->validate_currency($id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $this->Currencies_model->save_currency($id, $validate_data["data"]);

                if ($id) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_currency', 'currencies'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_currency', 'currencies'));
                }

                $this->view->setRedirect(site_url() . "admin/currencies/settings");
            }
            $data = array_merge($data, $validate_data["data"]);
        }

        $matches = array();
        // Parse the number format
        preg_match('/value\|([^]]*)/', $data['template'], $matches);
        $params = explode('|', $matches[1]);
        $params_str = '';
        foreach ($params as $param) {
            $param_arr = explode(':', $param);
            // Used parameters
            $number[$param_arr[0]] = $param_arr[1];
            $params_str .= '|' . $param_arr[0] . ':' . $param_arr[1];
        }
        // Remove the number format from template
        $data['template'] = str_replace($params_str, '', $data['template']);

        $format = array(
            'template' => array(
                '[abbr][value]',
                '[abbr] [value]',
                '[value][abbr]',
                '[value] [abbr]',
                '[gid][value]',
                '[gid] [value]',
                '[value][gid]',
                '[value] [gid]',
            ),
            'dec_sep' => array(
                'period' => '.',
                'comma' => ',',
            ),
            'gr_sep' => array(
                'period' => '.',
                'comma' => ',',
                'space' => ' ',
                'empty' => '',
            ),
            'dec_part' => array(
                'one' => '1',
                'two' => '2',
                'dash' => '-',
                'none' => '',
            ),
            'used' => array(
                'template' => $data['template'],
                'dec_sep' => $number['dec_sep'],
                'gr_sep' => $number['gr_sep'],
                'dec_part' => $number['dec_part'],
            ),
        );

        $this->view->assign('format', $format);
        $this->view->assign('data', $data);

        $this->pg_theme->add_js('admin-currencies.js', 'currencies');
        $this->Menu_model->set_menu_active_item('admin_payments_menu', 'settings_list_item');

        $this->view->setHeader(l('admin_header_currency_list', 'currencies'));

        $this->view->render('edit_settings');
    }

    /**
     * Remove currency
     *
     * @param integer $id currency identifier
     *
     * @return void
     */
    public function settings_delete($id)
    {
        $this->load->model("Currencies_model");
        $data = $this->Currencies_model->get_currency_by_id($id);
        if ($data["is_default"]) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_delete_is_default_currency', 'currencies'));
        } else {
            $this->Currencies_model->delete_currency($id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_currency', 'currencies'));
        }
        $this->view->setRedirect(site_url() . "admin/currencies/settings");
    }

    /**
     * Make currency as default
     *
     * @param integer $id currency identifier
     *
     * @return void
     */
    public function settings_use($id)
    {
        $this->load->model("Currencies_model");
        $this->Currencies_model->set_default($id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_currency', 'currencies'));
        $this->view->setRedirect(site_url() . "admin/currencies/settings");
    }

    /**
     * Change currency rates settings
     *
     * @return void
     */
    public function update_currency_rates()
    {
        switch (true) {
            case $this->input->post("bt_auto_x") || $this->input->post("bt_auto_y"):
                $status = $this->input->post("use_rates_update");
                $rates_driver = $this->input->post("rates_driver");
                $this->pg_module->set_module_config("currencies", "use_rates_update", ($status ? 1 : 0));
                $this->pg_module->set_module_config("currencies", "rates_update_driver", $rates_driver);
                if ($status) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                       l('success_rates_update_turn_on', 'currencies'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                       l('success_rates_update_turn_off', 'currencies'));
                }
                break;
            case $this->input->post("bt_manual_x") || $this->input->post("bt_manual_y"):
                $this->load->model("Currencies_model");
                $rates_driver = $this->input->post("rates_driver");
                $result = $this->Currencies_model->update_currency_rates($rates_driver);
                if (!empty($result["errors"])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $result["errors"]);
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_rates_updated', 'currencies'));
                }
                break;
        }
        $this->view->setRedirect(site_url() . "admin/currencies/settings");
    }

    /**
     * Turn on/off automatic update currency rates by ajax
     *
     * Available status values: 1 - turn on, 2 - turn off
     *
     * @param integer $status active status
     *
     * @return void
     */
    public function ajax_use_rates_update($status)
    {
        $response = array("error" => "", "success" => "");

        $this->pg_module->set_module_config("currencies", "use_rates_update", ($status ? 1 : 0));
        if ($status) {
            $response["success"] = l('success_rates_update_turn_on', 'currencies');
        } else {
            $response["success"] = l('success_rates_update_turn_off', 'currencies');
        }
        
        $this->view->assign($response);
        
        $this->view->render();
    }

    /**
     * Save provider of currency rates
     *
     * @param string $updater provider guid
     *
     * @return void
     */
    public function ajax_rates_driver_update($updater)
    {
        $response = array("error" => "", "success" => "");

        $this->pg_module->set_module_config("currencies", "rates_update_driver", $updater);
        $response["success"] = l('success_rates_driver_updated', 'currencies');
        
        $this->view->assign($response);
        
        $this->view->render();
    }

    /**
     * Update currency rates by ajax
     *
     * @param string $updater provider guid
     *
     * @return void
     */
    public function ajax_currency_rates_update($updater)
    {
        $response = array("error" => "", "success" => "");
        $this->load->model("Currencies_model");
        $result = $this->Currencies_model->update_currency_rates($updater);
        if (!empty($result["errors"])) {
            $response["error"] = implode("<br>", $result["errors"]);
        } else {
            $response["success"] = l('success_rates_updated', 'currencies');
        }
        
        $this->view->assign($response);
        
        $this->view->render();
    }

    /**
     * Activate/deactivate currency
     *
     * Available status values: 1 - activate, 0 - deactivate
     *
     * @param integer $currency_id currency identifier
     * @param boolean $status      active status
     *
     * @return void
     */
    public function settings_active($currency_id, $status)
    {
        $this->load->model("Currencies_model");
        $result = $this->Currencies_model->activate_currency($currency_id, $status);
        if ($result) {
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_currency_status_changed', 'currencies'));
        } else {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_deactivate_is_default_currency', 'currencies'));
        }
        $this->view->setRedirect(site_url() . 'admin/currencies/settings');
    }

}
