<?php

namespace Pg\Modules\Currencies\Models;

/**
 * currencies module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * currencies install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Currencies
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Currencies_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        'admin_menu' => array(
            'action' => 'none',
            'name' => 'Admin name',
            'items' => array(
                "settings_items" => array(
                    'action' => 'none',
                    'items' => array(
                        'system-items' => array(
                            'action' => 'none',
                            'items' => array(
                                'currencies_menu_item' => array(
                                    'action' => 'create',
                                    'link' => 'admin/currencies',
                                    'icon' => 'money',
                                    'status' => 1,
                                    'indicator_gid' => '',
                                    'sorter' => 1,
                                ),
                            ),
                        ),
                    ),
                ),
            ),
        ),
    );

    /**
     * Menu configuration for payments
     *
     * @var array
     */
    protected $menu_payments = array(
        'admin_payments_menu' => array(
            'action' => 'none',
            'name' => '',
            'items' => array(
                'payments_currencies_menu_item' => array(
                    'action' => 'create',
                    'link' => 'admin/currencies',
                    'icon' => 'money',
                    'status' => 1,
                    'indicator_gid' => '',
                    'sorter' => 3,
                ),
            ),
        ),
    );

    /**
     * Moderators configuration
     *
     * @var array
     */
    protected $moderators_methods = array(
        array('module' => 'currencies', 'method' => 'settings', 'is_default' => 1),
    );

    /**
     * Constructor
     *
     * @return Currencies_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        // load langs
        $this->ci->load->model('Install_model');
    }

    /**
     * Install data of menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read('currencies', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');
        } else {
            $this->ci->load->helper('menu');
            foreach ($this->menu as $gid => $menu_data) {
                linked_install_process_menu_items(
                        $this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_file
                );
            }
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $this->ci->load->helper('menu');

        $return = array("menu" => array());
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items(
                    $this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_ids
            );
            $return["menu"] = array_merge($return["menu"], $temp);
        }

        return $return;
    }

    /**
     * Uninstall data of menu model
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]['items']);
            }
        }
    }

    /**
     * Install data of payments module
     *
     * @return void
     */
    public function install_payments()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu_payments as $gid => $menu_data) {
            $this->menu_payments[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu_payments, 'create', $gid, 0, $menu_data["items"]);
        }
    }

    /**
     * Import languages of payments module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_payments_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read('currencies', 'menu_payments', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu payments (currencies) langs data');
            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->menu_payments as $gid => $menu_data) {
            linked_install_process_menu_items(
                    $this->menu_payments, 'update', $gid, 0, $menu_data["items"], $gid, $langs_file
            );
        }

        return true;
    }

    /**
     * Export languages of payments module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_payments_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->menu_payments as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu_payments, 'export', $gid, 0, $menu_data["items"],
                                                      $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu_payments" => $return);
    }

    /**
     * Uninstall data of payments model
     *
     * @return void
     */
    public function deinstall_payments()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu_payments as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $menu_data['items']);
            }
        }
    }

    /**
     * Install data of moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        // install moderators permissions
        $this->ci->load->model('Moderators_model');

        foreach ($this->moderators_methods as $method) {
            $this->ci->Moderators_model->save_method(null, $method);
        }
    }

    /**
     * Import languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read('currencies', 'moderators', $langs_ids);

        // install moderators permissions
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'currencies';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method['method']])) {
                $this->ci->Moderators_model->save_method($method['id'], array(), $langs_file[$method['method']]);
            }
        }
    }

    /**
     * Export languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'currencies';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method['method']] = $method['langs'];
        }

        return array('moderators' => $return);
    }

    /**
     * Uninstall data of moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        // delete moderation methods in moderators
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'currencies';
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install data of cronjob module
     *
     * @return void
     */
    public function install_cronjob()
    {
        $this->ci->load->model("Cronjob_model");
        $cron_data = array(
            "name" => "Currency rates updater",
            "module" => "currencies",
            "model" => "Currencies_model",
            "method" => "cron_update_currency_rates",
            "cron_tab" => "0 0 * * *",
            "status" => "1",
        );
        $this->ci->Cronjob_model->save_cron(null, $cron_data);
    }

    /**
     * Uninstall data of cronjob module
     *
     * @return void
     */
    public function deinstall_cronjob()
    {
        $this->ci->load->model("Cronjob_model");
        $cron_data = array();
        $cron_data["where"]["module"] = "currencies";
        $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
    }

}
