<?php

namespace Pg\Modules\Currencies\Models;

/**
 * Currencies module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * Currencies model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Currencies
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Yahoo_currency_rates_model extends \Model
{

    /**
     * Link to CodeIgnitor object
     *
     * @var object
     */
    protected $ci;

    /**
     * Service base url
     *
     * @var string
     */
    protected $yql_base_url = "http://query.yahooapis.com/v1/public/yql?q=%s&format=json&env=store://datatables.org/alltableswithkeys";

    /**
     * YQL base query
     *
     * @var string
     */
    protected $yql_base_query = "select * from yahoo.finance.xchange where pair IN('%s')";

    /**
     * Class constructor
     *
     * @return Yahoo_currency_rates_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Update exists currency rates
     *
     * @param string  $base_currency basic currency GUID
     * @param array   $currencies    site currencies
     * @param boolean $use_curl      use curl extension
     *
     * @return array
     */
    public function update_rates($base_currency, $currencies, $use_curl = false)
    {
        if (empty($currencies)) {
            return array();
        }

        $pairs = array();
        foreach ($currencies as $need_currency) {
            if ($need_currency["gid"] != $base_currency) {
                $pairs[] = $need_currency["gid"] . $base_currency;
            }
        }

        if (empty($pairs)) {
            return array();
        }

        $yql_query_url = sprintf($this->yql_base_url, urlencode(sprintf($this->yql_base_query, implode("','", $pairs))));
        if ($use_curl) {
            $session = curl_init($yql_query_url);
            curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
            $json = curl_exec($session);
        } else {
            $json = file_get_contents($yql_query_url);
        }
        $results = json_decode($json, true);

        if (!empty($results['error'])) {
            if (is_array($results['error'])) {
                $results['error'] = implode("\n", $results['error']);
            }
            throw new \Exception($results['error']);
        }

        $return = array();
        foreach ($results['query']['results']['rate'] as $result) {
            $currency_gid = preg_replace("/" . preg_quote($base_currency, "/") . "$/i", "", $result['id']);
            $return[$currency_gid] = $result['Rate'];
        }

        return $return;
    }

}
