<?php

namespace Pg\Modules\Export\Controllers;

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Export user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Export extends \Controller
{

    /**
     * Constructor
     *
     * @return Export
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Menu_model");
    }

    /**
     * Simple generation data of selection
     *
     * @param integer $selection_id selection identifier
     *
     * @return void
     */
    public function index($selection_id)
    {
        if ($this->session->userdata("auth_type") != "user") {
            show_404();

            return;
        }

        $this->load->model("export/models/export_module_model", "Export_module_model");
        $this->load->model("Export_model");

        $data = $this->Export_model->get_selection_by_id($selection_id, true);
        if ($data["output_type"] != "browser" || !$data["published"]) {
            show_404();

            return;
        }

        $user_id = $this->session->userdata("user_id");
        $this->Export_model->generate_selection($selection_id, $data['admin_form_data'], $user_id);
    }

    /**
     * Advanced generation data of selection
     *
     * @param integer $selection_id selection identifier
     *
     * @return void
     */
    public function advanced($selection_id)
    {
        if ($this->session->userdata("auth_type") != "user") {
            show_404();

            return;
        }

        $this->load->model("export/models/export_module_model", "Export_module_model");
        $this->load->model("Export_model");

        $data = $this->Export_model->get_selection_by_id($selection_id, true);
        if ($data["output_type"] != "browser" || !$data["published"] || !$data["editable"]) {
            show_404();

            return;
        }

        if ($this->input->post("btn_save")) {
            $form_data = $this->Export_module_model->process_search_form($data["id_object"], false);
            $user_id = $this->session->userdata("user_id");
            $this->Export_model->generate_selection($selection_id, $form_data, $user_id);
        }
        $this->Menu_model->breadcrumbs_set_active(l("header_my_export", "export") . " " . $data["driver"]["output_name"]);
        $search_form = $this->Export_module_model->get_search_form($data["id_object"], $data['admin_form_data'], false);
        $this->view->assign("export_form", $search_form);
        $this->view->assign("data", $data);
        $this->view->render("advanced");
    }

}
