<?php

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * CSV driver export model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Export_csv_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Driver configuration
     *
     * @var array
     */
    public $drivers = array(
        "gid"      => "csv",
        "link"     => "http://mastpoint.curzonnassau.com/csv-1203/",
        "settings" => array("delimiter" => ";", "enclosure" => "\""),
        "elements" => array(
            array("name" => "id", "type" => "int", "required" => 0),
            array("name" => "id_user", "type" => "int", "required" => 0),
            array("name" => "user_name", "type" => "text", "required" => 0),
            array("name" => "id_type", "type" => "int", "required" => 0),
            array("name" => "type", "type" => "text", "required" => 0),
            array("name" => "id_category", "type" => "int", "required" => 0),
            array("name" => "category_name", "type" => "text", "required" => 0),
            array("name" => "id_property_type", "type" => "int", "required" => 0),
            array("name" => "property_type", "type" => "text", "required" => 0),
            array("name" => "status", "type" => "text", "required" => 0),
            array("name" => "country_code", "type" => "text", "required" => 0),
            array("name" => "id_region", "type" => "int", "required" => 0),
            array("name" => "id_city", "type" => "int", "required" => 0),
            array("name" => "country", "type" => "text", "required" => 0),
            array("name" => "region", "type" => "text", "required" => 0),
            array("name" => "region_code", "type" => "text", "required" => 0),
            array("name" => "city", "type" => "text", "required" => 0),
            array("name" => "address", "type" => "text", "required" => 0),
            array("name" => "zip", "type" => "text", "required" => 0),
            array("name" => "sold", "type" => "int", "required" => 0),
            array("name" => "price", "type" => "int", "required" => 0),
            array("name" => "price_max", "type" => "int", "required" => 0),
            array("name" => "gid_currency", "type" => "text", "required" => 0),
            array("name" => "square", "type" => "int", "required" => 0),
            array("name" => "square_max", "type" => "int", "required" => 0),
            array("name" => "square_unit", "type" => "text", "required" => 0),
            array("name" => "date_created", "type" => "text", "required" => 0),
            array("name" => "date_modified", "type" => "text", "required" => 0),
            array("name" => "date_publish", "type" => "text", "required" => 0),
            array("name" => "date_expire", "type" => "text", "required" => 0),
            array("name" => "date_open", "type" => "text", "required" => 0),
            array("name" => "uploads", "type" => "file", "required" => 0),
        ),
        "editable" => 1,
        "status"   => 1,
    );

    /**
     *  Demo data
     *
     *  @return void
     */
    public $demo_data = array(
        'gid_driver'      => 'csv',
        'id_object'       => '2',
        'module'          => 'listings',
        'name'            => 'Test sample',
        'relations'       => 'a:20:{i:0;a:4:{s:4:"name";s:2:"id";s:4:"type";s:3:"int";s:4:"link";s:2:"id";s:5:"label";s:14:"ID ref # (int)";}i:2;a:4:{s:4:"name";s:9:"user_name";s:4:"type";s:4:"text";s:4:"link";s:7:"id_user";s:5:"label";s:26:"Listing provided by (text)";}i:4;a:4:{s:4:"name";s:4:"type";s:4:"type";s:4:"text";s:4:"link";s:9:"type_code";s:5:"label";s:30:"Type of operation (text, code)";}i:6;a:4:{s:4:"name";s:13:"category_name";s:4:"type";s:4:"text";s:4:"link";s:11:"id_category";s:5:"label";s:15:"Category (text)";}i:8;a:4:{s:4:"name";s:13:"property_type";s:4:"type";s:4:"text";s:4:"link";s:13:"property_type";s:5:"label";s:20:"Property type (text)";}i:9;a:4:{s:4:"name";s:6:"status";s:4:"type";s:4:"text";s:4:"link";s:6:"status";s:5:"label";s:13:"Active (text)";}i:13;a:4:{s:4:"name";s:7:"country";s:4:"type";s:4:"text";s:4:"link";s:10:"id_country";s:5:"label";s:14:"Country (text)";}i:14;a:4:{s:4:"name";s:6:"region";s:4:"type";s:4:"text";s:4:"link";s:9:"id_region";s:5:"label";s:13:"Region (text)";}i:16;a:4:{s:4:"name";s:4:"city";s:4:"type";s:4:"text";s:4:"link";s:7:"id_city";s:5:"label";s:11:"City (text)";}i:17;a:4:{s:4:"name";s:7:"address";s:4:"type";s:4:"text";s:4:"link";s:7:"address";s:5:"label";s:14:"Address (text)";}i:18;a:4:{s:4:"name";s:3:"zip";s:4:"type";s:4:"text";s:4:"link";s:3:"zip";s:5:"label";s:18:"Postal code (text)";}i:19;a:4:{s:4:"name";s:4:"sold";s:4:"type";s:3:"int";s:4:"link";s:4:"sold";s:5:"label";s:19:"Recently sold (int)";}i:20;a:4:{s:4:"name";s:5:"price";s:4:"type";s:3:"int";s:4:"link";s:5:"price";s:5:"label";s:11:"Price (int)";}i:21;a:4:{s:4:"name";s:9:"price_max";s:4:"type";s:3:"int";s:4:"link";s:9:"price_max";s:5:"label";s:15:"Max price (int)";}i:22;a:4:{s:4:"name";s:12:"gid_currency";s:4:"type";s:4:"text";s:4:"link";s:12:"gid_currency";s:5:"label";s:15:"Currency (text)";}i:23;a:4:{s:4:"name";s:6:"square";s:4:"type";s:3:"int";s:4:"link";s:6:"square";s:5:"label";s:19:"Property size (int)";}i:24;a:4:{s:4:"name";s:10:"square_max";s:4:"type";s:3:"int";s:4:"link";s:10:"square_max";s:5:"label";s:25:"Property size (max) (int)";}i:25;a:4:{s:4:"name";s:11:"square_unit";s:4:"type";s:4:"text";s:4:"link";s:11:"square_unit";s:5:"label";s:11:"Unit (text)";}i:26;a:4:{s:4:"name";s:12:"date_created";s:4:"type";s:4:"text";s:4:"link";s:12:"date_created";s:5:"label";s:14:"Created (text)";}i:31;a:4:{s:4:"name";s:7:"uploads";s:4:"type";s:4:"file";s:4:"link";s:7:"uploads";s:5:"label";s:12:"Photo (file)";}}',
        'admin_form_data' => '',
        'output_type'     => 'browser',
        'file_format'     => 'test.csv',
        'append_date'     => '1',
        'scheduler'       => 'a:1:{s:4:"type";s:6:"manual";}',
        'published'       => '0',
        'editable'        => '0',
        'status'          => '1',
        'date_created'    => '',
    );

    /**
     * Content type
     *
     * @var object
     */
    public $content_file = "text/csv";

    /**
     * Constructor
     *
     * @return Export_csv_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
    }

    /**
     * Return filename of export data by default
     *
     * @return string
     */
    public function get_filename()
    {
        return "export.csv";
    }

    /**
     * Generate output file
     *
     * @param string $filename output filename
     * @param array  $data     export data
     * @param array  $settings driver settings
     *
     * @return void
     */
    public function generate($filename, $data, $settings)
    {
        $f = fopen($filename, "wb");
        foreach ($data as $row) {
            fputcsv($f, $row, $settings["delimiter"] ? $settings["delimiter"] : ";", $settings["enclosure"] ? $settings["enclosure"] : "\"");
        }
        fclose($f);
    }

    /**
     * Validate export data
     *
     * @param array $data export data
     *
     * @return array
     */
    public function validate_data($data)
    {
        $return = array("errors" => array(), "data" => array());
        $return["data"] = $data;

        return $return;
    }
}
