<?php

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

define("EXPORT_TABLE", DB_PREFIX . "export");

/**
 * Export model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Export_model extends Model
{
    /**
     * link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Properties of export object in data source
     *
     * @var array
     */
    protected $_fields = array(
        "id",
        "gid_driver",
        "id_object",
        "name",
        "relations",
        "admin_form_data",
        "output_type",
        "file_format",
        "append_date",
        "scheduler",
        "published",
        "editable",
        "status",
        "date_created",
    );

    /**
     * Settings for formatting export object
     *
     * @var array
     */
    protected $format_settings = array(
        "use_format"  => true,
        "get_driver"  => true,
        "get_module"  => true,
    );

    /**
     * Export drivers cache
     *
     * @var array
     */
    protected $driver_cache = array();

    /**
     * Export modules cache
     *
     * @var array
     */
    protected $module_cache = array();

    /**
     * Path of temporal export data
     *
     * @var string
     */
    protected $save_path = "export/";

    /**
     * Output types options
     *
     * @var array
     */
    public $output_types = array("file", "browser");

    /**
     * Constructor
     *
     * @return Export_model
     * @required Export_driver_model
     * @required Export_module_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->ci->load->model("export/models/Export_driver_model");
        $this->ci->load->model("export/models/Export_module_model");
    }

    /**
     * Return driver object by GUID
     *
     * @param string $driver_gid driver guid
     *
     * @return array
     */
    public function get_driver_by_gid($driver_gid)
    {
        if (!isset($this->driver_cache[$driver_gid])) {
            $driver_data = $this->ci->Export_driver_model->get_driver_by_gid($driver_gid);
            if (!is_array($driver_data) || !count($driver_data)) {
                return false;
            }
            $this->driver_cache[$driver_data["id"]] = $driver_data;
            $this->driver_cache[$driver_gid] = $driver_data;
        }

        if (is_array($this->driver_cahce[$driver_gid]) && count($this->driver_cache[$type_gid])) {
            return $this->driver_cahce[$driver_gid];
        } else {
            return false;
        }
    }

    /**
     * Return selection object by identifier
     *
     * @param integer $selection_id selection identifier
     * @param boolean $formatted    format results
     *
     * @return array/false
     */
    public function get_selection_by_id($selection_id, $formatted = false)
    {
        $selection_id = intval($selection_id);

        $this->ci->db->select(implode(", ", $this->_fields))->from(EXPORT_TABLE)->where("id", $selection_id);

        //_compile_select;
        $result = $this->ci->db->get()->result_array();
        if (!empty($result)) {
            $rt = array($result[0]);
            if ($formatted) {
                $rt = $this->format_selection($rt);
            }

            return $rt[0];
        } else {
            return false;
        }
    }

    /**
     * Save selection object to data source
     *
     * @param integer $selection_id selection identifier
     * @param array   $data         selection data
     *
     * @return integer
     */
    public function save_selection($selection_id, $data)
    {
        if (!$selection_id) {
            $data["date_created"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(EXPORT_TABLE, $data);
            $selection_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where("id", $selection_id);
            $this->ci->db->update(EXPORT_TABLE, $data);
        }

        return $selection_id;
    }

    /**
     * Remove export selection object by identifier
     *
     * @param integer $selection_id export selection identifier
     *
     * @return integer
     */
    public function delete_selection($selection_id)
    {
        $this->ci->db->where("id", $selection_id);
        $this->ci->db->delete(EXPORT_TABLE);
    }

    /**
     * Activate selection object
     *
     * @param integer $selection_id selection identifier
     *
     * @return boolean
     */
    public function activate_selection($selection_id)
    {
        $selection = $this->get_selection_by_id($selection_id, true);
        if (!is_array($selection["relations"]) || empty($selection["relations"])) {
            return false;
        }
        $data["status"] = "1";
        $this->save_selection($selection_id, $data);

        return true;
    }

    /**
     * De-activate selection object
     *
     * @param integer $selection_id selection identifier
     *
     * @return boolean
     */
    public function deactivate_selection($selection_id)
    {
        $data["status"] = "0";
        $this->save_selection($selection_id, $data);

        return true;
    }

    /**
     * Return filter by module identifier
     *
     * @param integer $module_id module identifier
     *
     * @return array
     */
    protected function _get_selections_by_module($module_id)
    {
        if (!$object) {
            return array();
        }
        $params["where"]["id_module"] = $module_id;

        return $params;
    }

    /**
     * Return filter by driver guid
     *
     * @param string $driver_gid driver guid
     *
     * @return array
     */
    protected function _get_selections_by_driver($driver_gid)
    {
        if (!$object) {
            return array();
        }
        $params["where"]["gid_driver"] = $driver_gid;

        return $params;
    }

    /**
     * Return filter by active selections
     *
     * @param boolean $use use filter
     *
     * @return array
     */
    protected function _get_selections_by_active($use = true)
    {
        if (!$use) {
            return array();
        }
        $params["where"]["status"] = 1;

        return $params;
    }

    /**
     * Return filter by inactive selections
     *
     * @param boolean $use use filter
     *
     * @return array
     */
    protected function _get_selections_by_not_active($use = true)
    {
        if (!$use) {
            return array();
        }
        $params["where"]["status"] = 0;

        return $params;
    }

    /**
     * Return selection objects as array (internal)
     *
     * @param integer $page     page of results
     * @param string  $limits   results per page
     * @param array   $order_by sorting data
     * @param array   $params   filters data
     *
     * @return array
     */
    protected function _get_selections_list($page = null, $limits = null, $order_by = null, $params = array())
    {
        $this->ci->db->select(implode(", ", $this->_fields));
        $this->ci->db->from(EXPORT_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->_fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        } elseif ($order_by) {
            $this->ci->db->order_by($order_by);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($limits, $limits * ($page - 1));
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $r;
            }

            return $this->format_selection($data);
        }

        return array();
    }

    /**
     * Return number of selections objects (internal)
     *
     * @param array $params filters data
     *
     * @return integer
     */
    protected function _get_selections_count($params = null)
    {
        $this->ci->db->select("COUNT(*) AS cnt");
        $this->ci->db->from(EXPORT_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return selection objects as array
     *
     * @param array   $filters  filters data
     * @param integer $page     page of results
     * @param string  $limits   results per page
     * @param array   $order_by sorting data
     *
     * @return array
     */
    public function get_selections_list($filters = array(), $page = null, $limits = null, $order_by = null)
    {
        $params = array();
        foreach ($filters as $filter => $value) {
            $params = array_merge_recursive($params, $this->{"_get_selections_by_" . $filter}($value));
        }

        return $this->_get_selections_list($page, $limits, $order_by);
    }

    /**
     * Return number of selections objects
     *
     * @param array $filters filters data
     *
     * @return integer
     */
    public function get_selections_count($filters = array())
    {
        $params = array();
        foreach ($filters as $filter => $value) {
            $params = array_merge_recursive($params, $this->{"_get_selections_by_" . $filter}($value));
        }

        return $this->_get_selections_count($params);
    }

    /**
     * Validate selection data
     *
     * @param integer $id   selection identifier
     * @param array   $data selection data
     *
     * @return array
     */
    public function validate_selection($id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["name"])) {
            $return["data"]["name"] = trim(strip_tags($data['name']));
            if (empty($return["data"]["name"])) {
                $return["errors"][] = l("error_empty_name", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_name", "export");
        }

        if (isset($data["gid_driver"])) {
            $return["data"]["gid_driver"] = trim(strip_tags($data['gid_driver']));
            if (empty($return["data"]["gid_driver"]) || !($driver = $this->ci->Export_driver_model->get_driver_by_gid($return["data"]["gid_driver"]))) {
                $return["errors"][] = l("error_empty_driver", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_driver", "export");
        }

        if (isset($data["id_object"])) {
            $return["data"]["id_object"] = intval($data['id_object']);
            if (empty($return["data"]["id_object"]) || !($module = $this->ci->Export_module_model->get_module_by_id($return["data"]["id_object"]))) {
                $return["errors"][] = l("error_empty_object", "export");
            }
        } elseif (!$id) {
            $return["errors"][] = l("error_empty_object", "export");
        }

        if (isset($data["relations"])) {
            $return["data"]["relations"] = serialize($data['relations']);
        } elseif (!$id) {
            //$return["errors"][] = l("error_empty_relations", "export");
        }

        if (isset($data["output_type"])) {
            $return["data"]["output_type"] = trim(strip_tags($data['output_type']));
            if (empty($return["data"]["output_type"])) {
                $return["errors"][] = l("error_empty_output_type", "export");
            }
        } elseif (!$id) {
            $return["data"]["output_type"] = 'file';
            //$return["errors"][] = l("error_empty_output_type", "export");
        }

        if (isset($data["admin_form_data"])) {
            if (!is_array($data["admin_form_data"])) {
                $data["admin_form_data"] = array();
            }
            $return["data"]["admin_form_data"] = serialize($data["admin_form_data"]);
        }

        if (isset($data["file_format"])) {
            $return["data"]["file_format"] = trim(strip_tags($data['file_format']));
        }

        if (isset($data["scheduler"])) {
            $return["data"]["scheduler"] = $data["scheduler"];
            if ($return["data"]["output_type"] == "browser") {
                unset($return["data"]["scheduler"]);
            } elseif (empty($return["data"]["scheduler"])) {
                $return["errors"][] = l("error_empty_scheduler", "export");
            } else {
                switch ($return["data"]["scheduler"]["type"]) {
                    case "cron":
                        $return["data"]["scheduler"] = array(
                            "type"    => "cron",
                            "date"    => strval($return["data"]["scheduler"]["date"]),
                            "hours"   => intval($return["data"]["scheduler"]["hours"]),
                            "minutes" => intval($return["data"]["scheduler"]["minutes"]),
                            "period"  => strval($return["data"]["scheduler"]["period"]),
                        );
                        $return["data"]["scheduler"]["date_for_cron"] = strtotime($return["data"]["scheduler"]["date"] . " " . $return["data"]["scheduler"]["hours"] . ":" . $return["data"]["scheduler"]["minutes"]);
                        $return["data"]["scheduler"] = serialize($return["data"]["scheduler"]);
                    break;
                    case "intime":
                        $return["data"]["scheduler"] = array(
                            "type"    => "intime",
                            "date"    => strval($return["data"]["scheduler"]["date"]),
                            "hours"   => intval($return["data"]["scheduler"]["hours"]),
                            "minutes" => intval($return["data"]["scheduler"]["minutes"]),
                        );
                        $return["data"]["scheduler"]["date_for_cron"] = strtotime($return["data"]["scheduler"]["date"] . " " . $return["data"]["scheduler"]["hours"] . ":" . $return["data"]["scheduler"]["minutes"]);
                        $return["data"]["scheduler"] = serialize($return["data"]["scheduler"]);
                    break;
                    case "manual":
                        $return["data"]["scheduler"] = serialize(array("type" => "manual"));
                    break;
                    default:
                        unset($return["data"]["scheduler"]);
                    break;
                }
            }
        }

        if (isset($data["published"])) {
            $return["data"]["published"] = $data["published"] ? 1 : 0;
        }

        if (isset($data["editable"])) {
            $return["data"]["editable"] = $data["editable"] ? 1 : 0;
        }

        if (isset($data["append_date"])) {
            $return["data"]["append_date"] = $data['append_date'] ? 1 : 0;
        }

        return $return;
    }

    /**
     * Set settings for formatting selections
     *
     * @param array $data format settings
     *
     * @return array
     */
    public function set_format_settings($data)
    {
        foreach ($data as $key => $value) {
            if (isset($this->format_settings[$key])) {
                $this->format_settings[$key] = $value;
            }
        }
    }

    /**
     * Format selection data
     *
     * @param array $data selection data
     *
     * @return array
     */
    public function format_selection($data)
    {
        if (!$this->format_settings["use_format"]) {
            return $data;
        }

        $drivers_data = $this->ci->Export_driver_model->get_drivers();
        foreach ($drivers_data as $driver_data) {
            $this->driver_cache[$driver_data["gid"]] = $driver_data;
        }

        $modules_data = $this->ci->Export_module_model->get_modules();
        foreach ($modules_data as $module_data) {
            $this->module_cache[$module_data["id"]] = $module_data;
        }

        $drivers_search = array();
        $modules_search = array();

        foreach ($data as $key => $selection) {
            //get driver
            if ($this->format_settings["get_driver"]) {
                $drivers_search[] = $selection["gid_driver"];
            }
            //get module
            if ($this->format_settings["get_module"]) {
                $modules_search[] = $selection["id_object"];
            }

            $selection["relations"] = unserialize($selection["relations"]);
            $selection["admin_form_data"] = unserialize($selection["admin_form_data"]);
            if ($selection["output_type"] == "file") {
                $selection["scheduler"] = unserialize($selection["scheduler"]);
                if (!isset($selection["scheduler"]["type"])) {
                    $selection["scheduler"] = array("type" => "manual");
                }
            } else {
                unset($selection["scheduler"]);
            }

            $selection["output_name"] = $selection["name"];

            $data[$key] = $selection;
        }

        if ($this->format_settings["get_driver"] && !empty($drivers_search)) {
            foreach ($data as $key => $selection) {
                $data[$key]["driver"] = (isset($this->driver_cache[$selection["gid_driver"]])) ?
                    $this->driver_cache[$selection["gid_driver"]] : $this->ci->Export_driver_model->format_default_driver($selection["gid_driver"]);
            }
        }

        if ($this->format_settings["get_module"] && !empty($modules_search)) {
            foreach ($data as $key => $selection) {
                $data[$key]["module"] = (isset($this->module_cache[$selection["id_object"]])) ?
                    $this->module_cache[$selection["id_object"]] : $this->ci->Export_module_model->format_default_module($selection["id_object"]);
            }
        }

        return $data;
    }

    /**
     * Generate data of selection
     *
     * @param integer $selection_id selection identifier
     * @param array   $form_data    form data
     * @param integer $user_id      user who run generating selection
     *
     * @return void
     */
    public function generate_selection($selection_id, $form_data = null, $user_id = 0)
    {
        @ini_set("max_execution_time", 0);

        $selection = $this->get_selection_by_id($selection_id, true);
        if (!$selection || !$selection['status']) {
            return "";
        }

        if (is_null($form_data)) {
            $form_data = $selection['admin_form_data'];
        }

        $model_name = "Export_" . $selection["gid_driver"] . "_model";
        $this->ci->load->model("export/models/drivers/" . strtolower($model_name), $model_name);

        $errors = array();

        $archive = false;
        $attachements = array();

        $data = $this->ci->Export_module_model->export_data($selection["id_object"], $form_data, $selection["relations"], $user_id);

        foreach ($data['data'] as $i => $item) {
            $row = array();
            foreach ($selection["relations"] as $relation) {
                $value = $item[$relation["link"]];
                switch ($relation["type"]) {
                    case "text":
                    case "url":
                        if (isset($data['fk'][$i][$relation["link"]])) {
                            $value = $data['fk'][$i][$relation["link"]];
                        }
                        if (is_array($value)) {
                            foreach ($value as $k => $v) {
                                $value[$k] = strval(strip_tags($v));
                            }
                            $row[$relation["name"]] = implode('|', $value);
                        } else {
                            $row[$relation["name"]] = strval(strip_tags($value));
                        }
                    break;
                    case "int":
                        $row[$relation["name"]] = intval($value);
                    break;
                    case "file":
                        //add file to archive
                        $archive = true;
                        if (is_array($value)) {
                            foreach ($value as $k => $v) {
                                //if(!file_exists($v)) continue;
                                $attachements[] = $v;
                                $value[$k] = basename($v);
                            }
                            $row[$relation["name"]] = implode("|", $value);
                        } else {
                            $attachements[] = $value;
                            $row[$relation["name"]] = basename($value);
                        }
                    break;
                    default:
                        $row[$relation["name"]] = $value;
                    break;
                }
            }

            $result = $this->ci->{$model_name}->validate_data($row, $selection["driver"]["settings"]);
            if (!empty($result["errors"])) {
                $errors[] = array('data' => $row, 'errors' => $result["errors"]);
                if (isset($data['fk'][$i])) {
                    unset($data['fk'][$i]);
                }
                unset($data['data'][$i]);
            } else {
                $data['data'][$i] = $result["data"];
            }
        }

        if (!empty($errors)) {
            @mkdir(TEMPPATH . 'logs/export/' . $selection['id']);
            $log_path = TEMPPATH . 'logs/export/' . $selection['id'] . '/log-' . date('Y-m-d') . '.txt';
            $f = fopen($log_path, 'wb');
            foreach ($errors as $error) {
                fputs($f, 'Data:' . "\n");
                foreach ($error['data'] as $name => $value) {
                    fputs($f, "\t" . $name . ' = ' . $value . ';');
                }
                fputs($f, "\n" . 'Errors:' . "\n");
                foreach ($error['errors'] as $error) {
                    fputs($f, "\t" . $error . "\n");
                }
                fputs($f, "\n");
            }
            fclose($f);
        }

        $data = $data['data'];

        $columns = array();
        foreach ($selection["relations"] as $relation) {
            $columns[] = $relation["name"];
        }

        array_unshift($data, $columns);

        $path = TEMPPATH . $this->save_path . '/' . $selection['id'] . '/';
        if (!is_dir($path)) {
            @mkdir($path, 0777, true);
        }
        $filename = tempnam($path, $selection["gid_driver"]);
        $this->ci->{$model_name}->generate($filename, $data, $selection["driver"]["settings"]);

        $content_type = $this->ci->{$model_name}->content_type;
        $output_filename = $this->ci->{$model_name}->get_filename();

        if ($archive) {
            $this->ci->load->library("zip");

            $this->ci->zip->add_data($output_filename, file_get_contents($filename));
            unlink($filename);

            foreach ($attachements as $attachement) {
                $content = @file_get_contents($attachement);
                if (!$content) {
                    continue;
                }
                $this->ci->zip->add_data(basename($attachement), $content);
            }

            $filename = tempnam($path, $selection["gid_driver"]);
            $this->ci->zip->archive($filename);

            $content_type = "application/octet-stream";
            $output_filename = "export.zip";
        }

        switch ($selection["output_type"]) {
            case "browser":
                header("Content-Description: File Transfer");
                header("Content-Type: " . $content_type);
                header("Content-Disposition: attachment; filename=" . $output_filename);
                header("Content-Transfer-Encoding: binary");
                header("Expires: 0");
                header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
                header("Pragma: public");
                header("Content-Length: " . filesize($filename));
                echo file_get_contents($filename);
                unlink($filename);
                exit;
            break;
            case "file":
                if ($selection["file_format"]) {
                    $new_filename = $selection["file_format"];
                } else {
                    $new_filename = $output_filename;
                }
                if ($selection['append_date']) {
                    $new_filename = date('Y-m-d_H_i') . '_' . $new_filename;
                }
                $path = UPLOAD_DIR . $this->save_path . '/' . $selection['id'] . '/';
                if (!is_dir($path)) {
                    @mkdir($path, 0777, true);
                }
                $new_filename = $path . $new_filename;
                copy($filename, $new_filename);
                unlink($filename);
            break;
        }
    }

    /**
     * Return seo settings of export module
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    public function get_seo_settings($method = "", $lang_id = "")
    {
        if (!empty($method)) {
            return $this->_get_seo_settings($method, $lang_id);
        } else {
            $actions = array("index", "advanced");
            $return = array();
            foreach ($actions as $action) {
                $return[$action] = $this->_get_seo_settings($action, $lang_id);
            }

            return $return;
        }
    }

    /**
     * Return seo settings of export module (internal)
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    protected function _get_seo_settings($method, $lang_id = "")
    {
        if ($method == "index") {
            return array(
                "templates" => array("id"),
                "url_vars"  => array(
                    "id" => array("id" => "numeric"),
                ),
            );
        } elseif ($method == "advanced") {
            return array(
                "templates" => array("id"),
                "url_vars"  => array(
                    "id" => array("id" => "numeric"),
                ),
            );
        }
    }

    /**
     * Transform seo values of query parameters
     *
     * @param string $var_name_from variable name from query string
     * @param string $var_name_to   variable name of method
     * @param mixed  $value         parameter value
     *
     * @return mixed
     */
    public function request_seo_rewrite($var_name_from, $var_name_to, $value)
    {
        $user_data = array();

        if ($var_name_from == $var_name_to) {
            return $value;
        }

        if ($var_name_to == "id") {
            return $user_data["id"];
        }
    }

    /**
     * Return data for xml sitemap
     *
     * @return array
     */
    public function get_sitemap_xml_urls()
    {
        return array();
    }

    /**
     * Return data for site map page
     *
     * @return array
     */
    public function get_sitemap_urls()
    {
        return array();
    }

    /**
     * Generate data of active selections by cron
     *
     * @return void
     */
    public function cron_export()
    {
        $subscriptions_list = $this->get_selections_list(array("active" => 1));
        foreach ($subscriptions_list as $key => $value) {
            if ($value["scheduler"]["type"] == "manual" ||
               $value["scheduler"]["date_for_cron"] == 0 ||
               $value["scheduler"]["date_for_cron"] > time()) {
                continue;
            }

            $this->generate_selection($value["id"], $value["admin_form_data"]);
            switch ($value["scheduler"]["type"]) {
                case "intime":
                    $value["scheduler"]["date_for_cron"] = 0;
                break;
                case "cron":
                    $d = $value["scheduler"]["date_for_cron"];
                    switch ($value["scheduler"]["period"]) {
                        case "day":
                            $value["scheduler"]["date_for_cron"] = mktime(date("H", $d), date("i", $d), date("s", $d), date("n", $d), date("j", $d) + 1, date("Y", $d));
                        break;
                        case "week":
                            $value["scheduler"]["date_for_cron"] = mktime(date("H", $d), date("i", $d), date("s", $d), date("n", $d), date("j", $d) + 7, date("Y", $d));
                        break;
                        case "month":
                            $value["scheduler"]["date_for_cron"] = mktime(date("H", $d), date("i", $d), date("s", $d), date("n", $d) + 1, date("j", $d), date("Y", $d));
                        break;
                    }
                break;
            }

            $data = array("scheduler" => $value["scheduler"]);
            $validate_data = $this->validate_selection($value["id"], $data);
            if (!empty($validate_data['errors'])) {
                continue;
            }
            $this->save_selection($value["id"], $validate_data['data']);
        }
    }
}
