<?php

namespace Pg\Modules\Field_editor\Models\Fields;

/**
 * Field editor module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Checkbox field model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Field editor
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Checkbox_field_model extends Field_type_model
{

    /**
     * Field definition
     *
     * @var array
     */
    public $base_field_param = array(
        'type' => 'TINYINT',
        'constraint' => 3,
        'null' => false,
        'default' => 0,
    );

    /**
     * Default value configuration
     *
     * @var array
     */
    public $manage_field_param = array(
        'default_value' => array('type' => 'bool', "default" => false),
    );

    /**
     * Settings of form field
     *
     * @var array
     */
    public $form_field_settings = array();

    /**
     * Format form field
     *
     * @param array   $field   field data
     * @param boolean $content field value
     *
     * @return array
     */
    public function format_form_fields($field, $content)
    {
        $field["value"] = $content;

        return $field;
    }

    /**
     * Format field value for used in view
     *
     * @param array $settings field options
     * @param array $field    field data
     * @param mixed $value    field value
     *
     * @return array
     */
    public function format_view_fields($settings, $field, $value)
    {
        $field["value"] = $value;
        $field["value_str"] = $value ? l('yes_str', 'field_editor') : l('no_str', 'field_editor');

        return $field;
    }

    /**
     * Format field value for used in fulltext search
     *
     * @param array   $settings field settings
     * @param array   $field    field data
     * @param boolean $value    field value
     *
     * @return array
     */
    public function format_fulltext_fields($settings, $field, $value)
    {
        $return = (!empty($value)) ? $field["name"] . "; " : '';

        return $return;
    }

    /**
     * Validate field data
     *
     * @param array   $settings field settings
     * @param boolean $value    field value
     *
     * @return array
     */
    public function validate_field($settings, $value)
    {
        $return = array("errors" => array(), "data" => $value);
        $return["data"] = ($return["data"]) ? 1 : 0;

        return $return;
    }

    /**
     * Return criteria for used in search by filed
     *
     * @param array  $field    field data
     * @param array  $settings field settings
     * @param array  $data     search data
     * @param string $prefix   field name prefix
     *
     * @return array
     */
    public function get_search_field_criteria($field, $settings, $data, $prefix)
    {
        $criteria = array();
        $gid = $field['gid'];
        if (!empty($data[$gid]) && $data[$gid] == 1) {
            $criteria["where"][$prefix . $gid] = 1;
        }

        return $criteria;
    }

    /**
     * Save field option (not available)
     *
     * @param array  $field      field settings
     * @param string $option_gid option guid
     * @param string $data       option data
     *
     * @return void
     */
    public function set_field_option($field, $option_gid, $data)
    {
        return;
    }

    /**
     * Remove field options (not available)
     *
     * @param array  $field      field settings
     * @param string $option_gid option guid
     *
     * @return void
     */
    public function delete_field_option($field, $option_gid)
    {
        return;
    }

    /**
     * Resort field options (not available)
     *
     * @param array $field       field settings
     * @param array $sorter_data sorting data
     *
     * @return void
     */
    public function sorter_field_option($field, $sorter_data)
    {
        return;
    }

}
