<?php

namespace Pg\Modules\Geomap\Models;

/**
 * Geomap module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Google maps driver model
 *
 * version 3
 *
 * @package 	PG_RealEstate
 * @subpackage 	Geomap
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Googlemapsv3_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Constructor
     *
     * @return Googlemapsv3_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return code of map object
     *
     * @param string  $key           driver key
     * @param array   $settings      map settings
     * @param array   $view_settings view settings
     * @param array   $markers       markers data
     * @param integer $map_id        map identifier
     *
     * @return string
     */
    public function create_html($key, $settings, $view_settings, $markers = array(), $map_id = false)
    {
        $amenities = $this->ci->pg_language->ds->get_reference(
            'geomap', 'amenities_googlemapsv3', $this->ci->pg_language->current_lang_id
        );
        $amenities_names = array();
        foreach ($amenities['option'] as $amenity_code => $amenity_name) {
            $amenities_names[] = "'" . str_replace("'", "\'", $amenity_code) . "':'" . str_replace("'", "\'", $amenity_name) . "'";
        }

        $this->ci->view->assign('amenities_names_str', '{' . implode(',', $amenities_names) . '}');
        $this->ci->view->assign('map_reg_key', $key);
        $this->ci->view->assign('settings', $settings);
        $this->ci->view->assign('view_settings', $view_settings);
        foreach ($markers as $key => $value) {
            if (!empty($value['info'])) {
                $markers[$key]['info'] = @json_encode($value['info']);
            } else {
                $markers[$key]['info'] = '';
            }
        }
        $this->ci->view->assign('markers', $markers);

        $this->ci->view->assign('map_id', $map_id);
        $langs_file = APPPATH . 'modules/geomap/install/languages' . EXT;
        if (file_exists($langs_file)) {
            include($langs_file);
            $lang_code = $this->ci->pg_language->languages[$this->ci->pg_language->current_lang_id]['code'];
            if (isset($languages['google'][$lang_code])) {
                $this->ci->view->assign('lang_code', $languages['google'][$lang_code]);
            }
        }

        $this->ci->view->assign('rand', rand(100000, 999999));
        
        return $this->ci->view->fetch('googlemapsv3_html', null, 'geomap');
    }

    /**
     * Return code for updating map object
     *
     * @param integer $map_id  map identifier
     * @param array   $markers markers data
     *
     * @return string
     */
    public function update_html($map_id, $markers = array())
    {
        $this->ci->view->assign('map_id', $map_id);
        $this->ci->view->assign('markers', $markers);

        return $this->ci->view->fetch('googlemapsv3_update', null, 'geomap');
    }

    /**
     * Return code of geocoder object
     *
     * @param string $key driver key
     *
     * @return string
     */
    public function create_geocoder($key)
    {
        if (@include(APPPATH . 'modules/geomap/install/languages' . EXT)) {
            $lang_code = $this->pg_language->languages[$this->pg_language->current_lang_id]['code'];
            if (isset($languages['google'][$lang_code])) {
                $this->ci->view->assign('lang_code', $languages['google'][$lang_code]);
            }
        }
        
        $this->ci->view->assign('map_reg_key', $key);

        return $this->ci->view->fetch('googlemapsv3_geocoder', null, 'geomap');
    }

    /**
     * Return code for appending street view to map
     *
     * @param string $key driver key
     *
     * @return string
     */
    public function create_street_view($key)
    {
        if (@include(APPPATH . 'modules/geomap/install/languages' . EXT)) {
            $lang_code = $this->pg_language->languages[$this->pg_language->current_lang_id]['code'];
            if (isset($languages['google'][$lang_code])) {
                $this->ci->view->assign('lang_code', $languages['google'][$lang_code]);
            }
        }
        
        $this->ci->view->assign('map_reg_key', $key);

        return $this->ci->view->fetch('googlemapsv3_street_view', null, 'geomap');
    }

}
