<?php

/**
 * Import module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * CSV import driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Import
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Import_csv_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Driver configuration
     *
     * @var array
     */
    public $drivers = array(
        "gid"            => "csv",
        "link"           => "http://mastpoint.curzonnassau.com/csv-1203/",
        "editable"       => 0,
        "is_upload"      => 1,
        "is_integration" => 0,
        "is_logging"     => 0,
        "status"         => 1,
    );

    /**
     * Allowed file extensions
     *
     * @var string
     */
    public $allowed_types = "csv|zip";

    /**
     * Import file extension
     *
     * @var string
     */
    public $extension = "csv";

    /**
     * Rows limit per request
     *
     * @var integer
     */
    public $rows_per_request = 1;

    /**
     * Use two step
     *
     * @param boolean
     */
    public $two_step = false;

    /**
     * Driver settings
     *
     * @var array
     */
    public $settings = array(
        array("name" => "use_first_row", "type" => "checkbox", "default" => "1"),
        array("name" => "delimiter", "type" => "symbol", "default" => ";"),
        array("name" => "enclosure", "type" => "symbol", "default" => "\""),
    );

    /**
     * Driver options
     *
     * @var array
     */
    public $options = array();

    /**
     * Need import data from file
     *
     * @var boolean
     */
    public $need_upload = true;

    /**
     * Constructor
     *
     * @return Import_csv_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
    }

    /**
     * Set element depth for repeating request
     *
     * @return void
     */
    public function set_depth()
    {
    }

    /**
     * Analyze uploaded file
     *
     * @param array  $settings  driver settings
     * @param array  $data      selection data settings
     * @param array  $relations data relations
     * @param string $filename  import filename
     *
     * @return array
     */
    public function analyze($settings, $data = array(), $relations = array(), $filename = null)
    {
        $return = array("errors" => array(), "data" => array());

        $return['data']['total'] = 0;
        $return['data']['position'] = 0;
        $return['data']['relations'] = array();

        $handle = fopen($filename, "rb");

        if (!$data["delimiter"]) {
            $data["delimiter"] = ";";
        }
        if (!$data["enclosure"]) {
            $data["enclosure"] = "\"";
        }

        $first_row = fgetcsv($handle, 0, $data["delimiter"], $data["enclosure"]);

        if ($data["use_first_row"]) {
            foreach ($first_row as $i => $name) {
                if (!empty($relations[$i]) && ($name == $relations[$i]['name'] || empty($name))) {
                    $return['data']['relations'][$i] = $relations[$i];
                } else {
                    $return['data']['relations'][$i] = array("name" => $name, "type" => "", "link" => "");
                }
            }
            $return["data"]["position"] = ftell($handle);
        } else {
            $count = count($first_row);
            for ($i = 0;$i < $count; ++$i) {
                if (!empty($relations[$i])) {
                    $return['data']['relations'][$i] = $relations[$i];
                } else {
                    $return['data']['relations'][$i] = array("name" => "", "type" => "", "link" => "");
                }
            }
            ++$return['data']['total'];
        }

        $str = "";
        while (!feof($handle)) {
            $str = "";
            while ($word = fgets($handle, 4096)) {
                if ($word) {
                    $str = $word;
                }
                if (strpos($str, "\n") !== false) {
                    break;
                }
            }
            if (!$str) {
                continue;
            }
            ++$return['data']['total'];
        }

        fclose($handle);

        return $return;
    }

    /**
     * Parse uploaded file
     *
     * @param string  $filename    import filename
     * @param array   $settings    import settings
     * @param array   $driver      driver data
     * @param array   $relations   field relations
     * @param array   $search_data search criteria
     * @param integer $position    start position
     *
     * @return array
     */
    public function parse($filename, $settings, $driver, $relations, $search_data, $position)
    {
        $return = array("data" => array(), "position" => 0, "status" => 1, "processed" => 0);

        $handle = fopen($filename, "rb");

        if (!$handle) {
            return $return;
        }

        fseek($handle, $position);

        $row = 0;

        while (!feof($handle)) {
            if (++$row > $this->rows_per_request) {
                $return["status"] = 0;
                break;
            }

            $data = fgetcsv($handle, 0, $settings["delimiter"], $settings["enclosure"]);

            if (!empty($data)) {
                $return["data"][] = $data;
                ++$return["processed"];
            }
        }

        $return["position"] = ftell($handle);

        fclose($handle);

        return $return;
    }

    /**
     * Validate selection data
     *
     * @param array  $data       selection data
     * @param string $upload_gid upload gid
     *
     * @return array
     */
    public function validate_data($data, $upload_gid = 'import_file')
    {
        $return = array("errors" => array(), "data" => $data);

        $this->CI->load->helper("upload");

        //// upload src file
        $upload_config = array(
            "allowed_types" => $this->allowed_types,
            "overwrite"     => true,
        );

        $result = validate_file($upload_gid, $upload_config);
        if ($result["error"]) {
            $return["errors"] = $result["error"];
        }

        return $return;
    }

    /**
     * Validate settings for saving to data source
     *
     * @param array $settings settings data
     *
     * @return array
     */
    public function validate_settings($settings)
    {
        $return = array('errors' => array(), 'data' => array());

        return $return;
    }

    /**
     * Connect to driver
     *
     * @param array $settings driver settings
     *
     * @return array
     */
    public function connect($settings)
    {
        $return = array("errors" => array(), "data" => array());

        return $return;
    }

    /**
     * Return rets service metadata
     *
     * @param array $settings driver settings
     *
     * @return array
     */
    public function get_settings($settings, $data = array())
    {
        $return = array();

        return $return;
    }
}
