<?php

namespace Pg\Modules\Listings\Models;

use Pg\Libraries\Setup;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Listings install model
 * 
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	models
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     * 
     * @var object
     */
    protected $ci;
    protected $modules_data = [];

    /**
     * Constructor
     *
     * @return Listings_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->modules_data = Setup::getModuleData('listings', Setup::TYPE_MODULES_DATA);
    }

    /**
     * Check system requirements of module
     * 
     * @return array
     */
    public function _validate_requirements()
    {
        $result = array("data" => array(), "result" => true);

        //check for Mbstring
        $good = function_exists("mb_convert_encoding");
        $result["data"][] = array(
            "name" => "Mbstring extension (required for feeds parsing) is installed",
            "value" => $good ? "Yes" : "No",
            "result" => $good,
        );
        $result["result"] = $result["result"] && $good;

        //check for iconv
        $good = function_exists("iconv");
        $result["data"][] = array(
            "name" => "Iconv extension (required for feeds parsing) is installed",
            "value" => $good ? "Yes" : "No",
            "result" => $good,
        );
        $result["result"] = $result["result"] && $good;
        return $result;
    }

    /**
     * Install data of menu module
     * 
     * @var void
     */
    public function install_menu()
    {
        $this->ci->load->helper("menu");
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            if ($menu_data["action"] != 'none') {
                $this->modules_data['menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], get($menu_data["name"]));
            } else {
                $this->modules_data['menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"]);
            }
            linked_install_process_menu_items($this->modules_data['menu'], "create", $gid, 0, $this->modules_data['menu'][$gid]["items"]);
        }

        if (!empty($this->modules_data['menu_indicators'])) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->modules_data['menu_indicators'] as $data) {
                $this->ci->Indicators_model->save_type(null, $data);
            }
        }
    }

    /**
     * Import languages of menu module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("listings", "menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data");
        } else {
            $this->ci->load->helper("menu");

            foreach ($this->modules_data['menu'] as $gid => $menu_data) {
                linked_install_process_menu_items($this->modules_data['menu'], "update", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_file);
            }

            // Indicators
            if (!empty($this->modules_data['menu_indicators'])) {
                $langs_file = $this->ci->Install_model->language_file_read('listings', 'indicators', $langs_ids);
                if (!$langs_file) {
                    log_message('info', '(listings) Empty indicators langs data');
                    return false;
                } else {
                    $this->ci->load->model('menu/models/Indicators_model');
                    $this->ci->Indicators_model->update_langs($this->modules_data['menu_indicators'], $langs_file, $langs_ids);
                }
            }
        }

        return true;
    }

    /**
     * Export languages of menu module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $this->ci->load->helper("menu");

        $return = array("menu" => array());
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu'], "export", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_ids);
            $return["menu"] = array_merge($return["menu"], $temp);
        }

        if (!empty($this->modules_data['menu_indicators'])) {
            $this->ci->load->model('menu/models/Indicators_model');
            $indicators_langs = $this->ci->Indicators_model->export_langs($this->modules_data['menu_indicators'], $langs_ids);
        }

        return $return;
    }

    /**
     * Uninstall data of menu module
     * 
     * @return void
     */
    public function deinstall_menu()
    {

        $this->ci->load->helper("menu");
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $this->modules_data['menu'][$gid]["items"]);
            }
        }

        if (!empty($this->modules_data['menu_indicators'])) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->modules_data['menu_indicators'] as $data) {
                $this->ci->Indicators_model->delete_type($data['gid']);
            }
        }
    }

    /**
     * Install data of uploads module
     * 
     * @return void
     */
    public function install_uploads()
    {
        // upload config
        $this->ci->load->model("uploads/models/Uploads_config_model");

        $watermark_ids = array();
        $now = date("Y-m-d H:i:s");
        foreach ((array) $this->modules_data['uploads'] as $upload_data) {
            $config_data = array(
                "gid" => $upload_data["gid"],
                "name" => $upload_data["name"],
                "min_height" => isset($upload_data["min_height"]) ? $upload_data["min_height"] : 0,
                "min_width" => isset($upload_data["min_width"]) ? $upload_data["min_width"] : 0,
                "max_height" => $upload_data["max_height"],
                "max_width" => $upload_data["max_width"],
                "max_size" => $upload_data["max_size"],
                "name_format" => $upload_data["name_format"],
                "file_formats" => serialize((array) $upload_data["file_formats"]),
                "default_img" => $upload_data["default_img"],
                "indicator_img" => $upload_data["indicator_img"],
                "date_add" => $now,
            );
            $config_id = $this->ci->Uploads_config_model->save_config(null, $config_data);

            $wm_data = $this->ci->Uploads_config_model->get_watermark_by_gid("image-wm");

            foreach ((array) $upload_data["thumbs"] as $thumb_gid => $thumb_data) {
                if (isset($thumb_data["watermark"])) {
                    if (!isset($watermark_ids[$thumb_data["watermark"]])) {
                        $wm_data = $this->ci->Uploads_config_model->get_watermark_by_gid($thumb_data["watermark"]);
                        $watermark_ids[$thumb_data["watermark"]] = isset($wm_data["id"]) ? $wm_data["id"] : 0;
                    }
                    $watermark_id = $watermark_ids[$thumb_data["watermark"]];
                } else {
                    $watermark_id = 0;
                }

                $thumb_data["config_id"] = $config_id;
                $thumb_data["prefix"] = $thumb_gid;
                $thumb_data["effect"] = "none";
                $thumb_data["watermark_id"] = $watermark_id;

                $validate_data = $this->ci->Uploads_config_model->validate_thumb(null, $thumb_data);
                if (!empty($validate_data["errors"])) {
                    continue;
                }
                $this->ci->Uploads_config_model->save_thumb(null, $validate_data["data"]);
            }
        }
    }

    /**
     * Uninstall data of uploads module
     * 
     * @return void
     */
    public function deinstall_uploads()
    {
        $this->ci->load->model("uploads/models/Uploads_config_model");

        foreach ((array) $this->modules_data['uploads'] as $upload_data) {
            $config_data = $this->ci->Uploads_config_model->get_config_by_gid($upload_data["gid"]);
            if (!empty($config_data["id"])) {
                $this->ci->Uploads_config_model->delete_config($config_data["id"]);
            }
        }
    }

    /**
     * Install data of upload gallery module
     * 
     * @return void
     */
    public function install_upload_gallery()
    {
        $this->ci->load->model("Upload_gallery_model");

        foreach ((array) $this->modules_data['upload_gallery'] as $gallery_data) {
            $validate_data = $this->ci->Upload_gallery_model->validate_type(null, $gallery_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Upload_gallery_model->set_type(null, $validate_data["data"]);
        }
    }

    /**
     * Import languages of upload gallery module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_upload_gallery_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $this->ci->load->model("Upload_gallery_model");

        $langs_file = $this->ci->Install_model->language_file_read("listings", "upload_gallery", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty upload_gallery langs data");
            return false;
        }

        $this->ci->Upload_gallery_model->update_langs($this->modules_data['upload_gallery'], $langs_file, $langs_ids);
        return true;
    }

    /**
     * Export languages of upload gallery module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_upload_gallery_lang_export($langs_ids = null)
    {
        $this->ci->load->model("Upload_gallery_model");
        $langs = $this->ci->Upload_gallery_model->export_langs((array) $this->modules_data['upload_gallery'], $langs_ids);
        return array("upload_gallery" => $langs);
    }

    /**
     * Uninstall data of upload gallery module
     * 
     * @return void
     */
    public function deinstall_upload_gallery()
    {
        $this->ci->load->model("Upload_gallery_model");
        foreach ((array) $this->modules_data['upload_gallery'] as $gallery_data) {
            $type = $this->ci->Upload_gallery_model->get_type_by_gid($gallery_data["gid"]);
            if (!$type)
                continue;
            $this->ci->Upload_gallery_model->delete_type($type["id"]);
        }
    }

    /**
     * Install data of video uploads module
     * 
     * @return void
     */
    public function install_video_uploads()
    {
        ///// add video settings
        $this->ci->load->model("video_uploads/models/Video_uploads_config_model");
        foreach ((array) $this->modules_data['video_uploads'] as $config_data) {
            $validate_data = $this->ci->Video_uploads_config_model->validate_config(null, $config_data);
            if (!empty($validate_data["errors"]))
                continue;
            $validate_data["data"]["gid"] = $config_data["gid"];
            $validate_data["data"]["module"] = $config_data["module"];
            $validate_data["data"]["model"] = $config_data["model"];
            $validate_data["data"]["method_status"] = $config_data["method_status"];
            $this->ci->Video_uploads_config_model->save_config(null, $validate_data["data"]);
        }
    }

    /**
     * Uninstall data of video uploads module
     * 
     * @return void
     */
    public function deinstall_video_uploads()
    {
        // delete video settings
        $this->ci->load->model("video_uploads/models/Video_uploads_config_model");
        foreach ((array) $this->modules_data['video_uploads'] as $config_data) {
            $config_data = $this->ci->Video_uploads_config_model->get_config_by_gid($config_data["gid"]);
            if (empty($config_data["id"])) {
                continue;
            }
            $this->ci->Video_uploads_config_model->delete_config($config_data["id"]);
        }
    }

    /**
     * Install data of file uploads module
     * 
     * @return void
     */
    public function install_file_uploads()
    {
        // add file settings
        $this->ci->load->model("file_uploads/models/File_uploads_config_model");
        foreach ((array) $this->modules_data['file_uploads'] as $config_data) {
            $validate_data = $this->ci->File_uploads_config_model->validate_config(null, $config_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->File_uploads_config_model->save_config(null, $validate_data["data"]);
        }
    }

    /**
     * Uninstall data of file uploads module
     * 
     * @return void
     */
    public function deinstall_file_uploads()
    {
        // delete file settings
        $this->ci->load->model('file_uploads/models/File_uploads_config_model');
        foreach ((array) $this->modules_data['file_uploads'] as $config_data) {
            $config_data = $this->ci->File_uploads_config_model->get_config_by_gid($config_data["gid"]);
            if (empty($config_data["id"]))
                continue;
            $this->ci->File_uploads_config_model->delete_config($config_data["id"]);
        }
    }

    private function installNotifications()
    {
        $this->ci->load->model('notifications/models/Templates_model');
        $now = date('Y-m-d H:i:s');
        $templates_ids = array();
        foreach ((array) $this->modules_data['notifications']['templates'] as $tpl) {
            $template_data = array(
                'gid' => $tpl['gid'],
                'name' => $tpl["name"],
                'vars' => serialize($tpl["vars"]),
                'content_type' => $tpl["content_type"],
                'date_add' => $now,
                'date_update' => $now,
            );
            $validate_data = $this->ci->Templates_model->validate_template(null, $template_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $templates_ids[$tpl['gid']] = $this->ci->Templates_model->save_template(null, $template_data);
        }

        $this->ci->load->model("Notifications_model");
        foreach ((array) $this->modules_data['notifications']['notifications'] as $notification) {
            if (empty($templates_ids[$notification['template']])) {
                continue;
            }
            $notification['id_template_default'] = $templates_ids[$notification['template']];
            unset($notification['template']);
            $validate_data = $this->ci->Notifications_model->validate_notification(null, $notification);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Notifications_model->save_notification(null, $notification);
        }
    }

    /**
     * Install data of subscriptions module
     * 
     * @return void
     */
    public function install_subscriptions()
    {
        $this->ci->load->model("Subscriptions_model");
        $this->ci->load->model("subscriptions/models/Subscriptions_types_model");
        $this->ci->load->model("notifications/models/Templates_model");
        $this->installNotifications();

        foreach ((array) $this->modules_data['subscriptions']["types"] as $type) {
            $subscr_data = array(
                "gid" => $type["gid"],
                "module" => $type["module"],
                "model" => $type["model"],
                "method" => $type["method"],
            );
            $this->ci->Subscriptions_types_model->save_subscriptions_type(null, $subscr_data);
        }
        foreach ((array) $this->modules_data['subscriptions']["subscriptions"] as $subscription) {
            $subscr_type = $this->ci->Subscriptions_types_model->get_subscriptions_type_by_gid($subscription["content_type"]);
            $subscr_template = $this->ci->Templates_model->get_template_by_gid($subscription["template"]);
            $subsc_data = array(
                "gid" => $subscription["gid"],
                "id_template" => $subscr_template["id"],
                "subscribe_type" => $subscription["type"],
                "id_content_type" => $subscr_type["id"],
                "scheduler" => $subscription["scheduler"],
            );
            $this->ci->Subscriptions_model->save_subscription(null, $subsc_data);
        }
    }

    /**
     * Import languages of subscriptions module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_subscriptions_lang_update($langs_ids = null)
    {
        if (empty($langs_ids))
            return false;
        if (!is_array($langs_ids))
            $langs_ids = (array) $langs_ids;
        $this->ci->load->model("Subscriptions_model");
        $this->ci->load->model("Notifications_model");
        $no_data = false;

        $langs_file = $this->ci->Install_model->language_file_read("listings", "notifications", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty notifications langs data");
            $no_data = true;
        }
        $this->ci->Notifications_model->update_langs((array) $this->modules_data['notifications'], $langs_file, $langs_ids);

        $langs_file = $this->ci->Install_model->language_file_read("listings", "subscriptions", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty subscriptions langs data");
            $no_data = true;
        }
        $this->ci->Subscriptions_model->update_langs("listings", (array) $this->modules_data['subscriptions']["subscriptions"], $langs_file, $langs_ids);

        return !$no_data;
    }

    /**
     * Export languages of subscriptions module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_subscriptions_lang_export($langs_ids = null)
    {
        if (!is_array($langs_ids))
            $langs_ids = (array) $langs_ids;
        $this->ci->load->model("Subscriptions_model");
        $this->ci->load->model("Notifications_model");

        $langs["notifications"] = $this->ci->Notifications_model->export_langs((array)$this->modules_data['notifications'], $langs_ids);
        $langs["subscriptions"] = $this->ci->Subscriptions_model->export_langs((array) $this->modules_data['subscriptions']["subscriptions"], $langs_ids);
        return $langs;
    }

    /**
     * Uninstall data of subscriptions module
     * 
     * @return void
     */
    public function deinstall_subscriptions()
    {
        $this->ci->load->model("Subscriptions_model");
        $this->ci->load->model("subscriptions/models/Subscriptions_types_model");
        $this->ci->load->model("notifications/models/Templates_model");

        foreach((array)$this->modules_data['notifications']["templates"] as $tpl){
          $this->ci->Templates_model->delete_template_by_gid($tpl["gid"]);
          } 
        foreach ((array) $this->modules_data['subscriptions']["types"] as $type) {
            $this->ci->Subscriptions_types_model->delete_subscriptions_type_by_gid($type["gid"]);
        }
        foreach ((array) $this->modules_data['subscriptions']["subscriptions"] as $subscription) {
            $this->ci->Subscriptions_model->delete_subscription_by_gid($subscription["gid"]);
        }
        foreach ((array) $this->modules_data['subscriptions']["subscriptions"] as $subscription) {
            $this->ci->Subscriptions_model->delete_subscription_by_gid($subscription["gid"]);
        }
    }

    /**
     * Install data of moderation module
     * 
     * @return void
     */
    public function install_moderation()
    {
        $this->ci->load->model('moderation/models/Moderation_type_model');
        foreach ($this->modules_data['moderation_types'] as $mtype) {
            $mtype['date_add'] = date("Y-m-d H:i:s");
            $this->ci->Moderation_type_model->save_type(null, $mtype);
        }
    }

    /**
     * Import languages of moderation module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_moderation_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read('listings', 'moderation', $langs_ids);
        if (!$langs_file) {
            log_message('info', 'Empty moderation langs data');
        } else {
            $this->ci->load->model('moderation/models/Moderation_type_model');
            $this->ci->Moderation_type_model->update_langs($this->modules_data['moderation_types'], $langs_file);
        }

        return true;
    }

    /**
     * Export languages of moderation module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_moderation_lang_export($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $this->ci->load->model('moderation/models/Moderation_type_model');
        $moderation_types = $this->ci->Moderation_type_model->export_langs($this->modules_data['moderation_types'], $langs_ids);

        return array('moderation' => $moderation_types);
    }

    /**
     * Uninstall data of moderation module
     * 
     * @return void
     */
    public function deinstall_moderation()
    {
        $this->ci->load->model('moderation/models/Moderation_type_model');
        foreach ($this->modules_data['moderation_types'] as $mtype) {
            $type = $this->ci->Moderation_type_model->get_type_by_name($mtype["name"]);
            $this->ci->Moderation_type_model->delete_type($type['id']);
        }
    }

    /**
     * Install data of menu & moderation
     * 
     * @return void
     */
    public function install_moderation_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['moderation_menu'] as $gid => $menu_data) {
            if ($menu_data["action"] != 'none') {
                $this->modules_data['moderation_menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            } else {
                $this->modules_data['moderation_menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"]);
            }
            linked_install_process_menu_items($this->modules_data['moderation_menu'], "create", $gid, 0, $menu_data["items"]);
        }
    }

    /**
     * Import languages of menu & moderation
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_moderation_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("listings", "moderation_menu", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty menu langs data");
        } else {
            $this->ci->load->helper("menu");

            foreach ($this->modules_data['moderation_menu'] as $gid => $menu_data) {
                linked_install_process_menu_items($this->modules_data['moderation_menu'], "update", $gid, 0, $menu_data["items"], $gid, $langs_file);
            }
        }

        return true;
    }

    /**
     * Export languages of menu & moderation
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_moderation_menu_lang_export($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $moderation_menu = array();

        $this->ci->load->helper("menu");

        foreach ($this->modules_data['moderation_menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['moderation_menu'], "export", $gid, 0, $menu_data["items"], $gid, $langs_ids);
            $moderation_menu = array_merge($moderation_menu, $temp);
        }

        return array('moderation_menu' => $moderation_menu);
    }

    /**
     * Uninstall data of menu & moderation
     * 
     * @return void
     */
    public function deinstall_moderation_menu()
    {
        $this->ci->load->helper("menu");
        foreach ($this->modules_data['moderation_menu'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $menu_data["items"]);
            }
        }
    }

    /**
     * Install data of moderators module
     * 
     * @return void
     */
    public function install_moderators()
    {
        // install moderators permissions
        $this->ci->load->model("Moderators_model");

        foreach ((array) $this->modules_data['moderators'] as $method_data) {
            //$validate_data = $this->ci->Moderators_model->validate_method($method_data, true);
            $validate_data = array("errors" => array(), "data" => $method_data);
            if (!empty($validate_data["errors"]))
                continue;
            $this->ci->Moderators_model->save_method(null, $validate_data["data"]);
        }
    }

    /**
     * Import languages of moderators module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        if (!is_array($langs_ids))
            $langs_ids = (array) $langs_ids;
        $langs_file = $this->ci->Install_model->language_file_read("listings", "moderators", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty moderators langs data");
            return false;
        }

        // install moderators permissions
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "listings";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method["method"]])) {
                $this->ci->Moderators_model->save_method($method["id"], array(), $langs_file[$method["method"]]);
            }
        }
        return true;
    }

    /**
     * Export languages of moderators module
     * 
     * @param array $langs_ids languages identfiers
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        if (!is_array($langs_ids))
            $langs_ids = (array) $langs_ids;
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "listings";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method["method"]] = $method["langs"];
        }
        return array('moderators' => $return);
    }

    /**
     * Uninstall data of moderators module
     * 
     * @return void
     */
    public function deinstall_moderators()
    {
        $this->ci->load->model("Moderators_model");
        $params = array();
        $params["where"]["module"] = "listings";
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install data of field editor module
     * 
     * @return void
     */
    public function install_field_editor()
    {
        $this->ci->load->model("Listings_model");
        $this->ci->load->model("Field_editor_model");
        $this->ci->load->model("field_editor/models/Field_editor_forms_model");

        foreach ($this->modules_data['editor_types'] as $editor_type_gid) {
            include MODULEPATH . 'listings/install/listings_fields_' . $editor_type_gid . '_data.php';
            $this->ci->Field_editor_model->import_type_structure($editor_type_gid, $fe_sections, $fe_fields, $fe_forms);
            unset($fe_sections);
            unset($fe_fields);
            unset($fe_form);
        }

        return true;
    }

    /**
     * Import languages of field editor module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_field_editor_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("listings", "field_editor", $langs_ids);
        if (!$langs_file) {
            log_message('info', 'Empty field editor langs data');
            return false;
        }
        $this->ci->load->model('Field_editor_model');
        foreach ($this->modules_data['editor_types'] as $editor_type_gid) {
            include MODULEPATH . 'listings/install/listings_fields_' . $editor_type_gid . '_data.php';
            $this->ci->Field_editor_model->initialize($editor_type_gid);
            $this->ci->Field_editor_model->update_sections_langs($fe_sections, $langs_file);
            $this->ci->Field_editor_model->update_fields_langs($editor_type_gid, $fe_fields, $langs_file);
        }

        return true;
    }

    /**
     * Export languages of field editor module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_field_editor_lang_export($langs_ids = null)
    {
        $this->ci->load->model('Field_editor_model');

        $sections = $fields = array();

        foreach ($this->modules_data['editor_types'] as $editor_type_gid) {
            $this->ci->Field_editor_model->initialize($editor_type_gid);
            list($fe_sections, $fe_fields, $fe_forms) = $this->ci->Field_editor_model->export_type_structure($editor_type_gid, 'application/modules/listings/install/listings_fields_' . $editor_type_gid . '_data.php');
            if (!empty($fe_sections))
                $sections = array_merge($sections, $this->ci->Field_editor_model->export_sections_langs($fe_sections, $langs_ids));
            if (!empty($fe_fields))
                $fields = array_merge($fields, $this->ci->Field_editor_model->export_fields_langs($editor_type_gid, $fe_fields, $langs_ids));
        }

        return array('field_editor' => array_merge($sections, $fields));
    }

    /**
     * Uninstall data of field editor module
     * 
     * @return void
     */
    public function deinstall_field_editor()
    {
        $this->ci->load->model("Field_editor_model");
        $this->ci->load->model("field_editor/models/field_editor_forms_model");

        foreach ($this->modules_data['editor_types'] as $property_type_gid) {
            $this->ci->Field_editor_model->initialize($property_type_gid);

            list($fe_sections, $fe_fields, $fe_forms) = $this->ci->Field_editor_model->export_type_structure($property_type_gid, 'application/modules/listings/install/listings_fields_' . $property_type_gid . '_data.php');

            foreach ((array) $fe_forms as $form_gid => $form_data) {
                if ($form_data['data']["editor_type_gid"] != $property_type_gid)
                    continue;
                $this->ci->Field_editor_forms_model->delete_form_by_gid($form_data['data']['gid']);
            }

            foreach ((array) $fe_fields as $field_gid => $field_data) {
                if ($field_data['data']["editor_type_gid"] != $property_type_gid)
                    continue;
                $this->ci->Field_editor_model->delete_field_by_gid($field_data['data']['gid']);
            }
            foreach ((array) $fe_sections as $section_gid => $section_data) {
                if ($section_data['data']["editor_type_gid"] != $property_type_gid)
                    continue;
                $this->ci->Field_editor_model->delete_section_by_gid($section_data['data']['gid']);
            }
        }
    }

    /**
     * Install data of services module
     * 
     * @return void
     */
    public function install_services()
    {
        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $validate_data = $this->ci->Services_model->validate_template(null, $template_data);
            if (!empty($validate_data["errors"]))
                continue;
            $this->ci->Services_model->save_template(null, $validate_data["data"]);
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $validate_data = $this->ci->Services_model->validate_service(null, $service_data);
            if (!empty($validate_data["errors"]))
                continue;
            $this->ci->Services_model->save_service(null, $validate_data["data"]);
        }

        $this->ci->Services_model->add_icons();
    }

    /**
     * Import languages of services module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_services_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("listings", "services", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty services langs data");
        } else {
            $services_data = array(
                "service" => array(),
                "template" => array(),
                "param" => array(),
            );

            $this->ci->load->model("Services_model");

            foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
                $data_admin = isset($template_data["data_admin"]) ? array_keys($template_data["data_admin"]) : array();
                $data_user = isset($template_data["data_user"]) ? array_keys($template_data["data_user"]) : array();
                $data_membership = isset($template_data["data_membership"]) ? array_keys($template_data["data_membership"]) : array();
                $services_data["template"][] = $template_data["gid"];
                $services_data["param"][$template_data["gid"]] = array_unique(array_merge($data_admin, $data_user, $data_membership));
            }

            foreach ((array) $this->modules_data['services']["services"] as $service_data) {
                $services_data["service"][] = $service_data["gid"];
            }

            $this->ci->Services_model->update_langs($services_data, $langs_file);
        }
    }

    /**
     * Export languages of services module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_services_lang_export($langs_ids = null)
    {
        $services_data = array(
            "service" => array(),
            "template" => array(),
            "param" => array(),
        );

        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $data_admin = isset($template_data["data_admin"]) ? array_keys($template_data["data_admin"]) : array();
            $data_user = isset($template_data["data_user"]) ? array_keys($template_data["data_user"]) : array();
            $data_membership = isset($template_data["data_membership"]) ? array_keys($template_data["data_membership"]) : array();
            $services_data["template"][] = $template_data["gid"];
            $services_data["param"][$template_data["gid"]] = array_merge($data_admin, $data_user, $data_membership);
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $services_data["service"][] = $service_data["gid"];
        }

        $services = $this->ci->Services_model->export_langs($services_data, $langs_ids);

        return array("services" => $services,);
    }

    /**
     * Uninstall data of services module
     * 
     * @return void
     */
    public function deinstall_services()
    {
        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $this->ci->Services_model->delete_template_by_gid($template_data["gid"]);
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $this->ci->Services_model->delete_service_by_gid($service_data["gid"]);
        }
    }

    /**
     * Install data of notifications & services
     * 
     * @return void
     */
    public function install_notifications_services()
    {
        // add notification
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        $templates_ids = array();

        if (!empty($this->modules_data['notifications_services']["templates"])) {
            foreach ((array) $this->modules_data['notifications_services']["templates"] as $template_data) {
                if (is_array($template_data["vars"])) {
                    $template_data["vars"] = implode(",", $template_data["vars"]);
                }
                $validate_data = $this->ci->Templates_model->validate_template(null, $template_data);
                if (!empty($validate_data["errors"])) {
                    continue;
                }
                $templates_ids[$template_data['gid']] = $this->ci->Templates_model->save_template(null, $validate_data["data"]);
            }
        }
        if (!empty($this->modules_data['notifications_services']["notifications"])) {
            foreach ((array) $this->modules_data['notifications_services']["notifications"] as $notification_data) {
                if (!isset($templates_ids[$notification_data["template"]])) {
                    $template = $this->ci->Templates_model->get_template_by_gid($notification_data["template"]);
                    $templates_ids[$notification_data["template"]] = $template["id"];
                }
                $notification_data["id_template_default"] = $templates_ids[$notification_data["template"]];
                $validate_data = $this->ci->Notifications_model->validate_notification(null, $notification_data);
                if (!empty($validate_data["errors"])) {
                    continue;
                }
                $this->ci->Notifications_model->save_notification(null, $validate_data["data"]);
            }
        }
    }

    /**
     * Import languages of notifications & services
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_notifications_services_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("listings", "notifications_services", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty notifications langs data");
        } else {
            $this->ci->Notifications_model->update_langs($this->modules_data['notifications_services'], $langs_file, $langs_ids);
        }
    }

    /**
     * Export languages of notifications & services
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_notifications_services_lang_export($langs_ids = null)
    {
        $this->ci->load->model("Notifications_model");
        $langs = $this->ci->Notifications_model->export_langs((array) $this->modules_data['notifications_services'], $langs_ids);

        return array("notifications_services" => $langs);
    }

    /**
     * Uninstall data of notifications & services
     * 
     * @return void
     */
    public function deinstall_notifications_services()
    {
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        foreach ((array) $this->modules_data['notifications_services']["notifications"] as $notification_data) {
            $this->ci->Notifications_model->delete_notification_by_gid($notification_data["gid"]);
        }

        foreach ((array) $this->modules_data['notifications_services']["templates"] as $template_data) {
            $this->ci->Templates_model->delete_template_by_gid($template_data["gid"]);
        }
    }

    /**
     * Install data of dynamic blocks module
     * 
     * @return void
     */
    public function install_dynamic_blocks()
    {
        $this->ci->load->model("Dynamic_blocks_model");

        $area_ids = array();

        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {

            $validate_data = $this->ci->Dynamic_blocks_model->validate_block(null, $block_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }

            $id_block = $this->ci->Dynamic_blocks_model->save_block(null, $validate_data["data"]);

            if (!empty($block_data["area"])) {
                if (!isset($area_ids[$block_data["area"]["gid"]])) {
                    $area = $this->ci->Dynamic_blocks_model->get_area_by_gid($block_data["area"]["gid"]);
                    $area_ids[$block_data["area"]["gid"]] = $area["id"];
                }

                // index area
                $block_data["area"]["id_area"] = $area_ids[$block_data["area"]["gid"]];
                $block_data["area"]["id_block"] = $id_block;

                $validate_data = $this->ci->Dynamic_blocks_model->validate_area_block(null, $block_data["area"]);
                if (!empty($validate_data["errors"])) {
                    continue;
                }

                $this->ci->Dynamic_blocks_model->save_area_block(null, $validate_data["data"]);
            }

            if (!empty($block_data["presets"])) {
                foreach ($block_data["presets"] as $preset_data) {
                    $preset = $this->ci->Dynamic_blocks_model->get_preset_by_gid($preset_data['gid_preset']);
                    $preset_data['id_preset'] = $preset['id'];
                    $preset_data['id_block'] = $id_block;
                    $validate_data = $this->ci->Dynamic_blocks_model->validate_preset_block(null, $preset_data);
                    if (!empty($validate_data["errors"])) {
                        continue;
                    }
                    $this->ci->Dynamic_blocks_model->save_preset_block(null, $validate_data["data"]);
                }
            }
        }
    }

    /**
     * Import languages of dynamic blocks module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_dynamic_blocks_lang_update($langs_ids = null)
    {
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }

        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("listings", "dynamic_blocks", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty dynamic_blocks langs data");
        } else {
            $this->ci->load->model("Dynamic_blocks_model");

            $data = array();

            foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
                $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
                if (!$block) {
                    continue;
                }
                $data[] = $block;
            }

            $this->ci->Dynamic_blocks_model->update_langs($data, $langs_file, $langs_ids);
        }

        return true;
    }

    /**
     * Export languages of dynamic blocks module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_dynamic_blocks_lang_export($langs_ids = null)
    {
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }

        $this->ci->load->model("Dynamic_blocks_model");
        $data = array();
        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
            if (!$block) {
                continue;
            }
            $data[] = $block;
        }
        $langs = $this->ci->Dynamic_blocks_model->export_langs($data, $langs_ids);

        $return = array("dynamic_blocks" => $langs);

        return $return;
    }

    /**
     * Unistall data of dynamic blocks module
     * 
     * @return void
     */
    public function deinstall_dynamic_blocks()
    {
        $this->ci->load->model("Dynamic_blocks_model");
        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $this->ci->Dynamic_blocks_model->delete_block_by_gid($block_data["gid"]);
        }
    }

    /**
     * Install data of dynamic_blocks & services
     * 
     * @return void
     */
    public function install_dynamic_blocks_services()
    {
        $this->ci->load->model("Dynamic_blocks_model");

        $area_ids = array();

        foreach ((array) $this->modules_data['dynamic_blocks_services'] as $block_data) {
            $validate_data = $this->ci->Dynamic_blocks_model->validate_block(null, $block_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }

            $id_block = $this->ci->Dynamic_blocks_model->save_block(null, $validate_data["data"]);

            if (!empty($block_data["area"])) {
                if (!isset($area_ids[$block_data["area"]["gid"]])) {
                    $area = $this->ci->Dynamic_blocks_model->get_area_by_gid($block_data["area"]["gid"]);
                    $area_ids[$block_data["area"]["gid"]] = $area["id"];
                }

                // index area
                $block_data["area"]["id_area"] = $area_ids[$block_data["area"]["gid"]];
                $block_data["area"]["id_block"] = $id_block;

                $validate_data = $this->ci->Dynamic_blocks_model->validate_area_block(null, $block_data["area"]);
                if (!empty($validate_data["errors"])) {
                    continue;
                }
                $this->ci->Dynamic_blocks_model->save_area_block(null, $validate_data["data"]);
            }

            if (!empty($block_data["presets"])) {
                foreach ($block_data["presets"] as $preset_data) {
                    $preset = $this->ci->Dynamic_blocks_model->get_preset_by_gid($preset_data['gid_preset']);
                    $preset_data['id_preset'] = $preset['id'];
                    $preset_data['id_block'] = $id_block;
                    $validate_data = $this->ci->Dynamic_blocks_model->validate_preset_block(null, $preset_data);
                    if (!empty($validate_data["errors"])) {
                        continue;
                    }
                    $this->ci->Dynamic_blocks_model->save_preset_block(null, $validate_data["data"]);
                }
            }
        }

        $block = $this->ci->Dynamic_blocks_model->get_block_by_gid("form_slider_listings_block");
        if (empty($block['id'])) {
            log_message('error', '(listings) Block not found: "form_slider_listings_block" ');
        } else {
            $save_data = array(
                'views' => array(
                    //array("gid" => "featured_980_440"),
                    array("gid" => "featured_1600_440"),
                    //array("gid" => "latest_added_980_440"),
                    array("gid" => "latest_added_1600_440"),
                    //array("gid" => "sale_980_440"),
                    array("gid" => "sale_1600_440"),
                    //array("gid" => "rent_980_440"),
                    array("gid" => "rent_1600_440"),
                )
            );
            $validate = $this->ci->Dynamic_blocks_model->validate_block($block['id'], $save_data);
            if (empty($validate['errors'])) {
                $this->ci->Dynamic_blocks_model->save_block($block['id'], $validate['data']);
            }
        }

        $block = $this->ci->Dynamic_blocks_model->get_block_by_gid("slider_listings_block");
        if (empty($block['id'])) {
            log_message('error', '(listings) Block not found: "slider_listings_block" ');
        } else {
            $save_data = array(
                'views' => array(
//                    array("gid" => "featured_654_395"),
//                    array("gid" => "featured_980_440"),
//                    array("gid" => "latest_added_654_395"),
//                    array("gid" => "latest_added_980_440"),
//                    array("gid" => "sale_654_395"),
//                    array("gid" => "sale_980_440"),
//                    array("gid" => "rent_654_395"),
//                    array("gid" => "rent_980_440"),
                    array("gid" => "featured_1600_440"),
                    array("gid" => "latest_added_1600_440"),
                    array("gid" => "sale_1600_440"),
                    array("gid" => "rent_1600_440"),
                )
            );
            $validate = $this->ci->Dynamic_blocks_model->validate_block($block['id'], $save_data);
            if (empty($validate['errors'])) {
                $this->ci->Dynamic_blocks_model->save_block($block['id'], $validate['data']);
            }
        }
    }

    /**
     * Import languages of dynamic blocks & services
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_dynamic_blocks_services_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("listings", "dynamic_blocks_services", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty dynamic_blocks services langs data");
        } else {
            $this->ci->load->model("Dynamic_blocks_model");

            $data = array();

            foreach ((array) $this->modules_data['dynamic_blocks_services'] as $block_data) {
                $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
                if (!$block) {
                    continue;
                }
                $data[] = $block;
            }

            $this->ci->Dynamic_blocks_model->update_langs($data, $langs_file, $langs_ids);

            $this->ci->Dynamic_blocks_model->update_view_langs(
                'listings', 'form_slider_listings_block', 'featured_980_440', $langs_file, $langs_ids);

            $this->ci->Dynamic_blocks_model->update_view_langs(
                'listings', 'form_slider_listings_block', 'featured_1600_440', $langs_file, $langs_ids);

            $this->ci->Dynamic_blocks_model->update_view_langs(
                'listings', 'slider_listings_block', 'featured_654_395', $langs_file, $langs_ids);

            $this->ci->Dynamic_blocks_model->update_view_langs(
                'listings', 'slider_listings_block', 'featured_1600_440', $langs_file, $langs_ids);
        }
    }

    /**
     * Export languages of dynamic blocks & services
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_dynamic_blocks_services_lang_export($langs_ids = null)
    {
        $this->ci->load->model("Dynamic_blocks_model");

        $data = array();
        foreach ((array) $this->modules_data['dynamic_blocks_services'] as $block_data) {
            $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
            if (!$block) {
                continue;
            }
            $data[] = $block;
        }

        $langs = $this->ci->Dynamic_blocks_model->export_langs($data, $langs_ids);

        $langs = array_merge($langs, $this->ci->Dynamic_blocks_model->export_view_langs(
                'listings', 'form_slider_listings_block', 'featured_980_440', $langs_ids));

        $langs = array_merge($langs, $this->ci->Dynamic_blocks_model->export_view_langs(
                'listings', 'form_slider_listings_block', 'featured_1600_440', $langs_ids));

        $langs = array_merge($langs, $this->ci->Dynamic_blocks_model->export_view_langs(
                'listings', 'slider_listings_block', 'featured_654_395', $langs_ids));

        $langs = array_merge($langs, $this->ci->Dynamic_blocks_model->export_view_langs(
                'listings', 'slider_listings_block', 'featured_1600_440', $langs_ids));

        return array("dynamic_blocks_services" => $langs);
    }

    /**
     * Uninstall data of dynamic blocks & services
     * 
     * @return void
     */
    public function deinstall_dynamic_blocks_services()
    {
        $this->ci->load->model('Dynamic_blocks_model');
        foreach ((array) $this->modules_data['dynamic_blocks_services'] as $block_data) {
            $this->ci->Dynamic_blocks_model->delete_block_by_gid($block_data["gid"]);
        }

        $this->ci->Dynamic_blocks_model->delete_view_langs(
            'listings', 'form_slider_listings_block', 'featured_980_440');

        $this->ci->Dynamic_blocks_model->delete_view_langs(
            'listings', 'form_slider_listings_block', 'featured_1600_440');

        $this->ci->Dynamic_blocks_model->delete_view_langs(
            'listings', 'slider_listings_block', 'featured_654_395');

        $this->ci->Dynamic_blocks_model->delete_view_langs(
            'listings', 'slider_listings_block', 'featured_1600_440');
    }

    /**
     * Install data of export module
     * 
     * @return void
     */
    public function install_export()
    {
        $this->ci->load->model("export/models/Export_module_model");

        $validate_data = $this->ci->Export_module_model->validate_module(null, (array) $this->modules_data['export_data']);
        if (!empty($validate_data["errors"])) {
            return;
        }
        $this->ci->Export_module_model->save_module(null, $validate_data["data"]);
    }

    /**
     * Import languages of export module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_export_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("listings", "export", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty export langs data");
            return false;
        }

        $this->ci->load->model("export/models/Export_module_model");
        $this->ci->Export_module_model->update_lang(array("module_listings"), $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export languages of export module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_export_lang_export($langs_ids = null)
    {
        $this->ci->load->model("export/models/Export_module_model");
        $langs = $this->ci->Export_module_model->export_lang(array("module_listings"), $langs_ids);
        return array("export" => $langs);
    }

    /**
     * Unistall data of export module
     * 
     * @return void
     */
    public function deinstall_export()
    {
        $this->ci->load->model("export/models/Export_module_model");
        $this->ci->Export_module_model->remove_module("listings");
    }

    /**
     * Install data of import module
     * 
     * @return void
     */
    public function install_import()
    {
        $this->ci->load->model("import/models/Import_module_model");

        $validate_data = $this->ci->Import_module_model->validate_module(null, (array) $this->modules_data['import_data']);
        if (!empty($validate_data["errors"])) {
            return;
        }
        $this->ci->Import_module_model->save_module(null, $validate_data["data"]);
    }

    /**
     * Import languages of import module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_import_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("listings", "import", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty import langs data");
            return false;
        }

        $this->ci->load->model("import/models/Import_module_model");
        $this->ci->Import_module_model->update_lang(array("module_listings"), $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export languages of import module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_import_lang_export($langs_ids = null)
    {
        $this->ci->load->model("import/models/Import_module_model");

        $langs = $this->ci->Import_module_model->export_lang(array("module_listings"), $langs_ids);
        return array("import" => $langs);
    }

    /**
     * Unistall data of import module
     * 
     * @return void
     */
    public function deinstall_import()
    {
        $this->ci->load->model("import/models/Import_module_model");
        $this->ci->Import_module_model->remove_module("listings");
    }

    /**
     * Install data of site map module
     * 
     * @return void
     */
    public function install_site_map()
    {
        // site_map
        $this->ci->load->model("Site_map_model");
        $this->ci->Site_map_model->set_sitemap_module("listings", $this->modules_data['site_map_data']);
    }

    /**
     * Uninstall data of site map module
     * 
     * @return void
     */
    public function deinstall_site_map()
    {
        $this->ci->load->model("Site_map_model");
        $this->ci->Site_map_model->delete_sitemap_module("listings");
    }

    /**
     * Install data of geomap module
     * 
     * @return void
     */
    public function install_geomap()
    {
        //add geomap settings
        $this->ci->load->model("geomap/models/Geomap_settings_model");

        foreach ((array) $this->modules_data['geomap'] as $settings) {
            $map_gid = $settings["map_gid"];
            unset($settings["map_gid"]);

            $id_user = intval($settings["id_user"]);
            unset($settings["id_user"]);

            $id_object = intval($settings["id_object"]);
            unset($settings["id_object"]);

            $gid = $settings["gid"];
            unset($settings["gid"]);

            $validate_data = $this->ci->Geomap_settings_model->validate_settings($settings);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Geomap_settings_model->save_settings($map_gid, $id_user, $id_object, $gid, $validate_data["data"]);
        }
    }

    /**
     * Import languages of geomap module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_geomap_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("listings", "geomap", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty geomap langs data");
            return false;
        }

        $this->ci->load->model("geomap/models/Geomap_settings_model");

        $gids = array();
        foreach ((array) $this->modules_data['geomap'] as $geomap_data) {
            $gids[] = 'map_' . $geomap_data['gid'];
        }
        $this->ci->Geomap_settings_model->update_lang($gids, $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export languages of geomap module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_geomap_lang_export($langs_ids = null)
    {
        $this->ci->load->model("geomap/models/Geomap_settings_model");

        $gids = array();
        foreach ((array) $this->modules_data['geomap'] as $geomap_data) {
            $gids[] = 'map_' . $geomap_data['gid'];
        }
        $langs = $this->ci->Geomap_settings_model->export_lang($gids, $langs_ids);
        return array("geomap" => $langs);
    }

    /**
     * Uninstall data of geomap module
     * 
     * @return void
     */
    public function deinstall_geomap()
    {
        // add geomap settings
        $this->ci->load->model("geomap/models/Geomap_settings_model");

        foreach ((array) $this->modules_data['geomap'] as $settings) {
            $map_gid = $settings["map_gid"];
            unset($settings["data"]["map_gid"]);

            $id_user = $settings["id_user"];
            unset($settings["id_user"]);

            $id_object = $settings["id_object"];
            unset($settings["id_object"]);

            $gid = $settings["gid"];
            unset($settings["gid"]);

            $this->ci->Geomap_settings_model->delete_settings($map_gid, $id_user, $id_object, $gid);
        }
    }

    /**
     * Install data of reviews module
     * 
     * @return void
     */
    public function install_reviews()
    {

        $this->ci->load->model("Listings_model");

        // add reviews type
        $this->ci->load->model("reviews/models/Reviews_type_model");

        $this->ci->Listings_model->install_reviews_fields((array) $this->modules_data['reviews']["reviews_fields"]);

        foreach ((array) $this->modules_data['reviews']["reviews"] as $review_data) {
            $validate_data = $this->ci->Reviews_type_model->validate_type(null, $review_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Reviews_type_model->save_type(null, $validate_data["data"]);
        }
    }

    /**
     * Import languages of reviews module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_reviews_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model("Reviews_model");

        $langs_file = $this->ci->Install_model->language_file_read("listings", "reviews", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty reviews langs data");
            return false;
        }

        foreach ((array) $this->modules_data['reviews']["reviews"] as $review_data) {
            $this->ci->Reviews_model->update_langs($review_data, $langs_file, $langs_ids);
        }

        foreach ($langs_ids as $lang_id) {
            foreach ((array) $this->modules_data['reviews']["rate_types"] as $type_gid => $type_data) {
                $types_data = array();
                foreach ($type_data as $rate_type => $votes) {
                    $votes_data = array();
                    foreach ($votes as $vote) {
                        $votes_data[$vote] = isset($langs_file[$type_gid . '_' . $rate_type . "_votes_" . $vote][$lang_id]) ?
                            $langs_file[$type_gid . '_' . $rate_type . "_votes_" . $vote][$lang_id] : $vote;
                    }
                    $types_data[$rate_type] = array(
                        "header" => $langs_file[$type_gid . '_' . $rate_type . "_header"][$lang_id],
                        "votes" => $votes_data,
                    );
                }
                $this->ci->Reviews_model->add_rate_type($type_gid, $types_data, $lang_id);
            }
        }

        return true;
    }

    /**
     * Export languages of reviews module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_reviews_lang_export($langs_ids)
    {
        if (empty($langs_ids))
            return false;
        $this->ci->load->model("Reviews_model");
        $langs = array();
        foreach ((array) $this->modules_data['reviews']["reviews"] as $review_data) {
            $langs = array_merge($langs, $this->ci->Reviews_model->export_langs($review_data['gid'], $langs_ids));
        }
        return array("reviews" => $langs);
    }

    /**
     * Uninstall data of reviews module
     * 
     * @return void
     */
    public function deinstall_reviews()
    {

        $this->ci->load->model("Listings_model");

        // add reviews type
        $this->ci->load->model("reviews/models/Reviews_type_model");

        foreach ((array) $this->modules_data['reviews']["reviews"] as $review_data) {
            $this->ci->Reviews_type_model->delete_type($review_data["gid"]);
        }

        $this->ci->Listings_model->deinstall_reviews_fields(array_keys((array) $this->modules_data['reviews']["reviews_fields"]));
    }

    /**
     * Install data of spam module
     * 
     * @return void
     */
    public function install_spam()
    {
        // add spam type
        $this->ci->load->model("spam/models/Spam_type_model");

        foreach ((array) $this->modules_data['spam'] as $spam_data) {
            $validate_data = $this->ci->Spam_type_model->validate_type(null, $spam_data);
            if (!empty($validate_data["errors"]))
                continue;
            $this->ci->Spam_type_model->save_type(null, $validate_data["data"]);
        }
    }

    /**
     * Import languages of spam module
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_spam_lang_update($langs_ids = null)
    {
        if (empty($langs_ids))
            return false;

        $this->ci->load->model("spam/models/Spam_type_model");

        $langs_file = $this->ci->Install_model->language_file_read("listings", "spam", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty spam langs data");
            return false;
        }

        $this->ci->Spam_type_model->update_langs($this->modules_data['spam'], $langs_file, $langs_ids);
        return true;
    }

    /**
     * Export languages of spam module
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_spam_lang_export($langs_ids = null)
    {
        $this->ci->load->model("spam/models/Spam_type_model");
        $langs = $this->ci->Spam_type_model->export_langs((array) $this->modules_data['spam'], $langs_ids);
        return array("spam" => $langs);
    }

    /**
     * Uninstall data of spam module
     * 
     * @return void
     */
    public function deinstall_spam()
    {
        // add spam type
        $this->ci->load->model("spam/models/Spam_type_model");

        foreach ((array) $this->modules_data['spam'] as $spam_data) {
            $this->ci->Spam_type_model->delete_type($spam_data["gid"]);
        }
    }

    /**
     * Install data of banners module
     * 
     * @return void
     */
    public function install_banners()
    {
        // add banners module
        $this->ci->load->model("banners/models/Banner_group_model");
        $this->ci->Banner_group_model->set_module("listings", "Listings_model", "_banner_available_pages");
        $this->add_banners();
    }

    /**
     * Import languages of banners module
     * 
     * @return void
     */
    public function install_banners_lang_update()
    {
        $lang_ids = array_keys($this->ci->pg_language->languages);
        $lang_id = $this->ci->pg_language->get_default_lang_id();
        $lang_data[$lang_id] = "Listings pages";
        $this->ci->pg_language->pages->set_string_langs("banners", "banners_group_listings_groups", $lang_data, $lang_ids);
    }

    /**
     * Unistall data of banners module
     * 
     * @return void
     */
    public function deinstall_banners()
    {
        // delete banners module
        $this->ci->load->model("banners/models/Banner_group_model");
        $this->ci->Banner_group_model->delete_module("listings");
        $this->remove_banners();
    }

    /**
     * Add default banners data of listings group
     * 
     * @return void
     */
    public function add_banners()
    {
        $this->ci->load->model("Users_model");
        $this->ci->load->model("banners/models/Banner_group_model");
        $this->ci->load->model("banners/models/Banner_place_model");

        $group_attrs = array(
            "date_created" => date("Y-m-d H:i:s"),
            "date_modified" => date("Y-m-d H:i:s"),
            "price" => 1,
            "gid" => "listings_groups",
            "name" => "Listings pages",
        );
        $group_id = $this->ci->Banner_group_model->create_unique_group($group_attrs);
        $all_places = $this->ci->Banner_place_model->get_all_places();
        if ($all_places) {
            foreach ($all_places as $key => $value) {
                $this->ci->Banner_place_model->save_place_group($value["id"], $group_id);
            }
        }

        // add pages in group
        $this->ci->load->model("Listings_model");
        $pages = $this->ci->Listings_model->_banner_available_pages();

        if ($pages) {
            foreach ($pages as $key => $value) {
                $page_attrs = array(
                    "group_id" => $group_id,
                    "name" => $value["name"],
                    "link" => $value["link"],
                );
                $this->ci->Banner_group_model->add_page($page_attrs);
            }
        }
    }

    /**
     * Remove banners data of listings group
     * 
     * @return void
     */
    public function remove_banners()
    {
        $this->ci->load->model("banners/models/Banner_group_model");
        $group_id = $this->ci->Banner_group_model->get_group_id_by_gid("listings_groups");
        $this->ci->Banner_group_model->delete($group_id);
    }

    /**
     * Install data of social networking module
     * 
     * @return void
     */
    public function install_social_networking()
    {
        // add social netorking page
        $this->ci->load->model('social_networking/models/Social_networking_pages_model');
        foreach ((array) $this->modules_data['social_networking'] as $social_networking_data) {
            $validation_data = $this->ci->Social_networking_pages_model->validate_page(null, $social_networking_data);
            if (!empty($validation_data['errors']))
                continue;
            $this->ci->Social_networking_pages_model->save_page(null, $validation_data['data']);
        }
    }

    /**
     * Uninstall data of social networking module
     * 
     * @return void
     */
    public function deinstall_social_networking()
    {
        // delete social netorking page
        $this->ci->load->model('social_networking/models/Social_networking_pages_model');
        $this->ci->Social_networking_pages_model->delete_pages_by_controller('listings');
    }

    /**
     * Install data of cronjobs module
     * 
     * @return void
     */
    public function install_cronjob()
    {
        // add lift up cronjob
        $this->ci->load->model('Cronjob_model');
        foreach ((array) $this->modules_data['cronjobs'] as $cron_data) {
            $validation_data = $this->ci->Cronjob_model->validate_cron(null, $cron_data);
            if (!empty($validation_data['errors']))
                continue;
            $this->ci->Cronjob_model->save_cron(null, $validation_data['data']);
        }
    }

    /**
     * Uninstall data of cronjobs module
     * 
     * @return void
     */
    public function deinstall_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $cron_data = array();
        $cron_data["where"]["module"] = "listings";
        $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
    }

    /**
     * Install data of cronjob services
     * 
     * @return void
     */
    public function install_cronjob_services()
    {
        // add lift up cronjob
        $this->ci->load->model('Cronjob_model');
        foreach ((array) $this->modules_data['cronjobs_services'] as $cron_data) {
            $validation_data = $this->ci->Cronjob_model->validate_cron(null, $cron_data);
            if (!empty($validation_data['errors'])) {
                continue;
            }
            $this->ci->Cronjob_model->save_cron(null, $validation_data['data']);
        }
    }

    /**
     * Uninstall data of cronjob services
     * 
     * @return void
     */
    public function deinstall_cronjob_services()
    {
        $this->ci->load->model('Cronjob_model');
        foreach ((array) $this->modules_data['cronjobs_services'] as $cron_data) {
            $cron_data = array(
                "where" => array(
                    "module" => $cron_data["module"],
                    "model" => $cron_data["model"],
                    "method" => $cron_data["method"],
                ),
            );
            $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
        }
    }

    /**
     * Install users data to menu & users_private
     * 
     * @return void
     */
    public function install_users_private_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['users_private_menu'] as $gid => $menu_data) {
            if ($menu_data["action"] != 'none') {
                $this->modules_data['users_private_menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            } else {
                $this->modules_data['users_private_menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"]);
            }
            linked_install_process_menu_items($this->modules_data['users_private_menu'], "create", $gid, 0, $menu_data["items"]);
        }
    }

    /**
     * Import users languages to menu & users_private
     * 
     * @param array $langs_ids languages identifiers
     * @return boolean
     */
    public function install_users_private_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("listings", "users_private_menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data for users_private (users)");
            return false;
        }

        $this->ci->load->helper("menu");

        foreach ($this->modules_data['users_private_menu'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['users_private_menu'], "update", $gid, 0, $menu_data["items"], $gid, $langs_file);
        }

        return true;
    }

    /**
     * Export users languages from menu & users_private
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function install_users_private_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids))
            return false;
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->modules_data['users_private_menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['users_private_menu'], "export", $gid, 0, $menu_data["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("users_private_menu" => $return);
    }

    /**
     * Uninstall users data from menu & users_private
     * 
     * @return void
     */
    public function deinstall_users_private_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['users_private_menu'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $menu_data["items"]);
            }
        }
    }

    /**
     * Install fields of dedicated languages
     * 
     * @return void
     */
    public function _prepare_installing()
    {
        $this->ci->load->model("listings/models/Wish_list_model");
        foreach ($this->ci->pg_language->languages as $lang_id => $value) {
            $this->ci->Wish_list_model->lang_dedicate_module_callback_add($lang_id);
        }
    }

    /**
     * Install module data
     * 
     * @return void
     */
    public function _arbitrary_installing()
    {
        // add entries for lang data updates
        foreach ($this->modules_data['lang_dm_data'] as $lang_dm_data) {
            $this->ci->pg_language->add_dedicate_modules_entry($lang_dm_data);
        }

        // SEO
        $this->ci->pg_seo->set_seo_module("listings", $this->modules_data['seo_data']);

        // add seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][tpl:1:operation_type:literal:all][text:-index/][postfix:2:order:literal][text:/][postfix:3:order_direction:literal][text:/][postfix:4:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'index', $data);

        $xml_data['listings']['index'] = $this->ci->pg_seo->url_template_transform('listings', 'index', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][tpl:3:operation_type:literal:all][text:-by-][tpl:1:category:literal:all][text:/][tpl:2:property:literal:all][text:/][postfix:3:order:literal][text:/][postfix:4:order_direction:literal][text:/][postfix:5:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'category', $data);

        $xml_data['listings']['category'] = $this->ci->pg_seo->url_template_transform('listings', 'category', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:country-][tpl:1:country:literal:empty][text:/][tpl:2:operation_type:literal:all][text:/][postfix:3:order:literal][text:/][postfix:4:order_direction:literal][text:/][postfix:5:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'country', $data);

        $xml_data['listings']['country'] = $this->ci->pg_seo->url_template_transform('listings', 'country', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:region-][tpl:2:region:literal:empty][text:-of-][tpl:1:country:literal:empty][text:/][tpl:3:operation_type:literal:all][text:/][postfix:4:order:literal][text:/][postfix:5:order_direction:literal][text:/][postfix:6:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'region', $data);

        $xml_data['listings']['region'] = $this->ci->pg_seo->url_template_transform('listings', 'region', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][tpl:4:operation_type:literal:all][text:-in-][tpl:1:country:literal:all][text:/][tpl:2:region:literal:all][text:/][tpl:3:city:literal:all][text:/][postfix:4:order:literal][text:/][postfix:5:order_direction:literal][text:/][postfix:6:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'location', $data);

        $xml_data['listings']['location'] = $this->ci->pg_seo->url_template_transform('listings', 'location', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 0,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][tpl:2:operation_type:literal:all][text:-from-][opt:user:literal:user][text:/][tpl:1:id_user:numeric:0][text:/][postfix:2:order:literal][text:/][postfix:3:order_direction:literal][text:/][postfix:4:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'user', $data);

        $xml_data['listings']['user'] = $this->ci->pg_seo->url_template_transform('listings', 'user', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][tpl:2:operation_type:literal:all][text:-search/][tpl:1:keyword:literal:empty][text:/][postfix:2:order:literal][text:/][postfix:3:order_direction:literal][text:/][postfix:4:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'search', $data);

        $xml_data['listings']['search'] = $this->ci->pg_seo->url_template_transform('listings', 'search', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][tpl:1:operation_type:literal:all][text:-by-open_house][text:/][postfix:2:order:literal][text:/][postfix:3:order_direction:literal][text:/][postfix:4:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'open_house', $data);

        $xml_data['listings']['open_house'] = $this->ci->pg_seo->url_template_transform('listings', 'open_house', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][tpl:1:operation_type:literal:all][text:-by-privates][text:/][postfix:2:order:literal][text:/][postfix:3:order_direction:literal][text:/][postfix:4:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'privates', $data);

        $xml_data['listings']['privates'] = $this->ci->pg_seo->url_template_transform('listings', 'privates', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][tpl:1:operation_type:literal:all][text:-by-agents][text:/][postfix:2:order:literal][text:/][postfix:3:order_direction:literal][text:/][postfix:4:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'agents', $data);

        $xml_data['listings']['agents'] = $this->ci->pg_seo->url_template_transform('listings', 'agents', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][tpl:1:operation_type:literal:all][text:-by-discount][text:/][postfix:2:order:literal][text:/][postfix:3:order_direction:literal][text:/][postfix:4:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'discount', $data);

        $xml_data['listings']['discount'] = $this->ci->pg_seo->url_template_transform('listings', 'discount', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 0,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:listings/][opt:property:literal:all][text:-][opt:operation_type:literal:operation][text:-in-][opt:city:literal:all][text:-id-][tpl:1:id:numeric:0][text:/][tpl:2:section:literal:overview]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'view', $data);

        $xml_data['listings']['view'] = $this->ci->pg_seo->url_template_transform('listings', 'view', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:recommended-properties][text:/all/]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'wish_lists', $data);

        $xml_data['listings']['wish_lists'] = $this->ci->pg_seo->url_template_transform('listings', 'wish_lists', $data["url_template"], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:recommended-properties/][tpl:1:id:numeric:0][text:/][postfix:2:page:literal]',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'wish_list', $data);

        $xml_data['listings']['wish_list'] = $this->ci->pg_seo->url_template_transform('listings', 'wish_list', $data["url_template"], 'base', 'xml');

        $this->ci->Seo_model->set_xml_route_file_content($xml_data);
        $this->ci->Seo_model->rewrite_route_php_file();

        // DEMO
        if (!INSTALL_DONE) {
            $this->addDemoContent();
        }
    }

    /**
     * Import module languages
     * 
     * @param array $langs_ids languages identifiers
     * @return void
     */
    public function _arbitrary_lang_install($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("listings", "arbitrary", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty listings arbitrary langs data");
            return false;
        }

        $post_data = array(
            "title" => $langs_file["seo_tags_agents_title"],
            "keyword" => $langs_file["seo_tags_agents_keyword"],
            "description" => $langs_file["seo_tags_agents_description"],
            "header" => $langs_file["seo_tags_agents_header"],
            "og_title" => $langs_file["seo_tags_agents_og_title"],
            "og_type" => $langs_file["seo_tags_agents_og_type"],
            "og_description" => $langs_file["seo_tags_agents_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "agents", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_category_title"],
            "keyword" => $langs_file["seo_tags_category_keyword"],
            "description" => $langs_file["seo_tags_category_description"],
            "header" => $langs_file["seo_tags_category_header"],
            "og_title" => $langs_file["seo_tags_category_og_title"],
            "og_type" => $langs_file["seo_tags_category_og_type"],
            "og_description" => $langs_file["seo_tags_category_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "category", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_country_title"],
            "keyword" => $langs_file["seo_tags_country_keyword"],
            "description" => $langs_file["seo_tags_country_description"],
            "header" => $langs_file["seo_tags_country_header"],
            "og_title" => $langs_file["seo_tags_country_og_title"],
            "og_type" => $langs_file["seo_tags_country_og_type"],
            "og_description" => $langs_file["seo_tags_country_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "country", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_discount_title"],
            "keyword" => $langs_file["seo_tags_discount_keyword"],
            "description" => $langs_file["seo_tags_discount_description"],
            "header" => $langs_file["seo_tags_discount_header"],
            "og_title" => $langs_file["seo_tags_discount_og_title"],
            "og_type" => $langs_file["seo_tags_discount_og_type"],
            "og_description" => $langs_file["seo_tags_discount_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "discount", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_index_title"],
            "keyword" => $langs_file["seo_tags_index_keyword"],
            "description" => $langs_file["seo_tags_index_description"],
            "header" => $langs_file["seo_tags_index_header"],
            "og_title" => $langs_file["seo_tags_index_og_title"],
            "og_type" => $langs_file["seo_tags_index_og_type"],
            "og_description" => $langs_file["seo_tags_index_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "index", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_location_title"],
            "keyword" => $langs_file["seo_tags_location_keyword"],
            "description" => $langs_file["seo_tags_location_description"],
            "header" => $langs_file["seo_tags_location_header"],
            "og_title" => $langs_file["seo_tags_location_og_title"],
            "og_type" => $langs_file["seo_tags_location_og_type"],
            "og_description" => $langs_file["seo_tags_location_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "location", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_open_house_title"],
            "keyword" => $langs_file["seo_tags_open_house_keyword"],
            "description" => $langs_file["seo_tags_open_house_description"],
            "header" => $langs_file["seo_tags_open_house_header"],
            "og_title" => $langs_file["seo_tags_open_house_og_title"],
            "og_type" => $langs_file["seo_tags_open_house_og_type"],
            "og_description" => $langs_file["seo_tags_open_house_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "open_house", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_privates_title"],
            "keyword" => $langs_file["seo_tags_privates_keyword"],
            "description" => $langs_file["seo_tags_privates_description"],
            "header" => $langs_file["seo_tags_privates_header"],
            "og_title" => $langs_file["seo_tags_privates_og_title"],
            "og_type" => $langs_file["seo_tags_privates_og_type"],
            "og_description" => $langs_file["seo_tags_privates_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "privates", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_region_title"],
            "keyword" => $langs_file["seo_tags_region_keyword"],
            "description" => $langs_file["seo_tags_region_description"],
            "header" => $langs_file["seo_tags_region_header"],
            "og_title" => $langs_file["seo_tags_region_og_title"],
            "og_type" => $langs_file["seo_tags_region_og_type"],
            "og_description" => $langs_file["seo_tags_region_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "region", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_search_title"],
            "keyword" => $langs_file["seo_tags_search_keyword"],
            "description" => $langs_file["seo_tags_search_description"],
            "header" => $langs_file["seo_tags_search_header"],
            "og_title" => $langs_file["seo_tags_search_og_title"],
            "og_type" => $langs_file["seo_tags_search_og_type"],
            "og_description" => $langs_file["seo_tags_search_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "search", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_user_title"],
            "keyword" => $langs_file["seo_tags_user_keyword"],
            "description" => $langs_file["seo_tags_user_description"],
            "header" => $langs_file["seo_tags_user_header"],
            "og_title" => $langs_file["seo_tags_user_og_title"],
            "og_type" => $langs_file["seo_tags_user_og_type"],
            "og_description" => $langs_file["seo_tags_user_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "user", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_view_title"],
            "keyword" => $langs_file["seo_tags_view_keyword"],
            "description" => $langs_file["seo_tags_view_description"],
            "header" => $langs_file["seo_tags_view_header"],
            "og_title" => $langs_file["seo_tags_view_og_title"],
            "og_type" => $langs_file["seo_tags_view_og_type"],
            "og_description" => $langs_file["seo_tags_view_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "view", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_wish_list_title"],
            "keyword" => $langs_file["seo_tags_wish_list_keyword"],
            "description" => $langs_file["seo_tags_wish_list_description"],
            "header" => $langs_file["seo_tags_wish_list_header"],
            "og_title" => $langs_file["seo_tags_wish_list_og_title"],
            "og_type" => $langs_file["seo_tags_wish_list_og_type"],
            "og_description" => $langs_file["seo_tags_wish_list_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "wish_list", $post_data);

        $post_data = array(
            "title" => $langs_file["seo_tags_wish_lists_title"],
            "keyword" => $langs_file["seo_tags_wish_lists_keyword"],
            "description" => $langs_file["seo_tags_wish_lists_description"],
            "header" => $langs_file["seo_tags_wish_lists_header"],
            "og_title" => $langs_file["seo_tags_wish_lists_og_title"],
            "og_type" => $langs_file["seo_tags_wish_lists_og_type"],
            "og_description" => $langs_file["seo_tags_wish_lists_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "listings", "wish_lists", $post_data);

        return;
    }

    /**
     * Export module languages
     * 
     * @param array $langs_ids languages identifiers
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
        if (!is_array($langs_ids))
            $langs_ids = (array) $langs_ids;

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "agents", $langs_ids);
        $arbitrary_return["seo_tags_agents_title"] = $settings["title"];
        $arbitrary_return["seo_tags_agents_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_agents_description"] = $settings["description"];
        $arbitrary_return["seo_tags_agents_header"] = $settings["header"];
        $arbitrary_return["seo_tags_agents_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_agents_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_agents_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "category", $langs_ids);
        $arbitrary_return["seo_tags_category_title"] = $settings["title"];
        $arbitrary_return["seo_tags_category_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_category_description"] = $settings["description"];
        $arbitrary_return["seo_tags_category_header"] = $settings["header"];
        $arbitrary_return["seo_tags_category_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_category_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_category_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "country", $langs_ids);
        $arbitrary_return["seo_tags_country_title"] = $settings["title"];
        $arbitrary_return["seo_tags_country_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_country_description"] = $settings["description"];
        $arbitrary_return["seo_tags_country_header"] = $settings["header"];
        $arbitrary_return["seo_tags_country_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_country_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_country_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "discount", $langs_ids);
        $arbitrary_return["seo_tags_discount_title"] = $settings["title"];
        $arbitrary_return["seo_tags_discount_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_discount_description"] = $settings["description"];
        $arbitrary_return["seo_tags_discount_header"] = $settings["header"];
        $arbitrary_return["seo_tags_discount_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_discount_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_discount_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "index", $langs_ids);
        $arbitrary_return["seo_tags_index_title"] = $settings["title"];
        $arbitrary_return["seo_tags_index_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_index_description"] = $settings["description"];
        $arbitrary_return["seo_tags_index_header"] = $settings["header"];
        $arbitrary_return["seo_tags_index_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_index_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_index_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "location", $langs_ids);
        $arbitrary_return["seo_tags_location_title"] = $settings["title"];
        $arbitrary_return["seo_tags_location_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_location_description"] = $settings["description"];
        $arbitrary_return["seo_tags_location_header"] = $settings["header"];
        $arbitrary_return["seo_tags_location_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_location_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_location_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "open_house", $langs_ids);
        $arbitrary_return["seo_tags_open_house_title"] = $settings["title"];
        $arbitrary_return["seo_tags_open_house_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_open_house_description"] = $settings["description"];
        $arbitrary_return["seo_tags_open_house_header"] = $settings["header"];
        $arbitrary_return["seo_tags_open_house_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_open_house_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_open_house_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "privates", $langs_ids);
        $arbitrary_return["seo_tags_privates_title"] = $settings["title"];
        $arbitrary_return["seo_tags_privates_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_privates_description"] = $settings["description"];
        $arbitrary_return["seo_tags_privates_header"] = $settings["header"];
        $arbitrary_return["seo_tags_privates_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_privates_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_privates_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "region", $langs_ids);
        $arbitrary_return["seo_tags_region_title"] = $settings["title"];
        $arbitrary_return["seo_tags_region_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_region_description"] = $settings["description"];
        $arbitrary_return["seo_tags_region_header"] = $settings["header"];
        $arbitrary_return["seo_tags_region_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_region_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_region_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "search", $langs_ids);
        $arbitrary_return["seo_tags_search_title"] = $settings["title"];
        $arbitrary_return["seo_tags_search_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_search_description"] = $settings["description"];
        $arbitrary_return["seo_tags_search_header"] = $settings["header"];
        $arbitrary_return["seo_tags_search_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_search_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_search_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "user", $langs_ids);
        $arbitrary_return["seo_tags_user_title"] = $settings["title"];
        $arbitrary_return["seo_tags_user_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_user_description"] = $settings["description"];
        $arbitrary_return["seo_tags_user_header"] = $settings["header"];
        $arbitrary_return["seo_tags_user_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_user_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_user_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "view", $langs_ids);
        $arbitrary_return["seo_tags_view_title"] = $settings["title"];
        $arbitrary_return["seo_tags_view_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_view_description"] = $settings["description"];
        $arbitrary_return["seo_tags_view_header"] = $settings["header"];
        $arbitrary_return["seo_tags_view_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_view_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_view_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "wish_list", $langs_ids);
        $arbitrary_return["seo_tags_wish_list_title"] = $settings["title"];
        $arbitrary_return["seo_tags_wish_list_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_wish_list_description"] = $settings["description"];
        $arbitrary_return["seo_tags_wish_list_header"] = $settings["header"];
        $arbitrary_return["seo_tags_wish_list_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_wish_list_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_wish_list_og_description"] = $settings["og_description"];

        $settings = $this->ci->pg_seo->get_settings("user", "listings", "wish_lists", $langs_ids);
        $arbitrary_return["seo_tags_wish_lists_title"] = $settings["title"];
        $arbitrary_return["seo_tags_wish_lists_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_wish_lists_description"] = $settings["description"];
        $arbitrary_return["seo_tags_wish_lists_header"] = $settings["header"];
        $arbitrary_return["seo_tags_wish_lists_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_wish_lists_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_wish_lists_og_description"] = $settings["og_description"];

        return array("arbitrary" => $arbitrary_return);
    }

    /**
     * Uninstall module data
     * 
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        // remove seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'index', $data);

        $xml_data['listings']['index'] = $this->ci->pg_seo->url_template_transform('listings', 'index', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'category', $data);

        $xml_data['listings']['category'] = $this->ci->pg_seo->url_template_transform('listings', 'category', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'location', $data);

        $xml_data['listings']['location'] = $this->ci->pg_seo->url_template_transform('listings', 'location', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'user', $data);

        $xml_data['listings']['user'] = $this->ci->pg_seo->url_template_transform('listings', 'user', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'search', $data);

        $xml_data['listings']['search'] = $this->ci->pg_seo->url_template_transform('listings', 'search', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'open_house', $data);

        $xml_data['listings']['open_house'] = $this->ci->pg_seo->url_template_transform('listings', 'open_house', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'privates', $data);

        $xml_data['listings']['privates'] = $this->ci->pg_seo->url_template_transform('listings', 'privates', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'agents', $data);

        $xml_data['listings']['agents'] = $this->ci->pg_seo->url_template_transform('listings', 'agents', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'discount', $data);

        $xml_data['listings']['discount'] = $this->ci->pg_seo->url_template_transform('listings', 'discount', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'view', $data);

        $xml_data['listings']['view'] = $this->ci->pg_seo->url_template_transform('listings', 'view', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'wish_lists', $data);

        $xml_data['listings']['wish_lists'] = $this->ci->pg_seo->url_template_transform('listings', 'wish_lists', $data["url_template"], 'base', 'xml');

        $data = array(
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'listings', 'wish_list', $data);

        $xml_data['listings']['wish_list'] = $this->ci->pg_seo->url_template_transform('listings', 'wish_list', $data["url_template"], 'base', 'xml');

        $this->ci->Seo_model->set_xml_route_file_content($xml_data);
        $this->ci->Seo_model->rewrite_route_php_file();

        $this->ci->pg_seo->delete_seo_module("listings");

        /// delete entries in dedicate modules
        foreach ($this->modules_data['lang_dm_data'] as $lang_dm_data) {
            $this->ci->pg_language->delete_dedicate_modules_entry(array('where' => $lang_dm_data));
        }
    }

    /**
     * Install demo content
     * 
     * @return void
     */
    public function addDemoContent()
    {
        $this->ci->load->model('Listings_model');

        $demo_content = Setup::getModuleData('listings', Setup::TYPE_DEMO_CONTENT);

        foreach ((array) $demo_content['listings'] as $listing_data) {
            $validate_data = $this->ci->Listings_model->validate_listing(null, $listing_data);
            if (!empty($validate_data['errors'])) {
                continue;
            }
            $this->ci->Listings_model->save_listing(null, $validate_data['data']);
        }

        if ($this->ci->pg_module->is_module_active('upload_gallery')) {
            $this->ci->load->model('Upload_gallery_model');
            $this->ci->load->model('listings/models/Listings_uploads_model');

            $gallery_type = $this->ci->Upload_gallery_model->get_type_by_gid(
                $this->ci->Listings_uploads_model->gallery_type
            );

            foreach ((array) $demo_content['uploads'] as $upload) {
                $upload["type_id"] = $gallery_type["id"];

                if (isset($upload["id"])) {
                    if ($this->ci->Upload_gallery_model->get_file_by_id($upload["id"])) {
                        continue;
                    }
                }

                $validate_data = $this->ci->Upload_gallery_model->validate_file_data(
                    null, $gallery_type['id'], $upload
                );
                if (!empty($validate_data['errors'])) {
                    continue;
                }
                $validate_data['data'] = array_merge($upload, $validate_data['data']);

                $ret = $this->ci->Upload_gallery_model->save_file_data(
                    null, $validate_data['data'], "", false
                );

                $params = array();
                $params["where"]["type_id"] = $gallery_type["id"];
                $params["where"]["object_id"] = $upload["object_id"];
                $params["where"]["status"] = 1;

                $photos = $this->ci->Upload_gallery_model->get_files_by_param($params);

                $post_data = array('photo_count' => count($photos));
                $validate_data = $this->ci->Listings_model->validate_listing(
                    $upload["object_id"], $post_data
                );
                if (!empty($validate_data['errors'])) {
                    continue;
                }
                $this->ci->Listings_model->save_listing($upload["object_id"], $validate_data["data"]);
            }

            $vtour_type = $this->ci->Upload_gallery_model->get_type_by_gid(
                $this->ci->Listings_uploads_model->virtual_tour_type
            );

            foreach ((array) $demo_content['vtours'] as $vtour) {
                $vtour["type_id"] = $vtour_type["id"];

                if (isset($vtour["id"])) {
                    if ($this->ci->Upload_gallery_model->get_file_by_id($vtour["id"])) {
                        continue;
                    }
                }

                $validate_data = $this->ci->Upload_gallery_model->validate_file_data(
                    null, $vtour_type['id'], $vtour
                );
                if (!empty($validate_data['errors'])) {
                    continue;
                }
                $validate_data['data'] = array_merge($vtour, $validate_data['data']);

                $ret = $this->ci->Upload_gallery_model->save_file_data(
                    null, $validate_data['data'], "", false
                );

                $params = array();
                $params["where"]["type_id"] = $vtour_type["id"];
                $params["where"]["object_id"] = $vtour["object_id"];
                $params["where"]["status"] = 1;

                $panoramas = $this->ci->Upload_gallery_model->get_files_by_param($params);

                $post_data = array('is_vtour' => count($panoramas) > 0);
                $validate_data = $this->ci->Listings_model->validate_listing(
                    $vtour["object_id"], $post_data
                );
                if (!empty($validate_data['errors'])) {
                    continue;
                }
                $this->ci->Listings_model->save_listing($vtour["object_id"], $validate_data["data"]);
            }
        }

        $this->ci->load->model('listings/models/Wish_list_model');
        foreach ((array) $demo_content['wish_lists'] as $wish_list) {
            $data = array('id' => $wish_list['id']);
            foreach ($this->ci->pg_language->languages as $id => $value) {
                if (isset($wish_list["name_" . $value["code"]])) {
                    $data["name_" . $value["id"]] = $wish_list['name_' . $value["code"]];
                } else {
                    $data["name_" . $value["id"]] = $wish_list['name'];
                }
            }
            $validate_data = $this->ci->Wish_list_model->validate_wish_list($wish_list['id'], $data);
            if (!empty($validate_data['errors'])) {
                continue;
            }
            $this->ci->Wish_list_model->save_wish_list($wish_list['id'], $validate_data['data']);
        }

        if ($this->ci->pg_module->is_module_active("reviews")) {
            $this->ci->load->model('Reviews_model');
            foreach ((array) $demo_content['reviews'] as $review) {
                $validate_data = $this->ci->Reviews_model->validate_review(null, $review);
                if (!empty($validate_data['errors'])) {
                    continue;
                }
                $this->ci->Reviews_model->save_review(null, $validate_data['data']);
            }
        }

        $this->ci->Listings_model->resetRanges();
        return true;
    }

}
