<?php

namespace Pg\Modules\Mail_list\Controllers;

/**
 * Mail list module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Mail list admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Mail list
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Mail_list extends \Controller
{

    /**
     * Date format used in mails
     *
     * @var string
     */
    protected $_date_format = null;

    /**
     * Number items per page
     *
     * @var integer
     */
    protected $_items_on_page = null;

    /**
     * Constructor
     *
     * @return Admin_Mail_list
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model(array('Menu_model', 'Mail_list_model'));
        $this->Menu_model->set_menu_active_item('admin_menu', 'newsletters-items');

        // Date format
        $this->_date_format = $this->pg_date->get_format('date_time_literal', 'st');

        $this->_items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');
        $this->view->setHeader(l('mailing_lists', 'mail_list'));

        return true;
    }

    /**
     * Manage subscribers
     *
     * @param string  $filter filter value
     * @param integer $page   page of result
     *
     * @return void
     */
    public function index($filter = 'all', $page = 1)
    {
        $this->users($filter, $page);
    }

    /**
     * Manage subscribers
     *
     * @param string  $filter filter value
     * @param integer $page   page of result
     *
     * @return void
     */
    public function users($filter = 'all', $page = 1)
    {

        // Set default filter
        if (!in_array($filter, array('all', 'subscribed', 'not_subscribed'))) {
            // ...or page if filter isn't specified
            if ($page < (intval($filter))) {
                $page = $filter;
            }
            $filter = 'all';
        }
        $page = floor($page);
        if ($page < 1) {
            $page = 1;
        }

        $this->load->model('Subscriptions_model');
        $subscriptions = $this->Subscriptions_model->get_subscriptions_list();

        // Clear settings if btn_cancel pressed
        if ($this->input->post('btn_cancel', true)) {
            $current_settings = array();
        } else {
            // Apply filter if sent
            if ($this->input->post('id_filter', true)) {
                $_SESSION['mail_list'] = $this->Mail_list_model->get_filter($this->input->post('id_filter', true));
            }
            if (isset($_SESSION['mail_list'])) {
                $current_settings = $_SESSION['mail_list'];
            } else {
                $current_settings = array();
            }

            $fields = array(
                'email',
                'name',
                'date',
                'user_type',
                'id_country',
                'id_region',
                'id_city',
                'id_district',
            );

            // Fill current_settings with values from $_POST
            foreach ($fields as $field) {
                // We do need identity comparison
                if (false !== $this->input->post($field, true)) {
                    $current_settings[$field] = $this->input->post($field, true);
                }
            }
        }

        if (false !== $this->input->post('id_subscription', true)) {
            $current_settings['id_subscription'] = $this->input->post('id_subscription', true);
        } elseif (empty($current_settings['id_subscription'])) {
            // We need subscription id, so if $_POST doesn't contain it, we take first one from database
            $current_settings['id_subscription'] = $subscriptions[0]['id'];
        }
        $current_settings['filter'] = $filter;
        $current_settings['page'] = $page;

        $search_attrs = $this->Mail_list_model->format_search_attrs($current_settings);
        $users = $this->Mail_list_model->get_users($search_attrs, $page, $this->_items_on_page);

        // Save settings for further use
        $_SESSION['mail_list'] = $current_settings;
        $_SESSION['mail_list']['search_attrs'] = $search_attrs;

        // Count users
        unset($search_attrs['where']['id_subscription']);
        unset($search_attrs['where_not']['id_subscription']);
        $users_count['all'] = $this->Mail_list_model->get_users_count($search_attrs);
        $search_attrs['where']['id_subscription'] = $current_settings['id_subscription'];
        $users_count['subscribed'] = $this->Mail_list_model->get_users_count($search_attrs);
        $users_count['not_subscribed'] = $users_count['all'] - $users_count['subscribed'];

        $this->view->assign('user_types', ld('user_type', 'users'));

        // Pagination
        $this->load->helper('navigation');
        $url = site_url() . "admin/mail_list/users/$filter/";
        $page_data = get_admin_pages_data($url, $users_count[$filter], $this->_items_on_page, $page, 'briefPage');

        $this->view->assign('users_count', $users_count);
        $this->view->assign('page_data', $page_data);
        $this->view->assign('data', $current_settings);
        $this->view->assign('date_format', $this->_date_format);
        $this->view->assign('users', $users);
        $this->view->assign('subscriptions', $subscriptions);

        $this->view->render('users');
    }

    /**
     * Manage filters
     *
     * @param integer $page page of results
     *
     * @return void
     */
    public function filters($page = 1)
    {
        $filters = $this->Mail_list_model->get_filters($page, $this->_items_on_page);
        $filters_count = $this->Mail_list_model->get_filters_count();

        // Pagination
        $this->load->helper('navigation');
        $url = site_url() . 'admin/mail_list/filters/';
        $page_data = get_admin_pages_data($url, $filters_count, $this->_items_on_page, $page, 'briefPage');

        $this->pg_theme->add_js('admin-mail-list.js', 'mail_list');
        $this->view->assign('filters', $filters);
        $this->view->assign('page_data', $page_data);
        $this->view->assign('date_format', $this->_date_format);
        $this->view->render('filters');
    }

    /**
     * Save filter data by ajax
     *
     * @return void
     */
    public function ajax_save_filter()
    {
        $fields = array(
            'email',
            'name',
            'date',
            'user_type',
            'id_subscription',
            'id_country',
            'id_region',
            'id_city',
            'id_district',
        );

        // Fill $data with values from $_POST
        foreach ($fields as $field) {
            if ($this->input->post($field, true)) {
                $data[$field] = $this->input->post($field, true);
            }
        }

        if (is_array($data)) {
            $this->Mail_list_model->save_filter($data);
        }
    }

    /**
     * Remove filter data by ajax
     *
     * @return void
     */
    public function ajax_delete_filter()
    {
        $id_filter = $this->input->post('id_filter', true);
        $this->view->output($this->Mail_list_model->delete_filter($id_filter));
        $this->view->render();
    }

    /**
     * Subscribe/unsubscribe users
     *
     * @param string $action action name (post)
     *
     * @return void
     */
    public function ajax_subscribe()
    {
        $action = $this->input->post('action', true);
        $id_subscription = (int) $this->input->post('id_subscription', true);
        switch ($action) {
            case 'subscribe_one' :
                $id_user = array($this->input->post('id_user', true));
                $this->view->output($this->Mail_list_model->subscribe_users($id_subscription, $id_user));
                $this->view->render();
                break;
            case 'unsubscribe_one' :
                $id_user = array($this->input->post('id_user', true));
                $this->view->output($this->Mail_list_model->unsubscribe_users($id_subscription, $id_user));
                $this->view->render();
                break;
            case 'subscribe_selected' :
                $id_users = $this->input->post('id_users', true);
                $this->view->output($this->Mail_list_model->subscribe_users($id_subscription, $id_users));
                $this->view->render();
                break;
            case 'unsubscribe_selected' :
                $id_users = $this->input->post('id_users', true);
                $this->view->output($this->Mail_list_model->unsubscribe_users($id_subscription, $id_users));
                $this->view->render();
                break;
            case 'subscribe_all' :
                $id_users = $this->Mail_list_model->get_users($_SESSION['mail_list']['search_attrs'], null, null, true);
                $this->view->output($this->Mail_list_model->subscribe_users($id_subscription, $id_users));
                $this->view->render();
                break;
            case 'unsubscribe_all' :
                $id_users = $this->Mail_list_model->get_users($_SESSION['mail_list']['search_attrs'], null, null, true);
                $this->view->output($this->Mail_list_model->unsubscribe_users($id_subscription, $id_users));
                $this->view->render();
                break;
        }
    }

}
