<?php

namespace Pg\Modules\Mail_list\Models;

/**
 * Mail list module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

if (!defined('SUBSCRIPTIONS_USERS_TABLE')) {
    define('SUBSCRIPTIONS_USERS_TABLE', DB_PREFIX . 'subscriptions_users');
}
if (!defined('MAIL_LIST_FILTERS_TABLE')) {
    define('MAIL_LIST_FILTERS_TABLE', DB_PREFIX . 'mail_list_filters');
}

/**
 * Mail List Main Model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Mail list
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Mail_list_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Subscriber object properties in data source
     *
     * @var array
     */
    protected $fields_user = array(
        'id',
        'date_created',
        'email',
        'unique_name',
        'user_type',
        'password',
        'status',
        'confirm',
        'fname',
        'sname',
        'company_name',
        'lang_id',
        'group_id',
        'contact_email',
        'id_country',
        'id_region',
        'id_city',
        'id_district',
        'date_created',
        'date_modified',
        'lat',
        'lon',
        'address',
        'postal_code',
        'agent_company',
        'agent_status',
    );

    /**
     * User subcription object properties in data source
     *
     * @var array
     */
    protected $fields_subscriptions_users = array(
        'id_user',
        'id_subscription',
    );

    /**
     * Filter object properties in data source
     *
     * @var array
     */
    protected $fields_filters = array(
        'id',
        'search_data',
        'date_search',
    );

    /**
     * Constructor
     *
     * @return Mail_list object
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->load->model('Users_model');
        $this->load->model('Subscriptions_model');
        // Add table name as prefixes
        array_walk($this->fields_user, create_function('&$item', '$item = USERS_TABLE . "." . $item;'));
    }

    /**
     * Return formatted subscribers objects as array
     *
     * @param array   $params        sql criteria of query to data source
     * @param integer $page          page of results
     * @param integer $items_on_page items per page
     * @param boolean $only_id       if true, returns only a list of user id and won't split result into pages
     *
     * @return array/boolean
     */
    public function get_users($params, $page = 1, $items_on_page = null, $only_id = false)
    {
        $this->load->model('Users_model');
        if (false == $only_id) {
            $this->ci->db->select(implode(', ', $this->fields_user))
                    ->select(implode(', ', $this->fields_subscriptions_users));
        } else {
            $this->ci->db->select(USERS_TABLE . '.id');
        }
        $this->ci->db->from(USERS_TABLE);

        if (isset($params['where']['id_subscription']) && intval($params['where']['id_subscription'])) {
            $this->ci->db->join(SUBSCRIPTIONS_USERS_TABLE,
                                USERS_TABLE . '.id = ' . SUBSCRIPTIONS_USERS_TABLE . '.id_user AND ' .
                    SUBSCRIPTIONS_USERS_TABLE . ".id_subscription='" . intval($params['where']['id_subscription']) . "'");
            unset($params['where']['id_subscription']);
        } elseif (isset($params['where_not']['id_subscription']) && intval($params['where_not']['id_subscription'])) {
            $this->ci->db->join(SUBSCRIPTIONS_USERS_TABLE,
                                USERS_TABLE . '.id = ' . SUBSCRIPTIONS_USERS_TABLE . '.id_user AND ' .
                    SUBSCRIPTIONS_USERS_TABLE . ".id_user=" . USERS_TABLE . '.id', 'left');
            $this->ci->db->where(USERS_TABLE . '.id NOT IN',
                                 '(SELECT `id_user` FROM `' . SUBSCRIPTIONS_USERS_TABLE .
                    "` WHERE `id_subscription` = '{$params['where_not']['id_subscription']}')", false);
            $this->ci->db->group_by(USERS_TABLE . '.id');
            unset($params['where_not']['id_subscription']);
        } else {
            $join_subscr = 'AND ' . SUBSCRIPTIONS_USERS_TABLE . '.id_subscription = ' . $params['id_subscription'];
            $this->ci->db->join(SUBSCRIPTIONS_USERS_TABLE,
                                USERS_TABLE . '.id = ' . SUBSCRIPTIONS_USERS_TABLE . ".id_user $join_subscr", 'left');
        }

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                if ($value) {
                    $this->ci->db->where($field, $value);
                }
            }
        }

        if (isset($params['like']) && is_array($params['like']) && count($params['like'])) {
            foreach ($params['like'] as $field => $value) {
                if ($value) {
                    $this->ci->db->like($field, $value);
                }
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (false == $only_id) {
            if (intval($page) <= 0) {
                $page = 1;
            }
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $users = $this->ci->db->get()->result_array();
        if (!empty($users) && is_array($users)) {
            if (false == $only_id) {
                foreach ($users as $user) {
                    $data[] = $user;
                }
                $data = $this->Users_model->format_users($data);
            } else {
                foreach ($users as $user) {
                    $data[] = $user['id'];
                }
            }

            return $data;
        }

        return false;
    }

    /**
     * Return number of subscribers objects
     *
     * @param array $params sql criteria of query to data source
     *
     * @return integer
     */
    public function get_users_count($params)
    {
        $this->ci->db->select('COUNT(' . USERS_TABLE . '.id) AS cnt');
        $this->ci->db->from(USERS_TABLE);

        if (isset($params['where']['id_subscription']) && intval($params['where']['id_subscription'])) {
            $this->ci->db->join(SUBSCRIPTIONS_USERS_TABLE,
                                USERS_TABLE . '.id = ' . SUBSCRIPTIONS_USERS_TABLE . '.id_user AND ' . SUBSCRIPTIONS_USERS_TABLE . ".id_subscription='" . intval($params['where']['id_subscription']) . "'");
        } elseif (isset($params['where_not']['id_subscription']) && intval($params['where_not']['id_subscription'])) {
            $this->ci->db->select(implode(', ', $this->fields_subscriptions_users));
            $this->ci->db->join(SUBSCRIPTIONS_USERS_TABLE,
                                USERS_TABLE . ".id = " . SUBSCRIPTIONS_USERS_TABLE . ".id_user AND " . SUBSCRIPTIONS_USERS_TABLE . ".id_user=" . USERS_TABLE . ".id",
                                'left'); // . intval($params['where_not']['id_subscription']) . "'");
            $where = '(' . SUBSCRIPTIONS_USERS_TABLE . '.id_user is null OR ' . SUBSCRIPTIONS_USERS_TABLE . '.id_subscription <>' . $params['where_not']['id_subscription'] . ')';
            $this->db->where($where);
            unset($params['where_not']['id_subscription']);
        }

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                if ($value) {
                    $this->ci->db->where($field, $value);
                }
            }
        }

        if (isset($params['like']) && is_array($params['like']) && count($params['like'])) {
            foreach ($params['like'] as $field => $value) {
                if ($value) {
                    $this->ci->db->or_like($field, $value);
                }
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value);
            }
        }

        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            return intval($result[0]->cnt);
        } else {
            return 0;
        }
    }

    /**
     * Subscribe users to subscription
     *
     * @param integer $id_subscription subscription identifier
     * @param array   $id_users        users identifiers
     *
     * @return boolean
     */
    public function subscribe_users($id_subscription, $id_users)
    {
        if (!intval($id_subscription) || !$id_users) {
            return false;
        }
        $data['id_subscription'] = (int) $id_subscription;
        foreach ($id_users as $id_user) {
            $data['id_user'] = (int) $id_user;
            $this->ci->db->ignore()->insert(SUBSCRIPTIONS_USERS_TABLE, $data);
            $this->ci->db->insert_id();
        }

        return true;
    }

    /**
     * Unsubscribe users from subscription
     *
     * @param integer $id_subscription subscription identifier
     * @param array   $id_users        users identifiers
     *
     * @return boolean
     */
    public function unsubscribe_users($id_subscription, $id_users)
    {
        if (!intval($id_subscription) || !$id_users) {
            return false;
        }
        $data['id_subscription'] = (int) $id_subscription;
        foreach ($id_users as $id_user) {
            $data['id_user'] = (int) $id_user;
            $this->ci->db->where('id_user', $id_user)
                    ->where('id_subscription', $id_subscription)
                    ->delete(SUBSCRIPTIONS_USERS_TABLE);
        }

        return true;
    }

    /**
     * Save filter criteria to data source
     *
     * Return record id when saved, true when exists, false when error
     *
     * @param array $attrs data of search criteria
     *
     * @return boolean
     */
    public function save_filter($attrs)
    {
        if (false == is_array($attrs)) {
            return false;
        }
        $data['date_search'] = date('Y-m-d H:i:s');
        $data['search_data'] = serialize($attrs);

        // Check uniqueness
        $this->db->where('search_data', $data['search_data']);
        $this->db->from(MAIL_LIST_FILTERS_TABLE);
        $is_unique = 0 == $this->db->count_all_results();

        if ($is_unique) {
            $this->ci->db->insert(MAIL_LIST_FILTERS_TABLE, $data);

            return $this->ci->db->insert_id();
        }

        return true;
    }

    /**
     * Returns formatted filters objects as array
     *
     * @param integer $page          page of results
     * @param integer $items_on_page items per page
     *
     * @return array
     */
    public function get_filters($page = 1, $items_on_page = null)
    {
        $this->ci->db->select(implode(', ', $this->fields_filters));
        $this->ci->db->from(MAIL_LIST_FILTERS_TABLE);

        if (0 > intval($page)) {
            $page = 1;
        }

        $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));

        $searches = $this->ci->db->get()->result_array();
        if (!empty($searches) && is_array($searches)) {
            foreach ($searches as $key => $search) {
                $search['search_data'] = unserialize($search['search_data']);
                $data[$key] = $search;
            }

            return $this->format_filters($data);
        }
    }

    /**
     * Return filter object
     *
     * @param integer $id_filter filter identifier
     *
     * @return array
     */
    public function get_filter($id_filter)
    {
        $filter = $this->ci->db->select('search_data')
                        ->from(MAIL_LIST_FILTERS_TABLE)
                        ->where('id', $id_filter)
                        ->get()->row();

        return unserialize($filter->search_data);
    }

    /**
     * Format filters objects
     *
     * @param array $data filters data
     *
     * @return array
     */
    public function format_filters($data)
    {
        $locations = array();
        foreach ($data as $key => $filter) {
            $locations[$key] = array(
                'country' => !empty($filter['search_data']['id_country']) ? $filter['search_data']['id_country'] : 0,
                'region' => !empty($filter['search_data']['id_region']) ? $filter['search_data']['id_region'] : 0,
                'city' => !empty($filter['search_data']['id_city']) ? $filter['search_data']['id_city'] : 0,
                'district' => !empty($filter['search_data']['id_district']) ? $filter['search_data']['id_district'] : 0,
            );

            $data[$key]['search_data']['subscription'] = l(
                    "subscription_{$filter['search_data']['id_subscription']}", 'subscriptions'
            );
        }

        // Format locations
        if (!empty($locations)) {
            $this->ci->load->helper('countries');
            $user_friendly_location = cities_output_format($locations);

            foreach ($data as $key => $filter) {
                $data[$key]['location'] = (isset($user_friendly_location[$key])) ? $user_friendly_location[$key] : '';
            }
        }

        return $data;
    }

    /**
     * Return number of all filters objects
     *
     * @return integer
     */
    public function get_filters_count()
    {
        return $this->db->count_all(MAIL_LIST_FILTERS_TABLE);
    }

    /**
     * Removes filter object
     *
     * @param int $id_filter filter identifier
     *
     * @return boolean
     */
    public function delete_filter($id_filter)
    {
        $this->ci->db->where('id', $id_filter)
                ->delete(MAIL_LIST_FILTERS_TABLE);

        return true;
    }

    /**
     * Format data of search object
     *
     * @param array $settings settings data
     *
     * @return array
     */
    public function format_search_attrs($settings)
    {
        $search_attrs = array();

        if (!empty($settings['email'])) {
            $search_attrs['like']['email'] = $settings['email'];
        }

        if (!empty($settings['name'])) {
            $search_attrs['where_sql'][] = "(
				fname LIKE '%{$settings['name']}%' OR
				sname LIKE '%{$settings['name']}%' OR
				unique_name LIKE '%{$settings['name']}%')";
        }

        if (!empty($settings['date'])) {
            $search_attrs['where']['date_created >'] = $settings['date'];
        }

        if (!empty($settings['user_type'])) {
            $search_attrs['where']['user_type'] = $settings['user_type'];
        }

        // Location
        if (!empty($settings['id_district'])) {
            $search_attrs['where']['id_district'] = $settings['id_district'];
        } elseif (!empty($settings['id_city'])) {
            $search_attrs['where']['id_city'] = $settings['id_city'];
        } elseif (!empty($settings['id_region'])) {
            $search_attrs['where']['id_region'] = $settings['id_region'];
        } elseif (!empty($settings['id_country'])) {
            $search_attrs['where']['id_country'] = $settings['id_country'];
        }

        // Subscription
        if ('subscribed' == $settings['filter']) {
            $search_attrs['where']['id_subscription'] = $settings['id_subscription'];
        } elseif ('not_subscribed' == $settings['filter']) {
            $search_attrs['where_not']['id_subscription'] = $settings['id_subscription'];
        }
        $search_attrs['id_subscription'] = $settings['id_subscription'];

        return $search_attrs;
    }

}
