<?php

namespace Pg\Modules\Memberships\Controllers;

use Pg\Modules\Memberships\Models\Memberships_model;
use Pg\Libraries\View;

/**
 * Memberships module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Memberships user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Memberships
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Memberships extends \Controller
{

    /**
     * Class constructor
     *
     * @return Memberships
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Memberships_model");
    }

    /**
     * Module management
     *
     * @return void
     */
    public function index()
    {
        $where['is_active'] = 1;
        $memberships = $this->Memberships_model->getMembershipsList($where, null, null, array('priority' => 'ASC'));
        $this->view->assign('memberships', $memberships);

        $this->load->model('Menu_model');
        $this->Menu_model->breadcrumbs_set_parent('memberships_item');
        $this->view->render('index');
    }

    /**
     * Membership management
     *
     * @param string $membership_gid membership GUID
     */
    public function form($membership_gid)
    {
        $user_id = $this->session->userdata('user_id');

        $this->load->model('users/models/Auth_model');
        $this->Auth_model->update_user_session_data($user_id);

        $membership_data = $this->Memberships_model->getMembershipByGid($membership_gid);
        if (!$membership_data['is_active']) {
            show_404();
            return;
        }

        $this->Memberships_model->setFormatSettings('get_services', true);
        $membership_data = $this->Memberships_model->formatMembership($membership_data);
        $this->Memberships_model->setFormatSettings('get_services', false);

        if ($this->input->post('btn_system') || $this->input->post('btn_account')) {
            if ($this->input->post('btn_account')) {
                $return = $this->Memberships_model->accountMembershipPayment($membership_data["id"], $user_id,
                                                                             $membership_data["price"]);
                if ($return !== true) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $return);
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_memberships_apply', 'memberships'));

                    $redirect = $this->session->userdata('service_redirect');
                    $this->session->set_userdata(array('service_redirect' => ''));
                    $this->load->model('users/models/Auth_model');
                    $this->Auth_model->update_user_session_data($user_id);
                    $this->view->setRedirect($redirect);
                }
            } elseif ($this->input->post('btn_system')) {
                $system_gid = $this->input->post('system_gid', true);
                if (empty($system_gid)) {
                    $this->system_messages->addMessage(View::MSG_ERROR, l('error_select_payment_system', 'memberships'));
                } else {
                    $this->Memberships_model->systemMembershipPayment($system_gid, $user_id, $membership_data["id"],
                                                                      $membership_data["price"]);
                    $redirect = $this->session->userdata('service_redirect');
                    $this->session->set_userdata(array('service_redirect' => ''));
                    $this->load->model('users/models/Auth_model');
                    $this->Auth_model->update_user_session_data($user_id);
                }
            }
        }

        if ($membership_data["pay_type"] == Memberships_model::PAYMENT_TYPE_ACCOUNT || $membership_data["pay_type"] == Memberships_model::PAYMENT_TYPE_ACCOUNT_AND_DIRECT) {
            $this->load->model("Users_payments_model");
            $membership_data["user_account"] = $this->Users_payments_model->get_user_account($user_id);
            if ($membership_data["user_account"] <= 0 && $membership_data["price"] > 0) {
                $membership_data["disable_account_pay"] = true;
            } elseif ($membership_data["price"] > $membership_data["user_account"]) {
                $membership_data["disable_account_pay"] = true;
            }
        }

        if ($membership_data["pay_type"] == Memberships_model::PAYMENT_TYPE_ACCOUNT_AND_DIRECT || $membership_data["pay_type"] == Memberships_model::PAYMENT_TYPE_DIRECT) {
            $this->load->model("payments/models/Payment_systems_model");
            $billing_systems = $this->Payment_systems_model->get_active_system_list();
            $this->view->assign('billing_systems', $billing_systems);
        }

        $is_users_payments_installed = $this->pg_module->is_module_active('users_payments');
        $this->view->assign('is_users_payments_installed', $is_users_payments_installed);
        $this->view->assign('membership', $membership_data);

        $this->load->model('Menu_model');
        $this->Menu_model->breadcrumbs_set_parent('memberships_item');
        $this->Menu_model->breadcrumbs_set_active($membership_data['name']);

        $this->view->render('form');
    }

    /**
     * My memberships
     *
     * @param integer $page page of results
     *
     * @return void
     */
    public function my($page = 1)
    {
        $id_user = $this->session->userdata('user_id');

        $params['where']['id_user'] = $id_user;
        $this->load->model('memberships/models/Memberships_users_model');
        $user_memberships_count = $this->Memberships_users_model->getUserMembershipsCount($params);

        $items_on_page = 20;
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $user_memberships_count, $items_on_page);

        $user_memberships = $this->Memberships_users_model->getUserMembershipsList(null, $params);
        $this->view->assign('user_memberships', $user_memberships);

        $this->load->helper("navigation");
        $page_data = get_user_pages_data(site_url() . 'memberships/my/', $user_memberships_count, $items_on_page, $page,
                                         'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->load->model('Menu_model');
        $this->Menu_model->breadcrumbs_set_parent('my_mmeberships_item');

        $this->view->render('my');
    }

    /**
     * Apply membership
     *
     * @return void
     */
    public function apply($membership_gid)
    {
        $this->session->set_userdata(array('service_redirect' => $_SERVER['HTTP_REFERER']));
        $this->load->helper('payments');
        post_location_request(site_url() . 'memberships/form/' . $membership_gid, array());
    }

}
