<?php

namespace Pg\Modules\Payments\Models\Systems;

use Pg\Modules\Payments\Models\Payment_driver_model;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Usaepay payment system driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Usaepay_model extends Payment_driver_model
{

    /**
     *  Payment settings
     *
     *  @return void
     */
    public $payment_data = array();

    /**
     * System settings
     *
     * @var array
     */
    public $settings = array(
        "seller_id" => array("type" => "text", "content" => "string", "size" => "middle"),
    );

    /**
     * Response variables
     *
     * @var array
     */
    protected $variables = array(
    );

    /**
     * Checkout url
     *
     * @var string
     */
    protected $checkout_url = 'https://www.usaepay.com/gate.php';

    /**
     * Checkout url for test mode
     *
     * @var string
     */
    protected $test_checkout_url = '';

    /**
     * Do request to payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return void
     */
    public function func_request($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data);

        $send_data = array(
            "UMkey" => $system_settings["settings_data"]["seller_id"],
            "UMamount" => $payment_data["amount"],
            "UMinvoice" => $payment_data["id_payment"],
            "UMtestmode" => "0",
            "UMredirApproved" => site_url(),
        );

        if (isset($system_settings['is_test']) && $system_settings['is_test'] == 2) {
            $checkout_url = $this->test_checkout_url;
        } else {
            $checkout_url = $this->checkout_url;
        }

        $this->send_data($checkout_url, $send_data, "post");

        return $return;
    }

    /**
     * Process response from payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_responce($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data, "type" => "exit");

        return $return;
    }

    /**
     * Return formatted system settings
     *
     * @return array
     */
    public function get_settings_map()
    {
        foreach ($this->settings as $param_id => $param_data) {
            $this->settings[$param_id]["name"] = l('system_field_' . $param_id, 'payments');
        }

        return $this->settings;
    }

}
