<?php

/**
 * Polls module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Polls management
 *
 * @package     PG_RealEstate
 * @subpackage  Polls
 *
 * @category    helpers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('show_poll_place_block')) {

    /**
     * Display poll block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function show_poll_place_block($params)
    {
        $ci = &get_instance();

        $show_results = isset($params['show_results']) ? $params['show_results'] : false;

        $poll_block = show_poll_place(get($params['id_poll']), $params['one_poll_place'], $show_results);

        if (!$poll_block) {
            return false;
        }
        $ci->view->assign('poll_block', $poll_block);

        return $ci->view->fetch('poll_place_block', null, 'polls');
    }

}

if (!function_exists('show_poll_place')) {

    /**
     * Display poll helper
     *
     * @param integer $id_poll        poll identifier
     * @param integer $one_poll_place once output to user
     * @param boolean $show_results   show results
     *
     * @return string
     */
    function show_poll_place($id_poll, $one_poll_place, $show_results = false)
    {
        $ci = &get_instance();
        $ci->load->model('Polls_model');

        if ($ci->session->userdata("auth_type") == "user") {
            $ci->load->model('Users_model');
            $user_types = $ci->Users_model->getUserTypes();
            $user_types = array_flip($user_types);

            $id_user = $ci->session->userdata('user_id');
            $user_type = $ci->session->userdata("user_type");
            $user_type = $user_types[$user_type] + 3;
        } else {
            $id_user = 0;
            $user_type = null;
        }

        $denied_polls = $ci->Polls_model->get_denied_polls($id_user, $user_type);

        // Show precise poll
        if ($id_poll) {
            // Check polls existence
            if ($ci->Polls_model->is_exists($id_poll)) {
                // If current user may pass the poll
                if (!$show_results && !in_array($id_poll, $denied_polls)) {
                    return get_form($id_poll, $one_poll_place);
                } else {
                    // Template will decide whether to show results or just a text message
                    if ($one_poll_place || $ci->Polls_model->show_results($id_poll)) {
                        return get_results($id_poll, $one_poll_place);
                    } // else show random poll
                }
            } //else show random poll
        } elseif ($one_poll_place) {
            return false;
        }
        // If we reached here, show random poll
        // Get language and user type
        $language = $ci->pg_language->current_lang_id;

        // Pick a random poll...
        $id_poll_rnd = $ci->Polls_model->get_random_id(null, $language, $user_type, $denied_polls);

        if ($id_poll_rnd) {
            return get_form($id_poll_rnd, $one_poll_place);
        } else {
            // ...or results
            $id_poll_rnd = $ci->Polls_model->get_random_id(true);
            if ($id_poll_rnd) {
                return get_results($id_poll_rnd, $one_poll_place);
            }
        }
    }

}

if (!function_exists('get_form')) {

    /**
     * Display poll question form
     *
     * @param integer $id_poll        poll identifier
     * @param integer $one_poll_place once output to user
     *
     * @return string
     */
    function get_form($id_poll, $one_poll_place)
    {
        if (is_null($id_poll)) {
            return false;
        }

        $ci = &get_instance();
        $ci->load->model('Polls_model');

        $poll = $ci->Polls_model->get_poll_by_id($id_poll);
        $language = $ci->pg_language->current_lang_id;

        $params = array('where_in' => array('poll_type' => array(0)));
        if ($ci->session->userdata("auth_type") == "user") {
            $ci->load->model('Users_model');
            $user_types = array_flip($ci->Users_model->getUserTypes());

            $id_user = $ci->session->userdata('user_id');
            $user_type = $ci->session->userdata("user_type");
            $params['where_in']['poll_type'][] = 1;
            $params['where_in']['poll_type'][] = $user_types[$user_type] + 3;

            $denied_polls = $ci->Polls_model->get_denied_polls($id_user);
            if ($denied_polls) {
                $params['where_not_in']['id'] = array_unique($denied_polls);
            }
        } else {
            $params['where_in']['poll_type'][] = 2;
        }

        $polls_count = $ci->Polls_model->get_polls_count($params);

        $ci->view->assign('one_poll_place', $one_poll_place);
        $ci->view->assign('poll_data', $poll);
        $ci->view->assign('cur_lang', $language);
        $ci->view->assign('polls_count', $polls_count);

        $poll_html = $ci->view->fetch('poll_form', 'user', 'polls');

        return $poll_html;
    }

}

if (!function_exists('get_results')) {

    /**
     * Display poll results helper
     *
     * @param integer $id_poll        poll identifier
     * @param integer $one_poll_place once output to user
     *
     * @return string
     */
    function get_results($id_poll, $one_poll_place = false)
    {
        if (is_null($id_poll)) {
            return false;
        }

        $ci = &get_instance();
        $ci->load->model('Polls_model');

        $poll = $ci->Polls_model->get_poll_by_id($id_poll);
        $language = $ci->pg_language->current_lang_id;

        $max_answers = $ci->pg_module->get_module_config('polls', 'max_answers');
        if (!$max_answers) {
            $max_answers = 10;
        }
        $max_results = 0;

        // Results sorting
        for ($i = 1; $i <= $max_answers; ++$i) {
            if (isset($poll['results'][$i])) {
                $max_results = $max_results + floor($poll['results'][$i]);
            }
        }

        if (1 == $poll['sorter']) {
            asort($poll['results']);
        } elseif (2 == $poll['sorter']) {
            arsort($poll['results']);
        }

        $params = array('where_in' => array('poll_type' => array(0)));
        if ($ci->session->userdata("auth_type") == "user") {
            $ci->load->model('Users_model');
            $user_types = array_flip($ci->Users_model->getUserTypes());

            $user_type = $ci->session->userdata("user_type");
            $params['where_in']['poll_type'][] = 1;
            $params['where_in']['poll_type'][] = $user_types[$user_type] + 3;
        } else {
            $params['where_in']['poll_type'][] = 2;
        }
        $polls_count = $ci->Polls_model->get_polls_count($params);

        $ci->view->assign('one_poll_place', $one_poll_place);
        $ci->view->assign('polls_count', $polls_count);
        $ci->view->assign('poll_data', $poll);
        $ci->view->assign('poll_lang', $poll['language']);
        $ci->view->assign('cur_lang', $language);
        $ci->view->assign('max_results', $max_results);
        $ci->view->assign('max_answers', $max_answers);

        $poll_html = $ci->view->fetch('poll_results', null, 'polls');

        return $poll_html;
    }

}

if (!function_exists('show_poll_results_block')) {

    /**
     * Display poll results block (progressbars)
     *
     * @param integer $id_poll poll identifier
     *
     * @return string
     */
    function show_poll_results_block($id_poll)
    {
        $ci = &get_instance();

        if (is_null($id_poll)) {
            return false;
        }
        $poll = $ci->Polls_model->get_poll_by_id($id_poll);

        $poll['show_results'] = true;

        $max_answers = $ci->pg_module->get_module_config('polls', 'max_answers');
        if (!$max_answers) {
            $max_answers = 10;
        }
        $max_results = 0;

        // Results sorting
        for ($i = 1; $i <= $max_answers; ++$i) {
            if (isset($poll['results'][$i])) {
                $max_results = $max_results + floor($poll['results'][$i]);
            }
        }

        $ci->view->assign('poll_data', $poll);
        $ci->view->assign('max_results', $max_results);
        $ci->view->assign('max_answers', $max_answers);
        $ci->view->assign('poll_block', $ci->view->fetch('poll_results', null, 'polls'));
        return $ci->view->fetch('poll_place_block', null, 'polls');
    }

}

if (!function_exists('admin_home_polls_block')) {

    /**
     * Display homepage polls block in admin mode
     *
     * @return string
     */
    function admin_home_polls_block()
    {
        $ci = &get_instance();

        $auth_type = $ci->session->userdata("auth_type");
        if ($auth_type != "admin") {
            return '';
        }

        $user_type = $ci->session->userdata("user_type");

        $show = true;

        if ($user_type == 'moderator') {
            $show = false;
            $ci->load->model('Moderators_model');
            $methods = $ci->Moderators_model->get_module_methods('polls');
            if (is_array($methods) && !in_array('index', $methods)) {
                $show = true;
            } else {
                $permission_data = $ci->session->userdata("permission_data");
                if (isset($permission_data['polls']['index']) && $permission_data['polls']['index'] == 1) {
                    $show = true;
                }
            }
        }

        if (!$show) {
            return '';
        }

        $ci->load->model('Polls_model');

        $stat_polls["all"] = $ci->Polls_model->get_polls_count(array());
        $params["where_sql"][] = "( ( use_expiration = 0 OR (use_expiration = 1 AND date_end >= '" . date('Y-m-d H:i:s') . "') )  AND date_start < '" . date('Y-m-d H:i:s') . "' )";
        $stat_polls["active"] = $ci->Polls_model->get_polls_count($params);

        $ci->view->assign("stat_polls", $stat_polls);

        return $ci->view->fetch('helper_admin_home_block', 'admin', 'polls');
    }

}

if (!function_exists('polls_get_user_types')) {

    /**
     * Display user types for polls module
     *
     * @return string
     */
    function polls_get_user_types()
    {
        $ci = &get_instance();

        return ld('user_type', 'users');
    }

}
