<?php

namespace Pg\Modules\Properties\Models;

/**
 * Properties module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Properties install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Properties
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Properties_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        // admin menu
        'admin_menu' => array(
            'name' => 'Admin area menu',
            "action" => "none",
            "items" => array(
                'content_items' => array(
                    "action" => "none",
                    "items" => array(
                        'properties_items' => array("action" => "create", 'link' => 'admin/properties/index', 'status' => 1, 'sorter' => 3),
                    ),
                ),
            ),
        ),
    );

    /**
     * Moderators configuration
     *
     * @var array
     */
    protected $moderators_methods = array(
        array("module" => 'properties', 'method' => 'index', 'is_default' => 1),
        array("module" => 'properties', 'method' => 'property', 'is_default' => 0),
    );

    /**
     * Constructor
     *
     * @return Properties_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Install data of menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->model('Menu_model');
        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages idnetifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('properties', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');

            return false;
        }

        $this->ci->load->model('Menu_model');
        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid,
                                              $langs_file);
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model('Menu_model');
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid,
                                                      $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall data of menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->model('Menu_model');
        $this->ci->load->helper('menu');
        linked_install_delete_menu_items($menu_gid, $items);
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]["items"]);
            }
        }
    }

    /**
     * Install data of moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        ///// install moderators permissions
        $this->ci->load->model("moderators/models/Moderators_model");

        foreach ($this->moderators_methods as $method) {
            $this->ci->Moderators_model->save_method(null, $method);
        }
    }

    /**
     * Import languages of moderators module
     *
     * @param array $langs_ids languages idnetifiers
     *
     * @return void
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read('properties', 'moderators', $langs_ids);

        ///// install moderators permissions
        $this->ci->load->model("moderators/models/Moderators_model");
        $params['where']['module'] = 'properties';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method['method']])) {
                $this->ci->Moderators_model->save_method($method["id"], array(), $langs_file[$method['method']]);
            }
        }
    }

    /**
     * Export languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model("moderators/models/Moderators_model");
        $params['where']['module'] = 'properties';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method['method']] = $method['langs'];
        }

        return array('moderators' => $return);
    }

    /**
     * Uninstall data of moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        ///// delete moderation methods in moderators
        $this->ci->load->model("moderators/models/Moderators_model");
        $params['where']['module'] = 'properties';
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install module data
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        
    }

    /**
     * Import module languages
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function _arbitrary_lang_install($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }

        $this->ci->load->model("Install_model");
        $this->ci->load->model("Properties_model");

        /// install properties
        $langs_file = $this->ci->Install_model->language_file_read('properties', 'demo', $langs_ids);

        $default_lang_id = $this->ci->pg_language->get_default_lang_id();
        $module_gid = $this->ci->Properties_model->module_gid;
        $properties = array_merge($this->ci->Properties_model->categories, $this->ci->Properties_model->properties);

        foreach ($properties as $ref_gid) {
            foreach ($langs_ids as $lang_id) {
                if (!empty($langs_file[$ref_gid][$lang_id])) {
                    $value = $langs_file[$ref_gid][$lang_id];
                } elseif (!empty($langs_file[$ref_gid][$default_lang_id])) {
                    $value = $langs_file[$ref_gid][$default_lang_id];
                } else {
                    $value = array();
                }
                if (!empty($value)) {
                    $this->ci->pg_language->ds->set_module_reference($module_gid, $ref_gid, $value, $lang_id);
                }
            }
        }

        return true;
    }

    /**
     * Export module languages
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }

        $this->ci->load->model("Properties_model");
        $module_gid = $this->ci->Properties_model->module_gid;
        $properties = array_merge($this->ci->Properties_model->categories, $this->ci->Properties_model->properties);

        $demo_return = array();

        /// properties
        foreach ($langs_ids as $lang_id) {
            foreach ($properties as $ref_gid) {
                $demo_return[$ref_gid][$lang_id] = $this->ci->pg_language->ds->get_reference($module_gid, $ref_gid,
                                                                                             $lang_id);
            }
        }

        return array("demo" => $demo_return);
    }

    /**
     * Uninstall module data
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        $this->ci->load->model("Install_model");
        $this->ci->Install_model->_language_deinstall('data_properties');
    }

}
