<?php

namespace Pg\Modules\Social_networking\Models;

/**
 * Social networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Social networking widgets model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Social networking
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Social_networking_widgets_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Available locales
     *
     * @var array
     */
    public $locales = array();

    /**
     * Current locale data
     *
     * @var array
     */
    public $locale = array();

    /**
     * Class constructor
     *
     * @return Social_networking_widgets_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        if (count($this->locales) == 0) {
            if (@require(APPPATH . 'config/locales' . EXT)) {
                $this->locales = $locales;
            }
        }
        $lang = $this->ci->pg_language->get_lang_by_id($this->ci->pg_language->current_lang_id);
        $lang_code = isset($lang['code']) ? $lang['code'] : false;
        $locale = isset($this->locales[$lang_code]) ? $this->locales[$lang_code] : 'en_US';
        $this->locale = $locale;
    }

    /**
     * Return objects of all active services as array
     *
     * @return array
     */
    protected function _get_active_services()
    {
        static $services = null;
        if (is_null($services)) {
            $this->ci->load->model('social_networking/models/Social_networking_services_model');
            $services = $this->ci->Social_networking_services_model->get_services_list(
                    array('id' => 'ASC'), array('where' => array('status' => 1))
            );
            if (!$services) {
                $services = array();
            }
        }

        return $services;
    }

    /**
     * Return widget objects as array
     *
     * @param string $widget       widget GUID
     * @param array  $settings     widget settings of services
     * @param string $display_type output type
     *
     * @return string
     */
    public function get_widgets($widget = '', $settings = array(), $display_type = 'row')
    {
        $widgets = array();
        $active_services = $this->_get_active_services();
        $ns_widgets = NS_MODULES . 'Social_networking\\Models\\Widgets\\';
        foreach ($active_services as $active_service) {
            if (empty($settings[$active_service['gid']])) {
                continue;
            }
            $service_model = ucfirst($active_service['gid'] . '_widgets_model');
            $widget_class = $ns_widgets . $service_model;
            if (class_exists($widget_class)) {
                $widget_obj = new $widget_class();
                if (in_array($widget, $widget_obj->widget_types)) {
                    $method = 'get_' . $widget;
                    if (method_exists($widget_obj, 'get_header')) {
                        include(APPPATH . 'config/locales.php');
                        if (!empty($locales[$this->ci->pg_language->current_lang['code']])) {
                            $locale = $locales[$this->ci->pg_language->current_lang['code']];
                        } else {
                            $locale = '';
                        }                        
                        $widget_obj->get_header($active_service, $locale, array('like', 'share', 'comments'));
                    }
                    if (method_exists($widget_obj, $method)) {
                        $text = $widget_obj->$method();
                        if (empty($text)) {
                            continue;
                        }
                        $widgets[] = $text;
                    }
                }
            }
        }

        if (empty($widgets)) {
            return '';
        }

        $this->ci->view->assign('display_type', $display_type);
        $this->ci->view->assign('social_widgets', $widgets);

        return $this->ci->view->fetch('widgets', 'user', 'social_networking');
    }

    /**
     * Return header code of widgets
     *
     * @return string
     */
    public function get_header()
    {
        $header_text = '';
        $services = $this->_get_active_services();
        $ns_widgets = NS_MODULES . 'Social_networking\\Models\\Widgets\\';
        foreach ($services as $value) {
            if(empty($value['gid'])) {
                log_message('error', '(social_networking) Empty service gid');
                continue;
            }
            $service_model = ucfirst($value['gid'] . '_widgets_model');
            //$service_file = APPPATH . 'modules/social_networking/models/widgets/' . $service_model . '.php';
            $widget_class = $ns_widgets . $service_model;
            if (class_exists($widget_class)) {
                $service = new $widget_class();
                $method = 'get_header';
                if (method_exists($service, $method)) {
                    $header_text .= $service->$method($value, $this->locale, array('like', 'share', 'comments'));
                }
            }
        }
        return $header_text;
    }

    /**
     * Return available widgets actions as array
     *
     * @param array $services social services
     *
     * @return array
     */
    public function get_widgets_actions($services = array())
    {
        $actions = array();
        if (count($services) === 0) {
            return $actions;
        }
        $ns_widgets = NS_MODULES . 'Social_networking\\Models\\Widgets\\';
        foreach ($services as $id => $value) {
            if(empty($value['gid'])) {
                log_message('error', '(social_networking) Empty gid');
                continue;
            }
            $service_model = ucfirst($value['gid'] . '_widgets_model');
            $widget_class = $ns_widgets . $service_model;
            if (class_exists($widget_class)) {
                $service = new $widget_class();
                $actions[$id] = isset($service->widget_types) ? $service->widget_types : array();
            } else {
                log_message('error', '(social_networking) Empty widget class ' . $widget_class);
            }
        }
        return $actions;
    }

}
