<?php

namespace Pg\Modules\Social_networking\Models\Widgets;

/**
 * Social networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Social networking google widgets model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Social networking
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Google_widgets_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Available widgets
     *
     * @var array
     */
    public $widget_types = array('like', 'share');

    /**
     * Page header data is loaded
     *
     * @var boolean
     */
    public $head_loaded = false;

    /**
     * Class constructor
     *
     * @return Google_widgets_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return widgets data for page header
     *
     * @param array  $service_data service data
     * @param string $locale       locale code
     * @param array  $types        widget types
     *
     * @return string
     */
    public function get_header($service_data = array(), $locale = '', $types = array())
    {
        $lang_id = $this->ci->pg_language->current_lang_id;
        $lang = $this->ci->pg_language->get_lang_by_id($lang_id);
        if (empty($lang['code']) || count(array_intersect($this->widget_types, $types)) == 0) {
            return '';
        }

        $this->head_loaded = true;

        return '
	<script type="text/javascript">
		window.___gcfg = {lang: "' . $lang['code'] . '"};
		(function() {
			var po = document.createElement("script");
			po.type = "text/javascript"; po.async = true;
			po.src = "https://apis.google.com/js/plusone.js";
			var s = document.getElementsByTagName("script")[0];
			s.parentNode.insertBefore(po, s);
		})();
	</script>';
    }

    /**
     * Return code of 'Like' widget
     *
     * @return string
     */
    public function get_like()
    {
        return $this->head_loaded ? '<g:plusone></g:plusone>' : '';
    }

    /**
     * Return code of 'Share' widget
     *
     * @return string
     */
    public function get_share()
    {
        return $this->head_loaded ? '<div class="g-plus" data-action="share"></div>' : '';
    }

}
