<?php

namespace Pg\Modules\Upload_gallery\Controllers;

/**
 * Upload gallery module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Upload gallery admin side controller
 *
 * @package     PG_RealEstate
 * @subpackage  Upload gallery
 *
 * @category    controllers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Upload_gallery extends \Controller
{

    /**
     * Allow create gallery type
     *
     * @var boolean
     */
    protected $allow_gallery_add = false;

    /**
     * Constructor
     *
     * @return Admin_Upload_gallery
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Menu_model");
        $this->Menu_model->set_menu_active_item("admin_menu", "uploads-items");
        $this->load->model("Upload_gallery_model");
    }

    /**
     * Manage upload gallery type
     *
     * @return void
     */
    public function index()
    {
        $types = $this->Upload_gallery_model->get_types_list();
        $this->view->assign("types", $types);

        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $this->view->assign("allow_add", $this->allow_gallery_add);
        $this->view->setHeader(l("admin_header_list", "upload_gallery"));
        $this->view->render("list");
    }

    /**
     * Edit gallery type action
     *
     * @param integer $id gallery type identifier
     *
     * @return void
     */
    public function edit($id = null)
    {
        $this->load->model("Upload_gallery_model");
        $data = ($id) ? $this->Upload_gallery_model->get_type_by_id($id) : array();

        if ($this->input->post("btn_save")) {
            $post_data = $this->input->post("data", true);

            if (!$this->allow_gallery_add && isset($post_data["gid"])) {
                unset($post_data["gid"]);
            }

            $validate_data = $this->Upload_gallery_model->validate_type($id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                if ($id) {
                    $this->Upload_gallery_model->set_type($id, $validate_data["data"]);
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_update_type", "upload_gallery"));
                } else {
                    $this->Upload_gallery_model->set_type($id, $validate_data["data"]);
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_update_type", "upload_gallery"));
                }
                redirect(site_url() . "admin/upload_gallery/index");
            }
            $data = array_merge($data, $validate_data["data"]);
        }
        $this->view->assign("data", $data);
        $this->view->assign("allow_add", $this->allow_gallery_add);

        ///// Configs
        $this->load->model("uploads/models/Uploads_config_model");
        $settings = $this->Uploads_config_model->get_config_list();
        $this->view->assign("upload_config", $settings);

        $this->view->setHeader(l("admin_header_edit", "upload_gallery"));
        $this->view->render("edit_form");
    }

    /**
     * Remove gallery type
     *
     * @param integer $id gallery type identifier
     *
     * @return void
     */
    public function delete($id)
    {
        if (!empty($id)) {
            $this->Upload_gallery_model->delete_type($id);
        }
        redirect(site_url() . "admin/upload_gallery/index");
    }

    /**
     * Recrop upload
     *
     * @param integer $upload_id    upload identifier
     * @param string  $thumb_prefix thumb prefix
     *
     * @return void
     */
    public function ajax_recrop($upload_id, $thumb_prefix = '')
    {
        $upload_id = intval($upload_id);
        if (!$upload_id) {
            show_404();
        }

        $photo = $this->Upload_gallery_model->get_file_by_id($upload_id);

        $type_data = $photo['type_data'];

        if (!$thumb_prefix) {
            $thumb_prefix = $this->input->post("prefix", true);
        }
        $this->Upload_gallery_model->recrop_file($upload_id, $thumb_prefix);
        $return['success'] = l('success_' . $type_data['gid'] . '_recrop', 'upload_gallery');

        $this->view->assign($return);

        $this->view->render();
    }

    /**
     * Rotate upload
     *
     * @param integer        $upload_id upload identifier
     * @param integer/string $angle     rotate angle
     *
     * @return void
     */
    public function ajax_rotate($upload_id, $angle = 90)
    {
        $upload_id = intval($upload_id);
        if (!$upload_id) {
            show_404();
        }

        $photo = $this->Upload_gallery_model->get_file_by_id($upload_id);

        $type_data = $photo['type_data'];

        if ($angle < 0) {
            $angle += 360;
        }

        $data = $this->Upload_gallery_model->rotate_file($upload_id, $angle);

        $return['width'] = $data['settings']['width'];
        $return['height'] = $data['settings']['height'];
        $return['success'] = l('success_' . $type_data['gid'] . '_rotate', 'upload_gallery');

        $this->view->assign($return);

        $this->view->render();
    }

    /**
     * Upload photo to gallery
     *
     * @param integer $gallery_type_id gallery type identifier
     * @param integer $object_id       object identifier
     * @param integer $photo_id        photo identifier
     *
     * @return void
     */
    public function photo_upload($gallery_type_id, $object_id, $photo_id = 0)
    {
        $is_ajax = $this->input->is_ajax_request();

        $redirect = $this->input->post('redirect');

        $gallery_type_id = intval($gallery_type_id);
        if (!$gallery_type_id) {
            show_404();
        }

        $object_id = intval($object_id);
        if (!$object_id) {
            show_404();
        }

        $photo_id = intval($photo_id);

        $type_data = $this->Upload_gallery_model->get_type_by_id($gallery_type_id);

        if ($photo_id) {
            $photo = $this->Upload_gallery_model->get_file_by_id($photo_id);
            if ($object_id != $photo['object_id']) {
                show_404();
            }
        }

        $data = array('type_id' => $type_data['id'], 'object_id' => $object_id);
        $return = $this->Upload_gallery_model->save_file($photo_id, $data, 'photo_file', true);
        if (empty($return['errors'])) {
            if (strtolower($type_data["module"] . '_model') != strtolower($type_data["model"])) {
                $model_path = $type_data["module"] . "/models/" . $type_data["model"];
            } else {
                $model_path = $type_data["model"];
            }
            try {
                $this->load->model($model_path);
                $this->{$type_data["model"]}->{$type_data["callback"]}("upload_file", $object_id);
            } catch (Exception $e) {
                show_404();
            }
        }

        if ($is_ajax) {
            $return['errors'] = $return['error'] = implode('<br>', $return['errors']);

            $this->view->assign($return);

            $this->view->render();
            return;
        } else {
            if (!empty($return['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $return['errors']);
            }
            redirect($redirect);
        }
    }

    /**
     * Save photo data by ajax
     *
     * @param integer $gallery_type_id gallery type identifier
     * @param integer $object_id       object identifier
     * @param integer $photo_id        photo identifier
     *
     * @return void
     */
    public function ajax_save_data($gallery_type_id, $object_id, $photo_id)
    {
        $gallery_type_id = intval($gallery_type_id);
        if (!$gallery_type_id) {
            show_404();
        }

        $photo_id = intval($photo_id);
        if (!$photo_id) {
            show_404();
        }

        $photo = $this->Upload_gallery_model->get_file_by_id($photo_id);
        if ($photo['object_id'] != $object_id) {
            show_404();
        }

        $type_data = $this->Upload_gallery_model->get_type_by_id($gallery_type_id);

        $post_data['comment'] = $this->input->post('comment', true);
        $validate_data = $this->Upload_gallery_model->validate_file_data($photo_id, $type_data['id'], $post_data);
        if (!empty($validate_data['errors'])) {
            $return['error'] = implode('<br>', $validate_data['errors']);
        } else {
            $return = $this->Upload_gallery_model->save_file($photo_id, $validate_data['data'], '', true);
            if (!empty($return['errors'])) {
                $return['error'] = implode('<br>', $return['errors']);
            } else {
                $return['success'] = l('success_' . $type_data['gid'] . '_saved', 'upload_gallery');
            }
        }

        $this->view->assign($return);

        $this->view->render();
    }

    /**
     * Resort gallery photos
     *
     * @param integer $gallery_type_id gallery type identifier
     * @param integer $object_id       object identifier
     *
     * @return void
     */
    public function ajax_save_sorting($gallery_type_id, $object_id)
    {
        $gallery_type_id = intval($gallery_type_id);
        if (!$gallery_type_id) {
            show_404();
        }

        $object_id = intval($object_id);
        if (!$object_id) {
            show_404();
        }

        $type_data = $this->Upload_gallery_model->get_type_by_id($gallery_type_id);

        $sorter = (array) $this->input->post('sorter', true);
        foreach ($sorter as $item_str => $sort_index) {
            $photo_id = intval(str_replace('pItem', '', $item_str));
            $this->Upload_gallery_model->save_sorter($photo_id, $sort_index);
        }

        if (strtolower($type_data["module"] . '_model') != strtolower($type_data["model"])) {
            $model_path = $type_data["module"] . "/models/" . $type_data["model"];
        } else {
            $model_path = $type_data["model"];
        }
        try {
            $this->load->model($model_path);
            $this->{$type_data["model"]}->{$type_data["callback"]}("sorting_data", $object_id);
        } catch (Exception $e) {

        }

        $return['success'] = l('success_' . $type_data['gid'] . '_sorter', 'upload_gallery');

        $this->view->assign($return);

        $this->view->render();
    }

    /**
     * Remove photo by ajax
     *
     * @param integer $gallery_type_id gallery type identifier
     * @param integer $object_id       object identifier
     * @param integer $photo_id        photo identifier
     *
     * @return void
     */
    public function ajax_delete_data($gallery_type_id, $object_id, $photo_id)
    {
        $gallery_type_id = intval($gallery_type_id);
        if (!$gallery_type_id) {
            show_404();
        }

        $object_id = intval($object_id);
        if (!$object_id) {
            show_404();
        }

        $photo_id = intval($photo_id);
        if (!$photo_id) {
            show_404();
        }

        $photo = $this->Upload_gallery_model->get_file_by_id($photo_id);
        if ($object_id != $photo['object_id']) {
            show_404();
        }

        $type_data = $this->Upload_gallery_model->get_type_by_id($gallery_type_id);

        $this->Upload_gallery_model->delete_file($photo_id);

        $return['success'] = l('success_' . $type_data['gid'] . '_deleted', 'upload_gallery');

        $this->view->assign($return);

        $this->view->render();
    }

    /**
     * Render edit form by ajax
     *
     * @param integer $gallery_type_id gallery type idnetifier
     * @param integer $object_id       object identifier
     * @param integer $photo_id        photo identifier
     *
     * @return void
     */
    public function ajax_edit_form($gallery_type_id, $object_id, $photo_id = 0)
    {
        $gallery_type_id = intval($gallery_type_id);
        if (!$gallery_type_id) {
            show_404();
        }

        $object_id = intval($object_id);
        if (!$object_id) {
            show_404();
        }

        $type_data = $this->Upload_gallery_model->get_type_by_id($gallery_type_id);

        $photo_id = intval($photo_id);
        if ($photo_id) {
            $photo = $this->Upload_gallery_model->get_file_by_id($photo_id);
            if ($photo['object_id'] != $object_id) {
                show_404();
            }
            $this->view->assign('photo_data', $photo);
        }

        $this->load->model('uploads/models/Uploads_config_model');
        $upload_config = $this->Uploads_config_model->get_config_by_gid($type_data['gid_upload_config']);
        $upload_config['file_formats'] = implode(', ', unserialize($upload_config['file_formats']));
        $upload_config['requirements_str'] = l('text_upload_settings', 'upload_gallery');
        $upload_config['requirements_str'] = str_replace('[width]', $upload_config['max_width'],
                                                         $upload_config['requirements_str']);
        $upload_config['requirements_str'] = str_replace('[height]', $upload_config['max_height'],
                                                         $upload_config['requirements_str']);
        $upload_config['requirements_str'] = str_replace('[formats]', $upload_config['file_formats'],
                                                         $upload_config['requirements_str']);
        $this->view->assign('upload_config', $upload_config);

        $this->view->assign('gallery_type', $type_data['gid']);
        $this->view->assign('gallery_type_id', $gallery_type_id);

        $selections = array();
        $thumbs_config = $this->Uploads_config_model->get_config_thumbs($upload_config['id']);
        foreach ($thumbs_config as $thumb_config) {
            $selections[$thumb_config['prefix']] = array(
                'width' => $thumb_config['width'],
                'height' => $thumb_config['height'],
            );
        }
        $this->view->assign('selections', $selections);

        $gallery_object = $this->input->post('object', true);
        $this->view->assign('gallery_object', $gallery_object);

        $gallery_thumb = $this->input->post('thumb', true);
        $this->view->assign('gallery_thumb', $gallery_thumb);

        $gallery_panorama = $this->input->post('panorama', true);
        $this->view->assign('gallery_panorama', $gallery_panorama);

        $redirect = $this->input->post('redirect', true);
        $this->view->assign('redirect', $redirect);

        $aviary_post_data = array(
            'type' => $gallery_type_id,
            'object_id' => $object_id,
            'photo_id' => $photo_id,
        );

        $this->view->assign('aviary_post_data', $aviary_post_data);

        $rand = rand(100000, 999999);
        $this->view->assign('photo_form_rand', $rand);

        $this->view->render('photo_edit_form');
    }

    /**
     * Reload upload photo block by ajax
     *
     * @param integer $gallery_type_id gallery type idnetifier
     * @param integer $object_id       object identifier
     * @param integer $photo_id        photo identifier
     *
     * @return void
     */
    public function ajax_reload_block($gallery_type_id, $object_id, $photo_id)
    {
        $gallery_type_id = intval($gallery_type_id);
        if (!$gallery_type_id) {
            show_404();
        }

        $object_id = intval($object_id);
        if (!$object_id) {
            show_404();
        }

        $type_data = $this->Upload_gallery_model->get_type_by_id($gallery_type_id);

        $photo_id = intval($photo_id);
        if (!$photo_id) {
            show_404();
        }

        $photo = $this->Upload_gallery_model->get_file_by_id($photo_id);
        if ($object_id != $photo['object_id']) {
            show_404();
        }

        $this->view->assign('gallery_type', $type_data['gid']);

        $gallery_object = $this->input->post('object', true);
        $this->view->assign('gallery_object', $gallery_object);

        $gallery_thumb = $this->input->post('thumb', true);
        $this->view->assign('gallery_thumb', $gallery_thumb);

        $gallery_full = $this->input->post('full', true);
        $this->view->assign('gallery_full', $gallery_full);

        $gallery_panorama = $this->input->post('panorama', true);
        $this->view->assign('gallery_panorama', $gallery_panorama);

        $this->view->assign('photo_data', $photo);
        $this->view->render('photo_view_block');
    }

    /**
     * Reload panorama block by ajax
     *
     * @param integer $gallery_type_id gallery type idnetifier
     * @param integer $object_id       object identifier
     * @param integer $panorama_id     panorama identifier
     *
     * @return void
     */
    public function ajax_reload_panorama($gallery_type_id, $object_id, $panorama_id)
    {
        $gallery_type_id = intval($gallery_type_id);
        if (!$gallery_type_id) {
            show_404();
        }

        $object_id = intval($object_id);
        if (!$object_id) {
            show_404();
        }

        $panorama_id = intval($panorama_id);
        if (!$panorama_id) {
            show_404();
        }

        $panorama = $this->Upload_gallery_model->get_file_by_id($panorama_id);
        if ($object_id != $panorama['object_id']) {
            show_404();
        }

        $width = $this->input->post('width', true);
        $height = $this->input->post('height', true);
        $thumb_name = $this->input->post('thumb', true);
        $thumb_full = $this->input->post('full', true);

        $data = array('data' => $panorama, 'width' => $width, 'height' => $height, 'thumb_name' => $thumb_name, 'thumb_full' => $thumb_full);

        $this->load->helpers('upload_gallery');
        $this->view->output(show_panorama_block($data));
        
        $this->view->render();
    }

}
