<?php

namespace Pg\Modules\Uploads\Controllers;

/**
 * Uploads module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Uploads admin side controller
 *
 * @package PG_RealEstate
 * @subpackage Uploads
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Uploads extends \Controller
{

    /**
     * Allow to create config settings
     *
     * @var object
     */
    protected $allow_config_add = false;

    /**
     * Constructor
     *
     * @return Admin_Uploads
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'uploads-items');

        $this->load->model('uploads/models/Uploads_config_model');
    }

    /**
     * Manage module settings
     *
     * @return void
     */
    public function index()
    {
        $this->configs();
    }

    /**
     * Manage upload configurations
     *
     * @return void
     */
    public function configs()
    {
        $this->view->assign('configs', $this->Uploads_config_model->get_config_list());
        $this->view->assign('allow_config_add', $this->allow_config_add);

        $this->Menu_model->set_menu_active_item('admin_uploads_menu', 'configs_list_item');
        $this->view->setHeader(l('admin_header_configs_list', 'uploads'));
        $this->view->render('list_settings');
    }

    /**
     * Edit upload configuration settings
     *
     * @param integer $config_id configuration identifier
     *
     * @return void
     */
    public function config_edit($config_id = null)
    {
        if ($config_id) {
            $data = $this->Uploads_config_model->get_config_by_id($config_id);
        } else {
            $data = array();
        }

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "name" => $this->input->post('name', true),
                "gid" => $this->input->post('gid', true),
                'min_height' => $this->input->post('min_height', true),
                'min_width' => $this->input->post('min_width', true),
                "max_height" => $this->input->post('max_height', true),
                "max_width" => $this->input->post('max_width', true),
                "max_size" => $this->input->post('max_size', true),
                "name_format" => $this->input->post('name_format', true),
                "file_formats" => $this->input->post('file_formats', true),
                "default_img" => $_FILES['default_img'],
                "indicator_img" => $_FILES['indicator_img'],
            );
            $validate_data = $this->Uploads_config_model->validate_config($config_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = array_merge($data, $validate_data["data"]);
            } else {
                $data = $validate_data["data"];
                $this->Uploads_config_model->save_config($config_id, $data);

                if ($config_id) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_updated_config', 'uploads'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_added_config', 'uploads'));
                }

                $url = site_url() . "admin/uploads/configs";
                redirect($url);
            }
        }

        $this->view->assign('data', $this->Uploads_config_model->format_config($data));
        if ($config_id) {
            $this->view->assign('thumbs', $this->Uploads_config_model->get_config_thumbs($config_id));
        }
        $this->view->assign('formats', $this->Uploads_config_model->file_formats);
        $this->view->assign('lang_name_format', ld('upload_name_format', 'uploads'));
        $this->view->setHeader(l('admin_header_configs_list', 'uploads'));
        $this->view->render('edit_settings');
    }

    /**
     * Remove upload configuration
     *
     * @param integer $config_id configuration identifier
     *
     * @return void
     */
    public function config_delete($config_id)
    {
        $this->Uploads_config_model->delete_config($config_id);
        $url = site_url() . "admin/uploads/configs";
        redirect($url);
    }

    /**
     * Manage upload configuration thumbnails
     *
     * @param integer $config_id configuration identifier
     *
     * @return void
     */
    public function config_thumbs($config_id)
    {
        $config_data = $this->Uploads_config_model->get_config_by_id($config_id);
        $this->view->assign('thumbs', $this->Uploads_config_model->get_config_thumbs($config_id));

        $use_modify_thumbs = $this->pg_module->get_module_config('uploads', 'use_modify_thumbs');
        $this->view->assign("use_modify_thumbs", $use_modify_thumbs);

        $this->view->assign('config_id', $config_id);
        $this->Menu_model->set_menu_active_item('admin_uploads_menu', 'configs_list_item');
        $this->view->setHeader(l('admin_header_thumbs_list', 'uploads') . " " . $config_data["name"]);
        $this->view->setBackLink(site_url() . "admin/uploads/configs");
        $this->view->render('list_thumbs');
    }

    /**
     * Edit thumb settings
     *
     * @param integer $config_id configuration identifier
     * @param integer $thumb_id  thumbnail identifier
     *
     * @return void
     */
    public function thumb_edit($config_id, $thumb_id = null)
    {
        if ($thumb_id) {
            $data = $this->Uploads_config_model->get_thumb_by_id($thumb_id);
        } else {
            $data = array();
        }

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "prefix" => $this->input->post('prefix', true),
                "width" => $this->input->post('width', true),
                "height" => $this->input->post('height', true),
                "effect" => $this->input->post('effect', true),
                "watermark_id" => $this->input->post('watermark_id', true),
                "crop_param" => $this->input->post('crop_param', true),
                "crop_color" => $this->input->post('crop_color', true),
                "config_id" => $config_id,
            );
            $validate_data = $this->Uploads_config_model->validate_thumb($thumb_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = array_merge($data, $validate_data["data"]);
            } else {
                $data = $validate_data["data"];
                $this->Uploads_config_model->save_thumb($thumb_id, $data);

                if ($thumb_id) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_updated_thumb', 'uploads'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_added_thumb', 'uploads'));
                }

                $url = site_url() . "admin/uploads/config_thumbs/" . $config_id;
                redirect($url);
            }
        }

        $this->view->assign('data', $this->Uploads_config_model->format_thumb($data));
        $this->view->assign('lang_thumb_crop_param', ld('thumb_crop_param', 'uploads'));
        $this->view->assign('lang_thumb_effect', ld('thumb_effect', 'uploads'));

        $this->view->assign('watermarks', $this->Uploads_config_model->get_watermark_list());
        $this->view->assign('config_id', $config_id);

        $this->view->setHeader(l('admin_header_thumb_edit', 'uploads'));
        $this->view->render('edit_thumb');
    }

    /**
     * Remove thumbnail
     *
     * @param integer $config_id configuration identifier
     * @param integer $thumb_id  thumbnail identifier
     *
     * @return void
     */
    public function thumb_delete($config_id, $thumb_id)
    {
        $this->Uploads_config_model->delete_thumb($thumb_id);
        $url = site_url() . "admin/uploads/config_thumbs/" . $config_id;
        redirect($url);
    }

    /**
     * Manage watermarks
     *
     * @return void
     */
    public function watermarks()
    {
        $this->view->assign('watermarks', $this->Uploads_config_model->get_watermark_list());

        $this->view->setHeader(l('admin_header_watermarks_list', 'uploads'));
        $this->view->render('list_watermarks');
    }

    /**
     * Edit watermark data
     *
     * @param integer $watermark_id watermark identifier
     *
     * @return void
     */
    public function watermark_edit($watermark_id = null)
    {
        if ($watermark_id) {
            $data = $this->Uploads_config_model->get_watermark_by_id($watermark_id);
        } else {
            $data["id"] = 0;
        }

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "name" => $this->input->post('name', true),
                "gid" => $this->input->post('gid', true),
                "position_hor" => $this->input->post('position_hor', true),
                "position_ver" => $this->input->post('position_ver', true),
                "alpha" => $this->input->post('alpha', true),
                "wm_type" => $this->input->post('wm_type', true),
                "img" => $_FILES['img'],
                "font_size" => $this->input->post('font_size', true),
                "font_color" => $this->input->post('font_color', true),
                "font_face" => $this->input->post('font_face', true),
                "font_text" => $this->input->post('font_text', true),
                "shadow_color" => $this->input->post('shadow_color', true),
                "shadow_distance" => $this->input->post('shadow_distance', true),
            );
            $validate_data = $this->Uploads_config_model->validate_watermark($watermark_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = array_merge($data, $validate_data["data"]);
            } else {
                $data = $validate_data["data"];
                $this->Uploads_config_model->save_watermark($watermark_id, $data);

                if ($watermark_id) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_updated_watermark', 'uploads'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_added_watermark', 'uploads'));
                }

                $url = site_url() . "admin/uploads/watermarks";
                redirect($url);
            }
        }

        $this->view->assign('data', $this->Uploads_config_model->format_watermark($data));
        $this->view->assign('lang_positions_hor', ld('wm_position_hor', 'uploads'));
        $this->view->assign('lang_positions_ver', ld('wm_position_ver', 'uploads'));
        $this->view->assign('lang_font_face', ld('wm_font_face', 'uploads'));
        $this->view->assign('lang_wm_type', ld('wm_type', 'uploads'));
        $this->view->assign('watermark_test',
                            $this->Uploads_config_model->default_url . $this->Uploads_config_model->watermark_test_image);
        $this->view->assign('wm_text_limits', $this->Uploads_config_model->wm_text_limits);

        $this->wm_clear_preview_data();

        $this->view->assign('rand', rand(10000, 99999));

        $this->view->setHeader(l('admin_header_watermarks_edit', 'uploads'));
        $this->view->render('edit_watermark');
    }

    /**
     * Remove watermark
     *
     * @param integer $wm_id watermark identifier
     *
     * @return void
     */
    public function watermark_delete($wm_id)
    {
        $this->Uploads_config_model->delete_watermark($wm_id);
        $url = site_url() . "admin/uploads/watermarks";
        redirect($url);
    }

    /**
     * Preview watermark
     *
     * @param integer $wm_id           watermark identifier
     * @param string  $temp_data_index user data index
     *
     * @return void
     */
    public function wm_preview($wm_id = null, $temp_data_index = '')
    {
        $this->load->model('Uploads_model');
        $file = $this->Uploads_config_model->default_path . $this->Uploads_config_model->watermark_test_image;
        $sess_data = $this->session->all_userdata();

        if (!empty($temp_data_index) && isset($sess_data[$temp_data_index])) {
            $data = $sess_data[$temp_data_index];
        } else {
            $data = array();
        }
        $errors = $this->Uploads_model->watermark($file, $wm_id, $data, true);
    }

    /**
     * Set data to preview watermark
     *
     * @param integer $type            watermark type
     * @param string  $temp_data_index user data index
     *
     * @return void
     */
    public function wm_save_preview_data($type = 'file', $temp_data_index = 'wm_preview')
    {
        $sess_data = $this->session->all_userdata();
        $sdata[$temp_data_index] = $sess_data[$temp_data_index];
        switch ($type) {
            case "file":
                if (isset($_FILES["img"]) && !empty($_FILES["img"])) {
                    $this->load->helper('upload');
                    $config = array('allowed_types' => 'gif|jpg|jpeg|png');
                    $img_return = upload_file('img', SITE_PHYSICAL_PATH . TRASH_FOLDER, $config);
                    if (empty($img_return["error"]) && !empty($img_return["data"])) {
                        $sdata[$temp_data_index]["img"] = $img_return["data"]["file_name"];
                        $sdata[$temp_data_index]["img_url"] = SITE_VIRTUAL_PATH . TRASH_FOLDER . $img_return["data"]["file_name"];
                        $sdata[$temp_data_index]["img_path"] = SITE_PHYSICAL_PATH . TRASH_FOLDER . $img_return["data"]["file_name"];
                        $this->session->set_userdata($sdata);
                    }
                }
                break;
            case "data":
                $sdata[$temp_data_index]["position_hor"] = $this->input->post('position_hor', true);
                $sdata[$temp_data_index]["position_ver"] = $this->input->post('position_ver', true);
                $sdata[$temp_data_index]["alpha"] = intval($this->input->post('alpha', true));
                $sdata[$temp_data_index]["wm_type"] = $this->input->post('wm_type', true);
                $sdata[$temp_data_index]["font_size"] = intval($this->input->post('font_size', true));
                $sdata[$temp_data_index]["font_color"] = $this->input->post('font_color', true);
                $sdata[$temp_data_index]["font_face"] = $this->input->post('font_face', true);
                $sdata[$temp_data_index]["font_text"] = $this->input->post('font_text', true);
                $sdata[$temp_data_index]["shadow_color"] = $this->input->post('shadow_color', true);
                $sdata[$temp_data_index]["shadow_distance"] = intval($this->input->post('shadow_distance', true));
                $this->session->set_userdata($sdata);
                break;
        }
    }

    /**
     * Clear data of preview watermark
     *
     * @param string $temp_data_index user data index
     *
     * @return void
     */
    public function wm_clear_preview_data($temp_data_index = 'wm_preview')
    {
        unset($_SESSION[$temp_data_index]);
    }

}
