<?php

namespace Pg\Modules\Uploads\Models;

/**
 * Uploads module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Uploads install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Uploads
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Uploads_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        'admin_menu' => array(
            'action' => 'none',
            'name' => '',
            'items' => array(
                'settings_items' => array(
                    'action' => 'none',
                    'items' => array(
                        'uploads-items' => array(
                            'action' => 'none',
                            'items' => array(
                                'uploads_menu_item' => array('action' => 'create', 'link' => 'admin/uploads', 'status' => 1, 'sorter' => 1),
                            ),
                        ),
                    ),
                ),
            ),
        ),
        'admin_uploads_menu' => array(
            'action' => 'create',
            'name' => 'Admin mode - System - Uploads',
            'items' => array(
                'configs_list_item' => array('action' => 'create', 'link' => 'admin/uploads', 'status' => 1),
                'watermarks_list_item' => array('action' => 'create', 'link' => 'admin/uploads/watermarks', 'status' => 1),
            ),
        ),
    );

    /**
     * Constructor
     *
     * @return Uploads_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Validate module requirements
     *
     * @return array
     */
    public function _validate_requirements()
    {
        $result = array('data' => array(), 'result' => true);

        //check for GD library
        $is_gd_loaded = extension_loaded('gd');
        $result["data"][] = array(
            "name" => "GD library (works with graphics and images) is installed",
            "value" => $is_gd_loaded ? "Yes" : "No",
            "result" => $is_gd_loaded,
        );
        $result["result"] = $result["result"] && $is_gd_loaded;

        return $result;
    }

    /**
     * Install uploads data to menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import uploads languages to menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('uploads', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');

            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid,
                                              $langs_file);
        }

        return true;
    }

    /**
     * Export uploads languages from menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid,
                                                      $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall uploads data from menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]['items']);
            }
        }
    }

    /**
     * Install data of uploads module
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        $this->ci->load->model('uploads/models/Uploads_config_model');
        $data = array(
            "name" => "Text watermark",
            "gid" => "text-wm",
            "position_hor" => 'right',
            "position_ver" => 'bottom',
            "alpha" => '100',
            "wm_type" => "text",
            "img" => '',
            "font_size" => '30',
            "font_color" => '241424',
            "font_face" => 'avant',
            "font_text" => 'pilotgroup',
            "shadow_color" => 'f5f5f5',
            "shadow_distance" => '1',
        );
        $this->ci->Uploads_config_model->save_watermark(null, $data);

        $data = array(
            "name" => "Image watermark",
            "gid" => "image-wm",
            "position_hor" => 'center',
            "position_ver" => 'middle',
            "alpha" => '30',
            "wm_type" => "img",
            "img" => 'wm_image-wm.png',
            "font_size" => '16',
            "font_color" => 'ffffff',
            "font_face" => 'arial',
            "font_text" => 'Test string',
            "shadow_color" => '777777',
            "shadow_distance" => '3',
        );
        $this->ci->Uploads_config_model->save_watermark(null, $data);
    }

    /**
     * Uninstall data of uploads module
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        $this->ci->load->model('uploads/models/Uploads_config_model');

        $wm = $this->ci->Uploads_config_model->get_watermark_by_gid('text-wm');
        $this->ci->Uploads_config_model->delete_watermark($wm["id"]);

        $wm = $this->ci->Uploads_config_model->get_watermark_by_gid('image-wm');
        $this->ci->Uploads_config_model->delete_watermark($wm["id"]);
    }

}
