<?php

namespace Pg\Modules\Users\Models;

/**
 * Users module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

define("USERS_VISITORS_TABLE", DB_PREFIX . "users_profile_visitors");
define("USERS_VISITS_TABLE", DB_PREFIX . "users_profile_visits");

/**
 * Users visitor model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_visitor_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Link to database object
     *
     * @var object
     */
    protected $DB;

    /**
     * Visit object properties in data source
     *
     * @var array
     */
    protected $fields = array(
        "id",
        "id_user",
        "id_visitor",
        "last_visit_date",
        "visits_count",
    );

    /**
     * Constructor
     *
     * @return Users_visitor_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
        $this->DB = &$this->CI->db;
    }

    /**
     * Return visit object from data source by identifier
     *
     * @param integer $id visit identifier
     *
     * @return array/false
     */
    public function get_visitor_by_id($id)
    {
        $result = $this->DB->select(implode(", ", $this->fields))
                ->from(USERS_VISITORS_TABLE)
                ->where('id', $id)
                ->get()->result_array();
        if (empty($result)) {
            return false;
        }

        return $result[0];
    }

    /**
     * Return visit object from data source by user and visitor identifier
     *
     * @param integer $user_id    user identfier
     * @param integer $visitor_id visitor identifier
     *
     * @return array/false
     */
    public function get_visitor_by_visitor($user_id, $visitor_id)
    {
        $result = $this->DB->select(implode(", ", $this->fields))
                ->from(USERS_VISITORS_TABLE)
                ->where('id_user', $user_id)
                ->where('id_visitor', $visitor_id)
                ->get()->result_array();
        if (empty($result)) {
            return false;
        }

        return $result[0];
    }

    /**
     * Save visit object to data source
     *
     * @param integer $id   visit identifier
     * @param array   $data visit data
     *
     * @return integer
     */
    public function save_visitor($id, $data = array())
    {
        if (!isset($data['last_visit_date'])) {
            $data["last_visit_date"] = date("Y-m-d H:i:s");
        }
        if (!$id) {
            if (!isset($data['visits_count'])) {
                $data['visits_count'] = 1;
            }
            $this->DB->insert(USERS_VISITORS_TABLE, $data);
            $id = $this->DB->insert_id();
        } else {
            if (!isset($data['visits_count'])) {
                $this->DB->set("visits_count", "visits_count + 1", false);
            }
            $this->DB->where("id", $id);
            $this->DB->update(USERS_VISITORS_TABLE, $data);
        }

        return $id;
    }

    /**
     * Return criteria for used in search by fields
     *
     * @param array $filters filters data
     *
     * @return array
     */
    protected function _get_search_criteria($filters)
    {
        $params = array();

        $fields = array_flip($this->fields);
        foreach ($filters as $filter_name => $filter_data) {
            switch ($filter_name) {
                // By user
                case "user":
                    if (!$filter_data) {
                        continue;
                    }
                    $params = array_merge_recursive($params, array("where" => array(USERS_VISITORS_TABLE . ".id_user" => $filter_data)));
                    break;
                // By visitor
                case "visitor":
                    if (!$filter_data) {
                        continue;
                    }
                    $params = array_merge_recursive($params, array("where" => array(USERS_VISITORS_TABLE . ".id_visitor" => $filter_data)));
                    break;
                // By week
                case "week":
                    if (!$filter_data) {
                        continue;
                    }
                    $params = array_merge_recursive($params, array("where_sql" => array('(' . USERS_VISITORS_TABLE . ".last_visit_date >= DATE_SUB(CURDATE(), INTERVAL 1 WEEK))")));
                    break;
                // By month
                case "month":
                    if (!$filter_data) {
                        continue;
                    }
                    $params = array_merge_recursive($params, array("where_sql" => array('(' . USERS_VISITORS_TABLE . ".last_visit_date >= DATE_SUB(CURDATE(), INTERVAL 1 MONTH))")));
                    break;
                default:
                    if (isset($fields[$filter_name])) {
                        if (empty($filter_data)) {
                            break;
                        }
                        $params = array_merge_recursive($params, array("where_in" => array(USERS_VISITORS_TABLE . "." . $filter_name => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array("where" => array(USERS_VISITORS_TABLE . "." . $filter_name => $filter_data)));
                    }
                    break;
            }
        }

        return $params;
    }

    /**
     * Return visitors objects from data source as array
     *
     * @param integer $page     page of results
     * @param string  $limits   items per page
     * @param array   $order_by sorting data
     * @param array   $params   sql criteria
     *
     * @return array
     */
    protected function _get_visitors_list($page = null, $limits = null, $order_by = null, $params = array())
    {
        $this->DB->select(USERS_VISITORS_TABLE . "." . implode(", " . USERS_VISITORS_TABLE . ".", $this->fields));
        $this->DB->from(USERS_VISITORS_TABLE);

        if (isset($params["join"]) && is_array($params["join"]) && count($params["join"])) {
            foreach ($params["join"] as $join) {
                $this->DB->join($join["table"], $join["condition"], $join["type"]);
            }
        }

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->DB->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->DB->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->DB->where($value);
            }
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->DB->order_by($field . " " . $dir);
                }
            }
        } elseif ($order_by) {
            $this->DB->order_by($order_by);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->DB->limit($limits, $limits * ($page - 1));
        }

        $results = $this->DB->get()->result_array();

        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $r;
            }

            return $data;
        }

        return array();
    }

    /**
     * Return number of visitors objects in data source
     *
     * @param array $params sql criteria
     *
     * @return integer
     */
    protected function _get_visitors_count($params = null)
    {
        $this->DB->select("COUNT(*) AS cnt");
        $this->DB->from(USERS_VISITORS_TABLE);

        if (isset($params["join"]) && is_array($params["join"]) && count($params["join"])) {
            foreach ($params["join"] as $join) {
                $this->DB->join($join["table"], $join["condition"], $join["type"]);
            }
        }

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->DB->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->DB->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->DB->where($value);
            }
        }

        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return filtered visitors objects from data source as array
     *
     * @param array   $filters       filters data
     * @param integer $page          page of results
     * @param integer $items_on_page items per page
     * @param string  $order_by      sorting data
     *
     * @return array
     */
    public function get_visitors_list($filters = array(), $page = null, $items_on_page = null, $order_by = null)
    {
        $params = $this->_get_search_criteria($filters);

        return $this->_get_visitors_list($page, $items_on_page, $order_by, $params);
    }

    /**
     * Return number of filtered visitors objects in data source
     *
     * @param array $filters filters data
     *
     * @return array
     */
    public function get_visitors_count($filters = array())
    {
        $params = $this->_get_search_criteria($filters);

        return $this->_get_visitors_count($params);
    }

    /**
     * Validate visitor object for saving to data source
     *
     * @param integer $id   visitor identifier
     * @param array   $data visitor data
     *
     * @return array
     */
    public function validate_visitor($id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id'])) {
                unset($return['data']['id']);
            }
        }

        if (isset($data['id_user'])) {
            $return['data']['id_user'] = intval($data['id_user']);
        }

        if (isset($data['id_visitor'])) {
            $return['data']['id_visitor'] = intval($data['id_visitor']);
        }

        if (isset($data['last_visit_date'])) {
            $value = strtotime($data['last_visit_date']);
            if ($value > 0) {
                $return['data']['last_visit_date'] = date("Y-m-d", $value);
            }
        }

        if (isset($data['visit_counts'])) {
            $return['data']['visit_counts'] = intval($data['visit_counts']);
        }

        return $return;
    }

    /**
     * Check user profile viewed by new user
     *
     * @param integer $user_id user identifier
     *
     * @return void
     */
    public function check_views_count($user_id)
    {
        $views_array = $this->CI->session->userdata("views_users_array");
        if (!isset($views_array) || !is_array($views_array)) {
            $views_array = array();
        }
        if (in_array($user_id, $views_array)) {
            return;
        }
        $views_array[] = $user_id;
        $this->CI->session->set_userdata("views_users_array", $views_array);
        $this->save_views_count($user_id);
    }

    /**
     * Update views of user provile
     *
     * @param integer $user_id user identifier
     *
     * @return void
     */
    public function save_views_count($user_id)
    {
        if ($this->CI->session->userdata("auth_type") == "user") {
            $visitor_id = intval($this->CI->session->userdata("user_id"));
            if ($user_id != $visitor_id) {
                $this->CI->load->model('users/models/Users_visitor_model');
                $visitor = $this->CI->Users_visitor_model->get_visitor_by_visitor($user_id, $visitor_id);
                $save_data = array('id_user' => $user_id, 'id_visitor' => $visitor_id);
                $this->CI->Users_visitor_model->save_visitor($visitor['id'], $save_data);
            }
        } else {
            $visitor_id = 0;
        }

        //$this->DB->set("views", "views + 1", FALSE);
        //$this->DB->where("id", $user_id);
        //$this->DB->update(USERS_TABLE);

        $save_data = array(
            'id_user' => $user_id,
            'id_visitor' => $visitor_id,
            'date_add' => date('Y-m-d H:i:s'),
        );
        $this->DB->insert(USERS_VISITS_TABLE, $save_data);
    }

    /**
     * Return number views of users' profiles by last day
     *
     * @param array $users_ids users identifiers
     *
     * @return array
     */
    public function get_views_count($users_ids)
    {
        $this->DB->select('id_user, COUNT(*) AS count')
            ->from(USERS_VISITS_TABLE)
            ->where_in('id_user', $users_ids)
            ->group_by(array('id_user'));
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $result) {
                $data[$result['id_user']] = $result['count'];
            }

            return $data;
        }

        return array();
    }

    /**
     * Update number views of users' profiles by cron
     *
     * @return void
     */
    public function update_views_count_cron()
    {
        $date = date('Y-m-d', time()) . ' 00:00:00';

        $ids = array();
        $users = array();
        $this->DB->select('id, id_user, id_visitor')->from(USERS_VISITS_TABLE)->where('date_add <', $date);
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $result) {
                $ids[] = $result['id'];
                if (!isset($users[$result['id_user']])) {
                    $users[$result['id_user']] = 0;
                }
                ++$users[$result['id_user']];
            }

            $this->DB->where('date_add <', $date)->delete(USERS_VISITS_TABLE);

            $this->CI->load->model('Users_model');

            $this->CI->Users_model->set_format_settings('use_format', false);
            $users_data = $this->CI->Users_model->get_users_list(array('id' => array_keys($users)));
            $this->CI->Users_model->set_format_settings('use_format', true);

            foreach ($users_data as $user_data) {
                $save_data = array('views' => $user_data['views'] + $users[$user_data['id']]);
                $this->CI->Users_model->save_user($user_data['id'], $save_data);
            }
        }
    }

}
