<?php

namespace Pg\Modules\Widgets\Models;

/**
 * Widgets module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Widgets install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Widgets
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Widgets_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @param object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        'admin_menu' => array(
            'action' => 'none',
            'name' => '',
            'items' => array(
                'other_items' => array(
                    'action' => 'none',
                    'items' => array(
                        'add_ons_items' => array(
                            'action' => 'none',
                            'items' => array(
                                'widgets_menu_item' => array('action' => 'create', 'link' => 'admin/widgets', 'status' => 1, 'sorter' => 3),
                            ),
                        ),
                    ),
                ),
            ),
        ),
        'admin_widgets_menu' => array(
            'action' => 'create',
            'name' => 'Admin mode - Widgets',
            'items' => array(
                'widgets_installed_item' => array('action' => 'create', 'link' => 'admin/widgets/index', 'status' => 1, 'sorter' => 1),
                'widgets_enabled_item' => array('action' => 'create', 'link' => 'admin/widgets/install', 'status' => 1, 'sorter' => 2),
            ),
        ),
    );

    /**
     * Moderators configuration
     *
     * @var array
     */
    protected $moderators = array(
        array('module' => 'widgets', 'method' => 'index', 'is_default' => 1),
    );

    /**
     * Data depended on linked modules
     *
     * @var array
     */
    protected $lang_dm_data = array(
        array(
            "module" => "widgets",
            "model" => "Widgets_model",
            "method_add" => "lang_dedicate_module_callback_add",
            "method_delete" => "lang_dedicate_module_callback_delete",
        ),
    );

    /**
     * Constructor
     *
     * @return Widgets_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Install widgets data to menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import widgets languages to menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('widgets', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');

            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid,
                                              $langs_file);
        }

        return true;
    }

    /**
     * Export widgets languages from menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid,
                                                      $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall widgets data from menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]['items']);
            }
        }
    }

    /**
     * Install widgets data to moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        // install moderators permissions
        $this->ci->load->model('Moderators_model');

        foreach ($this->moderators as $method) {
            $this->ci->Moderators_model->save_method(null, $method);
        }
    }

    /**
     * Import widgets languages to moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read('widgets', 'moderators', $langs_ids);

        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'widgets';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method['method']])) {
                $this->ci->Moderators_model->save_method($method['id'], array(), $langs_file[$method['method']]);
            }
        }

        return true;
    }

    /**
     * Export widgets languages from moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'widgets';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method['method']] = $method['langs'];
        }

        return array('moderators' => $return);
    }

    /**
     * Uninstall widgets data from moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        // delete moderation methods in moderators
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'widgets';
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install data depended on linked modules
     *
     * @return void
     */
    public function _prepare_installing()
    {
        $this->ci->load->model("widgets/models/Widgets_model");
        foreach ($this->ci->pg_language->languages as $lang_id => $value) {
            $this->ci->Widgets_model->lang_dedicate_module_callback_add($lang_id);
        }
    }

    /**
     * Install data of widgets module
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        ///// add entries for lang data updates
        foreach ($this->lang_dm_data as $lang_dm_data) {
            $this->ci->pg_language->add_dedicate_modules_entry($lang_dm_data);
        }
    }

    /**
     * Uninstall data of widgets module
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        /// delete entries in dedicate modules
        foreach ($this->lang_dm_data as $lang_dm_data) {
            $this->ci->pg_language->delete_dedicate_modules_entry(array('where' => $lang_dm_data));
        }
    }

}
